/******************************************************************************/
/* exemple1.c                        20/03/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* 1. IMAGES EN ENTREE ET SORTIE                                              */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"


/*
 * Fonction ex1_copie appelee par l'operateur ExCopie
 *
 * Effet : realise une copie de l'image en entree.
 *
 * Appel : ex1_copie (ima1, ima2, xsiz, ysiz);
 * ou ima1 et ima2 sont des pointeurs sur les images.
 * Les dimensions des images sont xsiz*ysiz (ysiz lignes, xsiz colonnes).
 * Les points sont des Unt16 = des entiers-16 non signes (0..65535).
 * Les images sont en fait des vecteurs de Unt16 ; on accede a un point
 * de coordonnees x,y par ima[ y*xsiz + x ].
 *
*/
void ex1_copie (ima1, ima2, xsiz, ysiz)
Unt16 *ima1, *ima2;
int xsiz, ysiz;
{
    int x, y;

    /* Pour copier ima1 sur ima2, on balaie les 2 images ligne a ligne :
    for (y = 0; y < ysiz; y++)
    for (x = 0; x < xsiz; x++)
	ima2[y*xsiz+x] = ima1[y*xsiz+x];
    */

    /* ASTUCE : au lieu de calculer 2 fois y*xsiz+x comme ci-dessus
     * pour chaque point, ce qui est une perte de temps, on introduit
     * t = y*xsiz+x qui n'est calcule qu'une fois.
    */
    for (y = 0; y < ysiz; y++)
    for (x = 0; x < xsiz; x++) {
	int t = y*xsiz+x;
	ima2[t] = ima1[t];
    }
}


/*
 * Fonction ex1_sym_vert appelee par l'operateur ExSymVert
 *
 * Effet : realise une symetrie verticale de l'image en entree.
*/
void ex1_sym_vert (ima1, ima2, xsiz, ysiz)
Unt16 *ima1, *ima2;
int xsiz, ysiz;
{
    int x, y;

    /* Pour realiser la symetrie verticale, on balaie les 2 images
     * ligne a ligne, en copiant x,ysiz-1-y sur x,y.
    */
    for (y = 0; y < ysiz; y++)
    for (x = 0; x < xsiz; x++)
	ima2[(ysiz-1-y)*xsiz+x] = ima1[y*xsiz+x];

    /* RQ : on ne peut pas employer l'astuce de ex1_copie du calcul
     * unique de t = y*xsiz+x.
    */
}


/*
 * Fonction ex1_compar appelee par l'operateur ExCompar
 *
 * Effet : realise une comparaison des 2 images en entree.
 * Si le point x,y a la meme couleur dans ima1 et ima2, on le
 * met dans ima3 a 255, sinon a 0.
*/
void ex1_compar (ima1, ima2, ima3, xsiz, ysiz)
Unt16 *ima1, *ima2, *ima3;
int xsiz, ysiz;
{
    int x, y;

    /* Ici on peut utiliser t = y*xsiz+x pour optimiser.
    */
    for (y = 0; y < ysiz; y++)
    for (x = 0; x < xsiz; x++) {
	int t = y*xsiz+x;

	if (ima1[t] == ima2[t])
	     ima3[t] = 255;
	else ima3[t] = 0;
    }

    /* RQ on peut remplacer le if () .. else ..  par
     *
     * ima3[t] = (ima1[t] == ima2[t]) ? 255 : 0;
    */
}


/*
 * Fonction ex1_minmax appelee par l'operateur ExMinMax
 *
 * Effet : 
 *    ima3(x,y) = MIN { ima1(x,y) , ima2(x,y) }.
 *    ima4(x,y) = MAX { ima1(x,y) , ima2(x,y) }.
*/
void ex1_minmax (ima1, ima2, ima3, ima4, xsiz, ysiz)
Unt16 *ima1, *ima2, *ima3, *ima4;
int xsiz, ysiz;
{
    int x, y;

    for (y = 0; y < ysiz; y++)
    for (x = 0; x < xsiz; x++) {
	int t = y*xsiz+x;

	if (ima1[t] < ima2[t]) {
	     ima3[t] = ima1[t]; ima4[t] = ima2[t];
	} else {
	     ima3[t] = ima2[t]; ima4[t] = ima1[t];
        }
    }
}



