/******************************************************************************/
/* exemple6.c                        09/04/96 - Edouard.Thiel@lim.univ-mrs.fr */
/*                                                                            */
/* 6. COMPLEMENTS                                                             */
/******************************************************************************/

/* 
 * Le include suivant est OBLIGATOIRE dans tous vos .c 
*/ 
#include "../IpsDir/include/tel_oper.h"



/******************************************************************************
 * EFFET : A partir d'une image binaire, calcule une succession d'erosions
 *         avec INDICATEUR D'ETAT : CO_SPY_INT (&etape, debut, fin).
 *
 *****************************************************************************/
void ex6_erode_multi (ima1, ima2, ima3, nb, c, 
	cmin, xmax, ymax, xtot, ytot, xytot)
Unt16 *ima1, *ima2, *ima3;
int nb, c, cmin, xmax, ymax, xtot, ytot, xytot;
{
    int i;
 
    /* 
     * Entier de reference pour l'indicateur d'etat 
    */
    int etape = 0;

    /* 
     * On declenche le timer sur etape, qui va ici de 0 a 1+2*nb 
    */
    CO_SPY_INT (&etape, 0, 1+2*nb);

    /*
     * On erode nb fois
    */
    for (i = 0; i < nb; i++) {

	ex1_copie ((i == 0) ? ima1 : ima2, ima3, xtot, ytot);

	etape++;

	ex5a_erosion ( ima3, ima2, cmin, xmax, ymax, xtot, xytot, c);

	etape++;
    }

    /*
     * Le timer s'arretera tout seul lorsque l'on sortira de la section
    */
}


/******************************************************************************
 * EFFET : Calcule l'histogramme des niveaux de gris de ima1
 *         Ecrit cet histogramme dans fichier_data
 *
 *****************************************************************************/
void ex6_calc_histog (ima1, xytot, fichier_data)
Unt16 *ima1;
int xytot;
char *fichier_data;
{
    int t, v[256], i;
    FILE *f;

    /* init vecteur des niveaux de gris a 0 */
    for (i = 0; i < 256; i++) v[i] = 0;

    /* Parcours l'image et incremente v[] */
    for (t = 0; t < xytot; t++) if (ima1[t] < 256) v[ima1[t]]++;

    /* Ouverture de fichier_data */
    f = fopen (fichier_data, "w");
    if (f == NULL) {
	printf ("Can't write to file %s\n", fichier_data);
	return;
    }

    /* Ecriture dans fichier_data sous la forme "i v[i]" */
    fprintf (f, "# Fichier des niveaux de gris\n");
    for (i = 0; i < 256; i++) fprintf (f, "%d %d\n", i, v[i]);

    /* Fermeture de fichier_data */
    fclose (f);
}


/******************************************************************************
 * EFFET : Appel de gnuplot, qui execute le fichier de commandes f_commandes,
 *         dans lequel on demande de tracer le fichier de donnees f_data,
 *         en precisant le style de dessin et le le display.
 *
 *****************************************************************************/
void ex6_appel_gnuplot (f_data, f_commandes, display, appel_gnuplot, dessin)
char *f_data, *f_commandes, *display, *appel_gnuplot, *dessin;
{
    char appel_complet[200];
    FILE *f;

    /* On ouvre le fichier de commandes pour gnuplot */
    f = fopen (f_commandes, "w");
    if (f == NULL) {
	printf ("Can't write to file %s\n", f_commandes);
	return;
    }

    /* 
     * On ecrit les commandes pour gnuplot, avec des commentaires
    */
    fprintf (f, "# Commandes gnuplot pour le tracage de %s\n", f_data);
    fprintf (f, "\n# On demande d'afficher le resultat dans une fenetre X11\n");
    fprintf (f, "set term x11\n");
    fprintf (f, "# Si on voulait du postscript, on mettrait a la place\n");
    fprintf (f, "#     set term post eps 20\n");
    fprintf (f, "#     set ouput 'fichier.eps'\n");
    fprintf (f, "\n# Le titre\n");
    fprintf (f, "set title 'Histogramme des niveaux de gris - %s'\n", f_data);
    fprintf (f, "\n# On supprime la legende du coin en haut a droite\n");
    fprintf (f, "set nokey\n");
    fprintf (f, "\n# On demande un quadrillage\n");
    fprintf (f, "set grid\n");
    fprintf (f, "\n# L'intervalle en x\n");
    fprintf (f, "set xrange [0:259]\n");
    fprintf (f, "\n# Graduation en x : de 0 a 260 tous les 10\n");
    fprintf (f, "set xtics 0,10,260\n");
    fprintf (f, "\n# Trace le fichier avec le style de dessin\n");
    fprintf (f, "plot '%s' with %s\n", f_data, dessin);
    fprintf (f, "\n# Attend 120 secondes avant de quitter.\n");
    fprintf (f, "pause 120\n");
    fprintf (f, "# On peut attendre a la place un [return] avec :\n");
    fprintf (f, "#     pause -1\n");
    fprintf (f, "# Dans ce cas il faut avoir lance gnuplot dans un terminal\n");
    fprintf (f, "# interactif, par exemple avec :\n");
    fprintf (f, "#     xterm -e gnuplot fichier.gnu /dev/tty &\n\n");

    /* Fermeture du fichier de commandes */
    fclose (f);

    /* 
     * Appel de gnuplot. Par defaut, on a :
     * 
     *     appel_gnuplot = "gnuplot -display %s:0 %s &"
     *     display = "localhost"
     *     f_commandes = "histog.gnu"
     *
     * Le sprintf fera que
     * 
     *     appel_complet = "gnuplot -display localhost:0 histog.gnu &"
    */
    sprintf (appel_complet, appel_gnuplot, display, f_commandes);
    system (appel_complet);
}





