/***************************************************************************************/
/* icobox.c                                                   I.P.S  v 3.1  18/11/1993 */
/*                                                                                     */
/* gestion des icones                                    Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"

static Menu ico_menu_pane;

/***************************************************************************************
 *
 *                          PARTIE GESTION DES LISTES
 *                          =========================
 *
 * Initialisation generale
 *
 ***************************************************************************************/
void
ico_init ()
{
#ifdef DEBUG
  printf ("ico_init\n");
#endif

  glo.l_ico_z = NULL;
  glo.l_ico_a = NULL;

  glo.s_bouton    = 0;
  glo.prev_action = 0;

  glo.index_creat = 1;
  glo.index_selec = 1;

  glo.sx = 10;
  glo.sy = 16;

  ico_menu_pane = (Menu) xv_create (NULL, MENU,
      MENU_TITLE_ITEM,          "Ips Pane",
      MENU_STRINGS,            "Select All",  
                                NULL,
      MENU_ITEM,
          MENU_STRING,         "Position",
          MENU_PULLRIGHT,
              xv_create (NULL, MENU,
                  MENU_STRINGS,       "Horizontal", "Vertical", "Cascade", "Inside",
                                      NULL,
                  MENU_NOTIFY_PROC,   ico_menu_pane_proc,
                  NULL),
          NULL,
      MENU_STRINGS,             "Duplicate", "Refresh", "Clear",
                                NULL,
      MENU_NOTIFY_PROC,         ico_menu_pane_proc, 
      NULL); 

  method_menu_decla ();
}

/***************************************************************************************
 * Creation d'un Ico
 *
 ***************************************************************************************/
Ico *
ico_creat (x, y, nat_obj)
int x, y, nat_obj;
{
  Ico *clic;

  if ((clic = (Ico *) pcp_alloc (sizeof(Ico))) != NULL) {
      clic->xb   = x;
      clic->yb   = y;
      clic->xm   = 60;
      clic->ym   = 80;
      clic->pz   = NULL;
      clic->pa   = NULL;
      clic->sel  = FAUX;
      clic->lock = LOCK_NO;
      clic->index_creat = glo.index_creat++;
      ico_index_selec (clic);

      clic->nat_obj = nat_obj;
      method_init (clic);
  }

  return clic;
}

/***************************************************************************************
 * Insere Ico *clic en tete de la liste ->pz
 *
 ***************************************************************************************/
void
ico_tete_pz (clic)
Ico *clic;
{
  if (clic != NULL)
  {
    clic->pz = glo.l_ico_z;
    clic->pa = NULL;
    if (glo.l_ico_z != NULL) glo.l_ico_z->pa = clic;
                        else glo.l_ico_a     = clic;
    glo.l_ico_z = clic;
  }
}

/***************************************************************************************
 * Insere Ico *clic en tete de la liste ->pa
 *
 ***************************************************************************************/
void
ico_tete_pa (clic)
Ico *clic;
{
  if (clic != NULL)
  {
    clic->pa = glo.l_ico_a;
    clic->pz = NULL;
    if (glo.l_ico_a != NULL) glo.l_ico_a->pz = clic;
                        else glo.l_ico_z     = clic;
    glo.l_ico_a = clic;
  }
}

/***************************************************************************************
 * Supprime le maillon Ico *clic dans la liste chainee dans les deux sens
 *
 ***************************************************************************************/
void
ico_enlever_maillon (clic)
Ico *clic;
{
  if (clic != NULL)
  {
    if (clic->pa != NULL) clic->pa->pz = clic->pz;
                     else glo.l_ico_z  = clic->pz;
    if (clic->pz != NULL) clic->pz->pa = clic->pa;
                     else glo.l_ico_a  = clic->pa;
  }
}

/***************************************************************************************
 * Libere l'espace memoire alloue a Ico *clic
 *
 ***************************************************************************************/
void
ico_free (clic)
Ico *clic;
{
  if (clic != NULL)
  {
    method_free (clic);
    clic->index_creat = 0;
    pcp_free (clic);
  }
}

/***************************************************************************************
 * Duplique Ico *clic ainsi que son champs clic->obj
 *
 ***************************************************************************************/
Ico *
ico_duplic (clic)
Ico *clic;
{
  Ico *nouv = NULL;

  if (clic != NULL)
      if ((nouv = (Ico *) pcp_alloc (sizeof(Ico))) != NULL) {
          memcpy (nouv, clic, sizeof(Ico));
          nouv->index_creat = glo.index_creat++;
          nouv->lock = LOCK_NO;
          method_duplic (nouv, clic);
      }

  return nouv;
}

/***************************************************************************************
 * Recherche un ico a partir de son index_creat ; renvoie NULL sinon
 *
 ***************************************************************************************/
Ico *ico_get_from_index_creat (index_creat)
int index_creat;
{
    Ico *cur;

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
	if (cur->index_creat == index_creat)
	    return cur;
    return (Ico *) NULL;
}

/***************************************************************************************
 * Recherche quel icone est clique dans l'ordre z
 *
 ***************************************************************************************/
Ico *  
ico_find_clic (x, y)
int x, y;
{
  int  trouve = FAUX;
  Ico *cur;

  cur = glo.l_ico_z;
  while (cur != NULL && !trouve)
  {
    if (cur->xb <= x && x < cur->xb+cur->xm && cur->yb <= y && y < cur->yb+cur->ym)
         trouve = VRAI;
    else cur = cur->pz;
  }

  return cur;
}

/***************************************************************************************
 * Teste si Ico *p1 et *p2 s'intersectent
 *
 ***************************************************************************************/
int ico_inter (p1, p2)
Ico *p1, *p2;
{
    if (p1 == NULL || p2 == NULL) return FAUX;

    return ( p2->xb < p1->xb + p1->xm &&
             p1->xb < p2->xb + p2->xm &&
             p2->yb < p1->yb + p1->ym &&
             p1->yb < p2->yb + p2->ym );
}

/***************************************************************************************
 * Teste si Ico *p1 intersecte la zone xb, yb, xm, xb+xm, yb+ym
 *
 ***************************************************************************************/
int ico_inter_zone (p1, xb, yb, xm, ym)
Ico *p1;
int xb, yb, xm, ym;
{
    if (p1 == NULL) return FAUX;
 
    return ( xb < p1->xb + p1->xm &&
             p1->xb < xb + xm &&
             yb < p1->yb + p1->ym &&
             p1->yb < yb + ym );
}

/***************************************************************************************
 * Met a FAUX le champs cur->raf de tous les Ico *cur
 *
 ***************************************************************************************/
void ico_init_raf ()
{
    Ico *cur;

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
	cur->raf = FAUX;
}

/***************************************************************************************
 * Indique pour chaque Ico SOUS *cur si il intersecte Ico *clic
 *
 ***************************************************************************************/
void
ico_inter_raf (cur, clic)
Ico *cur, *clic;
{
  if (clic != NULL)
  {
    while (cur != NULL)
    {
      cur->raf = ico_inter (cur, clic);
      cur = cur->pz;
    }
  }
}

/***************************************************************************************
 * Crible sur les champs ->raf pour savoir quels Ico rafraichir
 *
 ***************************************************************************************/
void ico_inter_crible ()
{
    Ico *cur, *tmp;

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
	if (cur->raf)
	    for (tmp = cur->pa; tmp != NULL; tmp = tmp->pa)
		if (ico_inter (cur, tmp))
		    tmp->raf = VRAI;
}

/***************************************************************************************
 * Teste si Ico *clic est au premier plan
 *
 ***************************************************************************************/
int
ico_is_front (clic)
Ico *clic;
{
  int resultat = FAUX;


  return resultat;
}

/***************************************************************************************
 * Positionnement de Ico *clic :
 * --> changement de l'ordre en z et appel de l'affichage
 *
 * ENTREE : Ico *clic    : clique par la souris
 *          int  num_cas : 0 = dessus , 1 = dessous , 2 = supprimer , 3 = refresh
 ***************************************************************************************/
void
ico_posit_clic (clic, num_cas)
Ico *clic;
int  num_cas;
{
  if (clic != NULL)
  {
    /*****************************
     * PHASE 1 : initialisations *
     *****************************/
    ico_bascule_xor ();
    ico_init_raf ();
    switch (num_cas)
    {
      case 0 : /* dessus */
      case 3 : /* refresh : on allume juste clic */
        clic->raf = VRAI;
        break;

      case 1 : /* dessous : on allume les intersectes qui sont dessous clic */
        ico_inter_raf (clic->pz, clic);
        break;

      case 2 : /* supprimer : on allume tous les intersectes */
        ico_inter_raf (glo.l_ico_z, clic);
        break;
    }

    /***************************************
     * PHASE 2 : mise a jour listes z et a *
     ***************************************/
    switch (num_cas)
    {
      case 0 : /* dessus */
        ico_enlever_maillon (clic);
        ico_tete_pz (clic);
        break;

      case 1 : /* dessous */
        ico_enlever_maillon (clic);
        ico_tete_pa (clic);
        break;

      case 2 : /* supprimer */
        ico_affi_fond (clic);
        ico_enlever_maillon (clic);
        ico_free (clic);
        break;
    }

    /****************************************
     * PHASE 3 : rajout des elements > en z *
     ****************************************/
    switch (num_cas)
    {
      case 1 : /* dessous */
      case 2 : /* supprimer */
      case 3 : /* refresh */
        ico_inter_crible ();
        break;
    }

    /***********************
     * PHASE 4 : affichage *
     ***********************/
    ico_affi_raf ();
    ico_bascule_xor ();
  }
}

/***************************************************************************************
 * Affichage immediat de Ico *ico en Front
 *
 ***************************************************************************************/
void ico_refresh_now (ico)
Ico *ico;
{
    ico_posit_clic (ico, 0); /* Front */
    XFlush (glo.dpy);
}

/***************************************************************************************
 * Deplacement de Ico *clic : recherche des elements a rafraichir et affichage
 *
 ***************************************************************************************/
void
ico_depla_clic (clic, x, y)
Ico *clic;
int  x, y;
{
  if (clic != NULL)
  {
    /* PHASE 1 : initialisations */
    ico_inter_raf (glo.l_ico_z, clic);

    /* PHASE 2 deplacement */
    ico_affi_fond (clic);
    clic->xb = x;
    clic->yb = y;

    /* PHASE 3 : rajout des elements > en z */
    ico_inter_crible ();

    /* PHASE 4 : affichage */
    ico_affi_raf ();
  }
}

/***************************************************************************************
 * Homothetie de Ico *clic : recherche des elements a rafraichir et affichage
 * on en profite pour appeler ico_douane entre les 2 ico_bascule_xor.
 *
 * (xb, yb)    = coord de l'Ico clic
 * (o_xm,o_ym) = ancienne taille de clic
 * (n_xm,n_ym) = nouvelle taille de clic
 ***************************************************************************************/
void
ico_homot_clic (clic, xb, yb, o_xm, o_ym, n_xm, n_ym)
Ico *clic;
int  xb, yb, o_xm, o_ym, n_xm, n_ym;
{
  if (clic != NULL)
  {
    /* PHASE 1 : initialisations */
    clic->xm = o_xm; /* restaure anciennes tailles */
    clic->ym = o_ym;
    ico_bascule_xor ();
    ico_inter_raf (glo.l_ico_z, clic);

    /* PHASE 2 deplacement et nouvelle taille */
    ico_affi_fond (clic);
    clic->xb = xb;
    clic->yb = yb;
    clic->xm = n_xm;
    clic->ym = n_ym;
    ico_douane (clic);

    /* PHASE 3 : affichage */
    ico_inter_crible ();
    ico_affi_raf ();
    ico_bascule_xor ();
  }
}

/***************************************************************************************
 * Remet a l'inte'rieur les icones "perdus" 
 *
 ***************************************************************************************/
void ico_display_inside ()
{
    Ico *cur;
    int x, y;

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) {

	x = cur->xb; y = cur->yb;

	if (cur->xb < 0) x = 0;
	if (cur->yb < 0) y = 0;
	if (cur->xb+cur->xm > glo.canvas_xm) x = glo.canvas_xm - cur->xm;
	if (cur->yb+cur->ym > glo.canvas_ym) y = glo.canvas_ym - cur->ym;

	if (x != cur->xb || y != cur->yb) ico_depla_clic (cur, x, y);
    }
}

/***************************************************************************************
 * Empeche Ico de sortir de l'ecran
 *
 ***************************************************************************************/
void
ico_douane (clic)
Ico *clic;
{
  if (clic->xb < 0) clic->xb = 0;
  if (clic->yb < 0) clic->yb = 0;
  if (clic->xb + clic->xm > glo.canvas_xm) clic->xb = glo.canvas_xm - clic->xm;
  if (clic->yb + clic->ym > glo.canvas_ym) clic->yb = glo.canvas_ym - clic->ym;
}


/***************************************************************************************
 *
 *                               PARTIE SELECTIONS
 *                               =================
 *
 * IMPORTANT : chaque fois qu'un Ico *clic est selecte, faire ico_index_selec (clic);
 *
 * Incremente l'index de selection
 *
 ***************************************************************************************/
void
ico_index_selec (ico)
Ico *ico;
{
  ico->index_selec = glo.index_selec++;

#ifdef DEBUG
  printf ("ico_index_selec : creat = %d  selec = %d\n", ico->index_creat, ico->index_selec);
#endif
}

/***************************************************************************************
 * tri par insertion de Ico *ico dans la liste Ico **p, sur les index_selec
 *
 * p[a..b-1] = deja trie ; p[b] = emplacement libre
 ***************************************************************************************/
void
ico_tri_selec (ico, p, a, b)
Ico *ico, **p;
int  a, b;
{
  int i, j, c;

  i = a; c = VRAI;
  while (i < b && c)
  {
    if (p[i]->index_selec < ico->index_selec)
    i += 1;
    else c = FAUX;
  }
  if (i < b) for (j = b; j > i; j--) p[j] = p[j-1];
  p[i] = ico;
}

/***************************************************************************************
 * Deselecte tous les icones, selecte clic si != NULL, et calcule les raf
 *
 ***************************************************************************************/
void ico_unsel_sel_one (clic)
Ico *clic;
{
    Ico *cur;

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) {
	cur->raf = cur->sel; 
	cur->sel = FAUX; 
    }

    if (clic != NULL) { 
	clic->sel = VRAI; 
	clic->raf = VRAI; 
	ico_index_selec (clic);
	ico_enlever_maillon (clic);
	ico_tete_pz (clic);
    }
  
    ico_inter_crible ();
}

/***************************************************************************************
 * Ajoute a la selection tous les icones intersectant l'ensemble, et calcule les raf
 *
 ***************************************************************************************/
void ico_sel_ensemble (x1, y1, x2, y2)
int x1, y1, x2, y2;
{
    Ico *cur;
    int a;

    if (x1 > x2) { a = x1; x1 = x2; x2 = a; }
    if (y1 > y2) { a = y1; y1 = y2; y2 = a; }

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
	if (x1 <= cur->xb+cur->xm && cur->xb <= x2 &&
	    y1 <= cur->yb+cur->ym && cur->yb <= y2 ) {
	    cur->raf = cur->sel = VRAI;
	    ico_index_selec (cur);
	}

    ico_inter_crible ();
}

/***************************************************************************************
 * Affiche les cadres en xor de tous les selectes
 *
 ***************************************************************************************/
void ico_sel_drag_xor (dx, dy)
int dx, dy;
{
    Ico *cur;

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) 
	if (cur->sel) ico_affi_bord_xor (cur, dx, dy);
}

/***************************************************************************************
 * Scane les ->sel ,cherche les ico qu'ils intersectent, et leur met ->raf a VRAI
 *
 ***************************************************************************************/
void ico_sel_get_raf ()
{
    Ico *cur, *tmp;

    for (cur = glo.l_ico_z; cur != NULL; cur = cur->pz)
	if (cur->sel)
	    for (tmp = glo.l_ico_z; tmp != NULL; tmp = tmp->pz)
		if (ico_inter (cur, tmp))
		    tmp->raf = VRAI;
}

/***************************************************************************************
 * Renvoie le nombre de ->sel
 *
 ***************************************************************************************/
int ico_sel_get_nb ()
{
  Ico *cur;
  int  nb = 0;

    for (cur = glo.l_ico_z; cur != NULL; cur = cur->pz)
	if (cur->sel) nb++;

  return nb;
}

/***************************************************************************************
 * Deplace tous les selectes
 *
 ***************************************************************************************/
void ico_depla_sel (dx, dy)
int dx, dy;
{
    Ico *cur, *tmp;

    /* PHASE 1 : initialisations */
    ico_init_raf ();
    ico_sel_get_raf ();

    /* PHASE 2 deplacement */
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) 
	if (cur->sel) { 
	ico_affi_fond (cur);
	cur->xb += dx; 
	cur->yb += dy; 
    }

    /* PHASE 3 : refresh */
    ico_inter_crible ();
    ico_affi_raf ();
}

/***************************************************************************************
 * Duplique tous les Ico selectes, et selecte les dupliques
 *
 ***************************************************************************************/
void ico_duplic_sel ()
{
    Ico *cur, *dup, *deb;

    /* 1 : memorisation du premier ico de la liste dans l'ordre z */
    deb = glo.l_ico_z;

    /* 2 : Parcours la liste dans l'ordre a, duplique 
     *     et rajoute en tete pz (avec sel = FALSE )
    */
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
	if (cur->sel) {
	    if (cur->lock & LOCK_DUPLIC) {
		cur->sel = FAUX;
		cur->raf = VRAI;
	    } else {
		dup = ico_duplic (cur);
		dup->sel = FALSE;
		dup->xb += 10;
		dup->yb += 16;
		ico_tete_pz (dup);
		cur->sel = FAUX;
		cur->raf = VRAI;
	    }
	} else cur->raf = FAUX;

    /* 3 : Parcours la liste dans l'ordre a depuis deb, selecte et raf */
    if (deb != NULL) cur = deb->pa;
    while (cur != NULL) {
	cur->raf = cur->sel = VRAI;
	ico_index_selec (cur);
	cur = cur->pa;
    }

    /* 4 : refresh */
    ico_inter_crible ();
    ico_affi_raf ();
}

/***************************************************************************************
 * Demande a chaque Ico selecte si on doit le tuer
 *
 ***************************************************************************************/
void
ico_clear_sel ()
{
  Ico *cur, *suiv;
  int  result, x, y;
  int  dx = (int) xv_get (glo.canvas, XV_X);
  int  dy = (int) xv_get (glo.canvas, XV_Y);
  int  prompt = VRAI;

  cur = glo.l_ico_z;

  while (cur != NULL) { 
      suiv = cur->pz;

      if (cur->sel) {
          x = dx + cur->xb + cur->xm/2;
          y = dy + cur->yb + cur->ym/2;

          if ((cur->lock & LOCK_CLEAR) == 0) {
              if (prompt) {
                  result = notice_prompt(glo.panel1, NULL,   /* vol.7 p.273 */
                      NOTICE_FOCUS_XY,         x, y,
                      NOTICE_MESSAGE_STRINGS,  "Clear this icon ?", NULL,
                      NOTICE_BUTTON,           "ALL",   101,
                      NOTICE_BUTTON_YES,       "YES",
                      NOTICE_BUTTON,           "NO",    102,
                      NOTICE_BUTTON,           "STOP",  103,
                      NOTICE_NO_BEEPING,       TRUE,
                      NULL);

                  switch (result) {
                      case 101 : prompt = FAUX; ico_init_raf ();
                          break;
                      case NOTICE_YES : ico_posit_clic (cur, 2); /* clear */ 
                          break;
                      case 103 : return;
                  }
              }
              if (!prompt) {
                  Ico *hop = glo.l_ico_z;

                  while (hop != NULL) {  /* ico_inter_raf sans init des raf */
                      if (!hop->raf && ico_inter (hop, cur))
                          hop->raf = VRAI;
                      hop = hop->pz;
                  }
                  ico_affi_fond (cur);
                  ico_enlever_maillon (cur);
                  ico_free (cur);
              }
          }
      }
      cur = suiv;
  }

  if (!prompt) {
      ico_inter_crible ();
      ico_affi_raf ();
  }
}

/***************************************************************************************
 * Range la selection suivant how
 * 0 : cascade  - 1 : Horizontal - 2 : Vertical
 ***************************************************************************************/
void
ico_sel_display (how)
int how;
{
  Ico *cur, *tmp;
  int  test, x, y, dx, dy, lx, ly;

  /* PHASE initialisations */
  ico_init_raf ();
  ico_sel_get_raf ();

  /* PHASE cascade, tile */
  test = FAUX;
  for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
  if (cur->sel) {

       ico_affi_fond (cur);
       if (test)
            switch (how)
            {
              case 0 : /* cascade */
                x += 10; cur->xb = x;
                y += 16; cur->yb = y;
                break;
              case 1 : /* horizontal */
                x += 10 + dx; dx = cur->xm;
                if (x + dx + 10 - glo.sx < glo.canvas_xm )
                     { cur->xb = x; cur->yb = y; if (dy < cur->ym) dy = cur->ym; } 
                else { x = cur->xb = glo.sx; y += 10 + dy; dy = cur->ym; cur->yb = y; }
		if (x + dx > lx) lx = x + dx;
                break;
              case 2 : /* vertical */
                y += 10 + dy; dy = cur->ym;
                if (y + dy + 10 - glo.sy < glo.canvas_ym )
                     { cur->yb = y; cur->xb = x; if (dx < cur->xm) dx = cur->xm; } 
                else { y = cur->yb = glo.sy; x += 10 + dx; dx = cur->xm; cur->xb = x; }
		if (y + dy > ly) ly = y + dy;
                break;
            }
       else {
              test = VRAI; 
              x  = cur->xb = glo.sx;
              y  = cur->yb = glo.sy;
              dx = cur->xm; lx = x + dx;
              dy = cur->ym; ly = y + dy;
            }
  }

  /* PHASE : reborne horizontal, vertical */
  if (how == 1 && lx > glo.canvas_xm) {
    lx -= glo.canvas_xm -2;
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
      if (cur->sel) cur->xb -= lx;
  }
  else if (how == 2 && ly > glo.canvas_ym) {
    ly -= glo.canvas_ym -2;
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
      if (cur->sel) cur->yb -= ly;
  }

  /* PHASE : refresh */
  ico_inter_crible ();
  ico_affi_raf ();
}

/***************************************************************************************
 *
 *                               PARTIE AFFICHAGE
 *                               ================
 *
 * Affiche le fond de Ico *clic
 *
 ***************************************************************************************/
void
ico_affi_fond (clic)
Ico *clic;
{

  if (clic != NULL)
  {
    XSetForeground (glo.dpy, glo.gc, glo.c[0]);
    XFillRectangle (glo.dpy, glo.xwin, glo.gc, clic->xb, clic->yb, clic->xm, clic->ym);
  }
}

/***************************************************************************************
 * Affiche un rectangle en 2 couleurs b1 et b2
 *
 ***************************************************************************************/
void
ico_affi_rect_3d (xb, yb, xm, ym, b1, b2)
int xb, yb, xm, ym, b1, b2;
{
  int xc = xb+xm-1;
  int yc = yb+ym-1;

  XSetForeground (glo.dpy, glo.gc, b1);
  XDrawLine (glo.dpy, glo.xwin, glo.gc, xb, yb, xc-1, yb); 
  XDrawLine (glo.dpy, glo.xwin, glo.gc, xb, yb, xb, yc); 

  XSetForeground (glo.dpy, glo.gc, b2);
  XDrawLine (glo.dpy, glo.xwin, glo.gc, xb+1, yc, xc, yc); 
  XDrawLine (glo.dpy, glo.xwin, glo.gc, xc, yb, xc, yc); 
}

/***************************************************************************************
 * Affiche un rectangle en xor en coord xb, yb, xm, ym 
 *
 ***************************************************************************************/
void
ico_affi_rect_xor (xb, yb, xm, ym)
int xb, yb, xm, ym;
{
  XSetForeground (glo.dpy, glo.gc, 255 /*coul_16c[10]*/);
  XSetFunction (glo.dpy, glo.gc, GXxor);
  XDrawRectangle (glo.dpy, glo.xwin, glo.gc, xb, yb, xm-1, ym-1);
  XSetFunction (glo.dpy, glo.gc, GXcopy);
}

/***************************************************************************************
 * Affiche un rectangle en xor en coord x1, y1, x2, y2
 *
 ***************************************************************************************/
void
ico_affi_rec2_xor (x1, y1, x2, y2)
int x1, y1, x2, y2;
{
  int a;
  if (x1 > x2) { a = x1; x1 = x2; x2 = a; }
  if (y1 > y2) { a = y1; y1 = y2; y2 = a; }
  ico_affi_rect_xor (x1, y1, x2-x1+1, y2-y1+1);
}

/***************************************************************************************
 * Affiche le cadre en xor de Ico *cur a la position dx, dy
 *
 ***************************************************************************************/
void
ico_affi_bord_xor (cur, dx, dy)
Ico *cur;
int  dx, dy;
{
  ico_affi_rect_xor (cur->xb + dx, cur->yb + dy, cur->xm, cur->ym);
}

/***************************************************************************************
 * Annule ou retablit le xor en cours
 *
 * Pendant qu'un drag est fait en xor, il peut arriver des evenements qui
 * perturbent l'affichage du drag. Dans ce cas, il faut faire appel a ico_bascule_xor
 * au debut et a la fin de certaines fonctions.
 *
 * - Les fonctions concernees sont celles qui peuvent redessinent 1 ico pendant un
 *   drag : ico_posit_clic, ico_homot_clic.
 * - Les evenements pendant un drag peuvent modifier la selection ou la position des
 *   icones : changement des ->sel ; appel de ico_douane ; maj icone telOper et maj
 *   des resultats.
 * - Un dragNdrop a retardement doit commencer par annuler le drag courant.
 ***************************************************************************************/
void ico_bascule_xor ()
{
    if (glo.prev_action != 13) return;  /* 13 = bouton 1 tire */

    switch (glo.s_actb1) {

	case 1 : /* drag selection par ensemble */
	    ico_affi_rec2_xor (glo.x1, glo.y1, glo.x2, glo.y2);
	    break;

	case 2 : /* drag des icones selectes */
	    ico_sel_drag_xor (glo.x2, glo.y2);
	    break;

	case 3 : /* drag de l'icone */
	    ico_affi_bord_xor (glo.clic, glo.x2, glo.y2);
	    break;
    }
}

/***************************************************************************************
 * Affiche Ico *clic
 *
 ***************************************************************************************/
void
ico_affi_ico (clic)
Ico *clic;
{
  int a1, a2, a3;

  if (clic != NULL)
  {
    if (clic->sel)
         { a1 = glo.c[1]; a2 = glo.c[2]; a3 = glo.c[4]; }
    else { a1 = glo.c[4]; a2 = glo.c[3]; a3 = glo.c[1]; }
     
    XSetForeground (glo.dpy, glo.gc, a2);
    XFillRectangle (glo.dpy, glo.xwin, glo.gc, clic->xb+1, clic->yb+1, clic->xm-2, clic->ym-2);
    ico_affi_rect_3d (clic->xb, clic->yb, clic->xm, clic->ym, a1, a3);

    method_affi_ico (clic);
  }
}

/***************************************************************************************
 * Affiche dans l'ordre ->pa les Ico *cur si cur->raf
 *
 ***************************************************************************************/
void ico_affi_raf ()
{
    Ico *cur;

#ifdef DEBUG
    printf ("affi_raf");
#endif

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) { 
#ifdef DEBUG
	printf(" %d", cur->raf);
#endif
	if (cur->raf) ico_affi_ico (cur);
    }

#ifdef DEBUG
    printf ("\n");
#endif
}

/***************************************************************************************
 * Affichage dans le Canvas glo.canvas
 *
 ***************************************************************************************/
void
ico_repaint (canvas, p_win, dpy, xwin, area)
Canvas       canvas;
Xv_Window    p_win;
Display      *dpy;
Window       xwin;
Xv_xrectlist *area;
{
    Ico *cur;

#ifdef DEBUG
    printf("ico_repaint\n");
#endif

    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
	ico_affi_ico (cur);
}

/***************************************************************************************
 * Appel direct de ico_repaint
 *
 ***************************************************************************************/
void
ico_repaint_direct()
{
  ico_repaint ( glo.canvas, glo.p_win, glo.dpy, glo.xwin, (Xv_xrectlist*) NULL);
}

/***************************************************************************************
 * Changer la taille de la fenetre : zone drop_site
 *
 ***************************************************************************************/
void
ico_resize (canvas, width, height)
Canvas canvas;
int    width, height;
{
#ifdef DEBUG
  printf ("ico_resize %d %d\n", width, height);
#endif

  glo.canvas_xm = width; glo.canvas_ym = height;

  xv_set (glo.drop_site, 
      DROP_SITE_DELETE_REGION, NULL,
      DROP_SITE_REGION,        xv_get (canvas_paint_window (glo.canvas), WIN_RECT),
      NULL);

}

/***************************************************************************************
 *
 *                           PARTIE GESTION DES EVENEMENTS
 *                           =============================
 *
 * analyse de la souris : en fonction des evenements ACTION_SELECT, ACTION_ADJUST,
 * ACTION_MENU et LOC_DRAG, calcule l'etat des boutons (un seul actif a la fois)
 *
 * RETURN action = 10*bouton + etat  avec bouton = 1, 2, 3
 *                                 et etat = 1 enfonce ; 2 relache ; 3 tire ; 4 annule
 *               = 0 sinon
 ***************************************************************************************/
int
ico_analyse_souris (p_win, event)
Xv_Window p_win;
Event *event;
{
  int  etat     = 0;  /* 1 enfonce ; 2 relache ; 3 tire ; 4 annule */
  int  bout     = 0;  /* numero bouton souris actif */
  int  action   = 0;  /* action = bouton*10 + etat  */
  int  ev       = event_action(event);


  if ( ev == ACTION_SELECT ||
       ev == ACTION_ADJUST ||
       ev == ACTION_MENU   ||
       ev == LOC_DRAG       )
  {

    /********************************
     * Memorisation des coordonnees *
     ********************************/
    glo.sx = event_x(event);
    glo.sy = event_y(event);
 
    /***************************
     * Test du bouton en cours *
     ***************************/
    switch (ev)
    {
      case ACTION_SELECT : bout = 1; break;
      case ACTION_ADJUST : bout = 2; break;
      case ACTION_MENU   : bout = 3; break;
    }
    if (bout != 0)
    {
      if (glo.s_bouton == 0 && event_is_down(event))
           { /* un nouveau bouton est presse ; on initialise */
             glo.s_bouge  = FAUX;
             glo.s_bouton = bout;
           }
      else if (glo.s_bouton != bout)
           { /* annuler le bouton en cours */
             etat = 4;
           }
    }

    /***************************
     * determination de `etat' *
     ***************************/
    if (etat == 0)
    switch (ev)
    {
      case ACTION_SELECT :
      case ACTION_ADJUST :
      case ACTION_MENU   :
        if (event_is_down(event))
             etat = 1;  /* enfonce */
        else etat = 2;  /* relache */
        break;

      case LOC_DRAG :
        glo.s_bouge = VRAI;
        if (0 <= glo.sx && glo.sx < glo.canvas_xm &&
            0 <= glo.sy && glo.sy < glo.canvas_ym)
             etat  = 3;  /* tiree */
        else etat  = 4;  /* annule */
        break;
    }

    /********************************************
     * calcule action et cherche l'icone pointe *
     ********************************************/
    if (glo.s_bouton != 0)
    {
      action = glo.s_bouton*10 + etat;
      if (etat == 1) glo.clic = ico_find_clic (glo.sx, glo.sy);
    }

    /************
     * Controle *
     ************/
#ifdef DEBUG
    printf("bouton %d ",glo.s_bouton);
    switch (etat) {
      case 1 : printf ("enfonce"); break;
      case 2 : printf ("relache"); if (glo.s_bouge) printf (" bouge"); break;
      case 3 : printf ("tire"); break;
      case 4 : printf ("annule"); break; 
    }
    printf ("\n");
#endif

    /******************************
     * Annuler le bouton en cours *
     ******************************/
    if (etat   ==  2   /* relache */
     || etat   ==  4   /* annule  */
     || action == 31)  /* menu    */
    {
      glo.s_bouton = 0;
#ifdef DEBUG
      printf ("aucun bouton en cours\n");
#endif
    }
  }

  return action;
}

/***************************************************************************************
 * traitement des evenements dans le canvas principal
 *
 ***************************************************************************************/
void
ico_event_proc (p_win,event)
Xv_Window p_win;
Event     *event;
{
  int   action = 0;

  /*********************************
   * calcul de l'action a executer *
   *********************************/
  switch (event_action(event))
  {
    case ACTION_DRAG_COPY :    /* vol.7 p.446 */
    case ACTION_DRAG_MOVE :
      action = 5;
      break;
/*
    case ACTION_DRAG_PREVIEW :
      switch (event_id (event))
      {
        case LOC_WINENTER : printf ("PREVIEW WINENTER\n"); break;
        case LOC_WINEXIT  : printf ("PREVIEW WINEXIT\n"); break;
        case LOC_DRAG     : printf ("PREVIEW DRAG\n"); break;
        default           : printf ("PREVIEW default\n"); break;
      }
    break;
*/
    default :
      action = ico_analyse_souris (p_win, event);
      break;
  }

  /************************
   * executer la commande *
   ************************/
   switch (action)
   {
     case 11 :  /* bouton 1 enfonce */
       if (glo.clic == NULL)
            glo.s_actb1 = 1;  /* sur aucun icone */
       else if (glo.clic->sel)
            glo.s_actb1 = 2;  /* sur la selection */
       else glo.s_actb1 = 3;  /* sur icone non selecte */

       glo.x1 = glo.sx;
       glo.y1 = glo.sy;

       glo.clic_time_prev = glo.clic_time;
       glo.clic_time = (double) event->ie_time.tv_sec
                     + (double) event->ie_time.tv_usec / (double) 1E6;
       break;

     case 12 :  /* bouton 1 relache */
       if (glo.prev_action == 11) switch (glo.s_actb1)  /* pas bouge */
       {
         case 3 : /* rajoute cet icone a la selection */
           glo.clic->sel = VRAI;
           ico_index_selec (glo.clic);
           ico_posit_clic (glo.clic, 3); /* refresh */
	   break;
         case 1 : /* deselection generale */
           ico_unsel_sel_one (glo.clic);
           ico_affi_raf      ();
           break;
         case 2 : /* test double clic */
           if ( glo.clic_time != 0 && glo.clic_time_prev != 0
             && glo.clic_time - glo.clic_time_prev <= glo.clic_timeout )
           method_menu_dbl_clic (glo.clic);
           break;
       }
       else switch (glo.s_actb1)  /* on a bouge */
       {
         case 1 : /* drag selection par ensemble */
           ico_affi_rec2_xor (glo.x1, glo.y1, glo.x2, glo.y2);
           ico_sel_ensemble  (glo.x1, glo.y1, glo.x2, glo.y2);
           ico_affi_raf      ();
           break;

         case 2 : /* drag des icones selectes */
           ico_sel_drag_xor (glo.x2, glo.y2);
           ico_depla_sel    (glo.x2, glo.y2);
           break;

         case 3 : /* drag de l'icone */
           ico_affi_bord_xor (glo.clic, glo.x2, glo.y2);
           ico_depla_clic    (glo.clic, glo.clic->xb+glo.x2, glo.clic->yb+glo.y2);
           break;
       }
       break;

     case 13 :  /* bouton 1 tire */
       if (glo.prev_action == 11 && abs(glo.x1-glo.sx) + abs(glo.y1-glo.sy) < 10)
            action = 0;
       else switch (glo.s_actb1)
       {
         case 1 : /* drag selection par ensemble */
           if (glo.prev_action == 11)
                { 
                  ico_unsel_sel_one (glo.clic); 
                  ico_affi_raf (); 
                }
           else ico_affi_rec2_xor (glo.x1, glo.y1, glo.x2, glo.y2);
           glo.x2 = glo.sx; 
           glo.y2 = glo.sy;
           ico_affi_rec2_xor (glo.x1, glo.y1, glo.x2, glo.y2);
           break;

         case 2 : /* drag des icones selectes */
           if (glo.prev_action != 11) 
	       ico_sel_drag_xor (glo.x2, glo.y2);
           glo.x2 = glo.sx - glo.x1;
           glo.y2 = glo.sy - glo.y1;
           ico_sel_drag_xor (glo.x2, glo.y2);
           break;

         case 3 : /* drag de l'icone */
           if (glo.prev_action != 11) 
	       ico_affi_bord_xor (glo.clic, glo.x2, glo.y2);
           glo.x2 = glo.sx - glo.x1;
           glo.y2 = glo.sy - glo.y1;
           ico_affi_bord_xor (glo.clic, glo.x2, glo.y2);
           break;
       }
       break;

     case 14 :  /* bouton 1 annuler */
       if (glo.prev_action != 11) switch (glo.s_actb1)
       {
         case 1 : /* annule selection par ensemble */
           ico_affi_rec2_xor (glo.x1, glo.y1, glo.x2, glo.y2);
           break;

         case 2 : /* fin drag des icones selectes, essai exportation */
           ico_sel_drag_xor (glo.x2, glo.y2);
           XFlush (glo.dpy);
	   glo.prev_action = action;      /* car sd_sel_exporte peut faire reentrer */
           sd_sel_exporte ((Ico *) NULL); /*     dans ico_event_proc */
           break;

         case 3 : /* fin drag de l'icone, essai exportation */
           ico_affi_bord_xor (glo.clic, glo.x2, glo.y2);
           XFlush (glo.dpy);
	   glo.prev_action = action;
           sd_sel_exporte (glo.clic);
           break;
       }
       break;

     case 22 :  /* bouton 2 relache */
       if (glo.clic != NULL) 
       { 
         glo.clic->sel = !glo.clic->sel;   /* selection on/off de glo.clic */
         if (glo.clic->sel) ico_index_selec (glo.clic);
         ico_posit_clic (glo.clic, 3); /* refresh */
       }
       break;

     case 31 :  /* bouton 3 enfonce */
       if (glo.clic == NULL)
              ico_menu_affi (ico_menu_pane, event);
       else { 
              method_menu_contextuel (glo.clic);
#             ifdef DEBUG
              printf ("index creat = %d  selec = %d\n", 
                      glo.clic->index_creat, glo.clic->index_selec);
#             endif
              xv_set (glo.clic->menu, MENU_CLIENT_DATA, glo.clic, NULL);
              ico_menu_affi (glo.clic->menu, event);
            }
       break;

     case 5 :  /* ACTION_DRAG_COPY */
	ico_bascule_xor ();
	glo.sx = event_x(event);
	glo.sy = event_y(event);
	sd_req_importe (event);
	break;
   }

  /* memo action */
  if (action != 0) glo.prev_action = action;
}

/***************************************************************************************
 *
 *                             PARTIE GESTION DES MENUS
 *                             ========================
 *
 * Afficher un menu apres un ACTION_MENU dans glo.canvas
 *
 * Probleme : il faut normalement transmettre p_win a menu_show (vol.7 p.245)
 *            mais alors le menu affiche herite ses couleurs de p_win
 * Solution : on fait croire a menu_show que l'event c'est produit dans
 *            le frame principal (structure de Event vol.7 p.116)
 *
 ***************************************************************************************/
void
ico_menu_affi (menu, event)
Menu menu;
Event *event;
{
  event->ie_win = glo.frame_principal;
  event->ie_locx += (int) xv_get (glo.canvas, XV_X);
  event->ie_locy += (int) xv_get (glo.canvas, XV_Y);
  menu_show (menu, event->ie_win, event, NULL);
}

/***************************************************************************************
 * Menu Pane, qui s'affiche sur le plan de travail
 *
 ***************************************************************************************/
void
ico_menu_pane_proc (menu, menu_item)
Menu menu;
Menu_item menu_item;
{ 
  char *choice = (char *) xv_get (menu_item, MENU_STRING);
  Ico  *clic, *cur;
  int   cas;

#ifdef DEBUG
  printf ("ico_menu_pane_proc %s\n",choice);
#endif

  if      (!strcmp (choice, "Duplicate"  )) cas = 1;
  else if (!strcmp (choice, "Clear"      )) cas = 2;
  else if (!strcmp (choice, "Select All" )) cas = 3;
  else if (!strcmp (choice, "Refresh"    )) cas = 4;
  else if (!strcmp (choice, "Cascade"    )) cas = 5;
  else if (!strcmp (choice, "Horizontal" )) cas = 6;
  else if (!strcmp (choice, "Vertical"   )) cas = 7;
  else if (!strcmp (choice, "Inside"     )) cas = 8;
  else cas = -1;

  switch (cas)
  {
    case 1 : /* Duplicate */
        ico_duplic_sel ();
        break;

    case 2 : /* Clear */
        ico_clear_sel ();
        break;

    case 3 : /* Select All */
	for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) {
	    cur->raf = cur->sel = VRAI;
	    ico_index_selec (cur);
        }
        ico_affi_raf ();
        break;

    case 4 : /* Refresh */
	for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) 
	    method_obsolete (cur);
	ico_repaint_direct();
        break;

    case 5 : 
    case 6 : 
    case 7 : /* Cascade, Horizontal, Vertical */
        ico_sel_display (cas-5);
        break;

    case 8 : /* inside */
	ico_display_inside ();
	break;
  }
}

/***************************************************************************************
 *
 *                                PARTIE FOURRE-TOUT
 *                                ==================
 *
 ***************************************************************************************
 * erreur = ico_sel_scan1 (v_ico, v_max, nature, nom_nature, nombre) :
 *
 * - Parcourt les icones selectionnes et les trie dans v_ico
 * - teste si *nombre <= v_max
 * - teste si tous les icos ont la meme nature
 ***************************************************************************************/
int ico_sel_scan1 (v_ico, v_max, nature, nom_nature, nombre)
Ico  **v_ico;
int    v_max, nature;
char  *nom_nature;
int   *nombre;
{
    Ico *cur;
    int  i;

    for (i = 0; i < v_max; i++) v_ico[i] = NULL;
    *nombre = 0;
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
    if (cur->sel) { 
	if (cur->nat_obj != nature) {
	    pcp_avertissement ("WARNING ! Only",nom_nature,"must be selected");
	    return FAUX;
	}
	if (*nombre >= v_max) {
	    char bla[256];
	    sprintf (bla, "more than %d icons", v_max);
	    pcp_avertissement ("WARNING !",bla,"were selected");
	    return FAUX;
	}
	ico_tri_selec (cur, v_ico, 0, *nombre);  /* tri par insertion */
	(*nombre)++;
    }
    return VRAI;
}

/***************************************************************************************
 * erreur = ico_sel_scan2 (&ico, nature, nom_nature) :
 *
 * - Parcourt les icones et cherche si un seul ico selectionne
 * - teste si ico a la bonne nature
 ***************************************************************************************/
int ico_sel_scan2 (ico, nature, nom_nature)
Ico  **ico;
int    nature;
char  *nom_nature;
{
    Ico *cur;
    int trouve = FAUX;

    ico = NULL;
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
    if (cur->sel) {
	if (cur->nat_obj != nature) {
	    pcp_avertissement ("WARNING ! Only",nom_nature,"must be selected");
	    return FAUX;
	}
	if (trouve) {
	    pcp_avertissement ("WARNING ! Only one",nom_nature,"must be selected");
	    return FAUX;
	}
	trouve = VRAI;
	*ico = cur;
    }
    return trouve;
}

/***************************************************************************************
 * erreur = ico_sel_scan3 (clic, v_ico, v_max, nombre) :
 *
 * - Appele depuis menu contextuel de l'ico clic
 * - Parcourt les icones selectionnes et les trie dans v_ico ,sauf v_ico[0] = clic.
 * - teste si *nombre <= v_max
 ***************************************************************************************/
int ico_sel_scan3 (clic, v_ico, v_max, nombre)
Ico   *clic, **v_ico;
int    v_max, *nombre;
{
    Ico *cur;
    int  i;

    if (clic == NULL) return FAUX;

    for (i = 1; i < v_max; i++) v_ico[i] = NULL;
    v_ico[0] = clic; *nombre = 1;
    for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa)
    if (cur->sel && cur != clic) { 
	if (*nombre >= v_max) {
	    char bla[256];
	    sprintf (bla, "more than %d icons", v_max);
	    pcp_avertissement ("WARNING !",bla,"were selected");
	    return FAUX;
	}
	ico_tri_selec (cur, v_ico, 1, *nombre);  /* tri par insertion */
	(*nombre)++;
    }
    return VRAI;
}

