/***************************************************************************************/
/* messa.c                                                    I.P.S  v 3.3   17/3/1994 */
/*                                                                                     */
/* gestion des messages                                  Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include "inclus.h"

static int		messa_index = 1,
			messa_client_n = 0,
			messa_reentrance = 0;

static Messa_client	*messa_client_list = NULL;


/****************************************************************************************
 * FUNCTION : messa_NewClient
 *
 * Ajoute ce client dans la liste messa_client_list, et renvoie handle.
 *
 ***************************************************************************************/
int messa_NewClient (messaProc, client_data, ico_ind_v, ico_ind_n)
void *messaProc;
void *client_data;
int  *ico_ind_v, ico_ind_n;
{
    Messa_client *v;
    int i;

    if (messa_client_list == NULL)
	 v = (Messa_client*) malloc (sizeof(Messa_client));
    else v = (Messa_client*) realloc (messa_client_list,
	(messa_client_n+1) * sizeof(Messa_client));

    if (v == NULL) {
	pcp_error ("messa_NewClient", "Realloc error", NULL);
	return 0;
    }

    messa_client_list = v;
    v[messa_client_n].messaProc = messaProc;
    v[messa_client_n].client_data = client_data;
    v[messa_client_n].index_creat = ++messa_index;
    v[messa_client_n].ico_ind_n = ico_ind_n;
    for (i = 0; i < ico_ind_n; i++)
	v[messa_client_n].ico_ind_v[i] = ico_ind_v[i];
    messa_client_n++;

#   ifdef DEBUG
    printf ("messa_NewClient %d	nb %d\n", messa_index, messa_client_n);
    for (i = 0; i < ico_ind_n; i++)
	printf ("  ico_ind_v[%d] = %d\n", i, ico_ind_v[i]);
    for (i = 0; i < messa_client_n; i++)
	printf ("Client %d en %d\n", v[i].index_creat, i);
#   endif

    return messa_index;
}

/****************************************************************************************
 * FUNCTION : messa_DeleteClient
 *
 * Retire le client ayant cet index de la liste.
 * Attention, il faut que la win aie memorise l'index au prealable.
 *
 ***************************************************************************************/
void messa_DeleteClient (index)
int index;
{
    int i, k = -1;

    for (i = 0; i < messa_client_n; i++)
	if (messa_client_list[i].index_creat == index)
	    { k = i; break; }

#   ifdef DEBUG
    if (k == -1)
         printf ("messa_DeleteClient %d pas trouve\n", index);
    else printf ("messa_DeleteClient %d	nb %d\n", index, messa_client_n-1);
    for (i = 0; i < messa_client_n; i++)
	printf ("Client %d en %d\n", messa_client_list[i].index_creat, i);
#   endif

    /* Client pas trouve */
    if (k == -1) return;

    /* Un client de moins */
    messa_client_n--;

    /* Recopie de l'apres-dernier dans k */
    if (k < messa_client_n) 
	memcpy (messa_client_list + k,
		messa_client_list + messa_client_n, 
		sizeof (Messa_client));

    /* Libere memoire */
    if (messa_client_n == 0) {
	free (messa_client_list);
	messa_client_list = NULL;
    }
    /* BUG FIXED 16.3.95 : realloc peut aussi deplacer les donnees quand mem diminue */
    else messa_client_list = (Messa_client*) realloc (messa_client_list, 
	messa_client_n * sizeof (Messa_client));

#   ifdef DEBUG
    printf ("Liste des clients apres le realloc :\n");
    for (i = 0; i < messa_client_n; i++)
	printf ("Client %d en %d\n", messa_client_list[i].index_creat, i);
#   endif
}

/****************************************************************************************
 * FUNCTION : messa_ChangeIcoClient
 *
 * - Cherche le client ayant cet index dans la liste
 * - supprime old_ico et new_ico
 * - rajoute new_ico
 *
 * Si old_ico == 0, pas de suppression. Si new_ico == 0, pas d'insertion.
 ***************************************************************************************/

void messa_ChangeIcoClient (index, old_ico, new_ico)
int index, old_ico, new_ico;
{
    int i, k = -1, *v;

    for (i = 0; i < messa_client_n; i++)
	if (messa_client_list[i].index_creat == index)
	    { k = i; break; }
    v = messa_client_list[k].ico_ind_v;

#   ifdef DEBUG
    if (k == -1)
         printf ("messa_ChangeIcoClient %d pas trouve\n", index);
    else printf ("messa_ChangeIcoClient %d	nb %d\n", index, messa_client_n-1);
    for (i = 0; i < messa_client_n; i++)
	printf ("Client %d en %d\n", messa_client_list[i].index_creat, i);
#   endif

    /* Client pas trouve */
    if (k == -1) return;

#   ifdef DEBUG
    printf ("Liste des icones avant messa_ChangeIco_Client\n");
    for (i = 0; i <  messa_client_list[k].ico_ind_n; i++)
	printf ("  ico_ind_v[%d] = %d\n", i, v[i]);
    printf ("\n");
#   endif

    /* Supprime new_ico */
    if (new_ico > 0)
    for (i = 0; i < messa_client_list[k].ico_ind_n; i++)
    if (v[i] == new_ico) { v[i] = v[--(messa_client_list[k].ico_ind_n)]; break; }

    /* Supprime old_ico */
    if (old_ico > 0)
    for (i = 0; i < messa_client_list[k].ico_ind_n; i++)
    if (v[i] == old_ico) { v[i] = v[--(messa_client_list[k].ico_ind_n)]; break; }

    /* Ajoute new_ico */
    if (new_ico > 0)
    v[(messa_client_list[k].ico_ind_n)++] = new_ico;

#   ifdef DEBUG
    printf ("Liste des icones apres messa_ChangeIco_Client\n");
    for (i = 0; i <  messa_client_list[k].ico_ind_n; i++)
	printf ("  ico_ind_v[%d] = %d\n", i, v[i]);
    printf ("\n");
#   endif

}

/****************************************************************************************
 * FUNCTION : messa_Send
 *
 * 
 *
 ***************************************************************************************/
void messa_Send (int ordre, ...)
{
    va_list  ap;
    Messa_ev message;
    int      i, j, k, ico_ind_v[MESSA_MAX_ICO_N], ico_ind_n = 0;

    /*
     * Init message
    */
    message.from_who	 = 0;
    message.from_index	 = 0;
    message.ico_ind_n	 = 0;
    message.b_refresh	 = FAUX;
    message.b_kill	 = FAUX;
    message.b_done	 = FAUX;
    message.b_rescan	 = FAUX;
    message.b_lut	 = FAUX;
    message.b_name	 = FAUX;
    message.b_redraw_rect= FAUX;

    /*
     * Lecture arguments
    */
    for (va_start (ap, ordre); ordre != 0; ordre = va_arg (ap, int))
    switch (ordre) {  

	case MESSA_FROM_ICO :
	    message.from_who   = MESSA_FROM_ICO_ID;
	    message.from_index = (va_arg (ap, Ico *))->index_creat;
	    break;

	case MESSA_FROM_ICO_ID :
	case MESSA_FROM_WIN_ID :
	    message.from_who   = ordre;
	    message.from_index = va_arg (ap, int);
	    break;

	case MESSA_TO_ICO :
	    ico_ind_n    = 1;
	    ico_ind_v[0] = (va_arg (ap, Ico *))->index_creat;
	    break;

	case MESSA_TO_ICO_ID :
	    ico_ind_n    = 1;
	    ico_ind_v[0] = va_arg (ap, int);
	    break;

	case MESSA_TO_ICO_ID_LIST : 
	    while ((ico_ind_v[ico_ind_n] = va_arg (ap, int)) != 0) ico_ind_n++;
	    break;

	case MESSA_REFRESH : message.b_refresh = va_arg (ap, int); break;
	case MESSA_KILL    : message.b_kill    = va_arg (ap, int); break;
	case MESSA_DONE    : message.b_done    = va_arg (ap, int); break;
	case MESSA_RESCAN  : message.b_rescan  = va_arg (ap, int); break;
	case MESSA_LUT     : message.b_lut     = va_arg (ap, int); break;
	case MESSA_NAME    : message.b_name    = va_arg (ap, int); break;

	case MESSA_REDRAW_RECT :
	    message.b_redraw_rect = VRAI;
	    message.xb = va_arg (ap, int);
	    message.yb = va_arg (ap, int);
	    message.xm = va_arg (ap, int);
	    message.ym = va_arg (ap, int);
	    break;
    }

    /*
     * Fin lecture args
    */
    va_end (ap);

    /*
     * Test reentrance
    */
    if (messa_reentrance > 0) {
	printf ("ACHTUNG : tentative de reentrance dans messa_Send !!\n");
	return;
    }
    messa_reentrance++;

    /*
     * Envoi message aux clients
     *
     * Attention : les _messaProc pouvant appeler messa_DeleteClient,
     * l'ordre des clients dans messa_client_list peut etre bouleverse !
     * => Il faut parcourir la liste dans l'ordre decroissant.
     * (autre sol plus lourde : sauver l'environnement)
    */
    i = messa_client_n;
    while (i > 0 && messa_client_n > 0) {
	if (i > messa_client_n) i = messa_client_n;
	i--;

	/* si client == sender on passe au client suivant */
	if (message.from_who == MESSA_FROM_WIN_ID
	 && messa_client_list[i].index_creat == message.from_index)
	    continue;

	/* on envoie uniquement les icos concernes */
	message.ico_ind_n = 0;
	for (j = 0; j < messa_client_list[i].ico_ind_n; j++)
	for (k = 0; k < ico_ind_n; k++)
	if (messa_client_list[i].ico_ind_v[j] == ico_ind_v[k])
	    message.ico_ind_v[message.ico_ind_n++] = ico_ind_v[k];

	/* Appel des _messaProc */
	if (message.ico_ind_n > 0)
	    (*messa_client_list[i].messaProc)
		( messa_client_list[i].client_data, &message );
    }

    /*
     * Envoi message aux icos
    */
    for (k = 0; k < ico_ind_n; k++) {

 	Ico *cur = ico_get_from_index_creat (ico_ind_v[k]);
	if (cur != (Ico *) NULL) {

	    /* si ico == sender on passe a l'ico suivant */
	    if (message.from_who == MESSA_FROM_ICO_ID
	     && cur->index_creat == message.from_index)
		continue;

	    /* Appel de method_messaProcIco */
	    method_messaProcIco (cur, &message);
	}
    }

    messa_reentrance--;
}


/****************************************************************************************
 * FUNCTION : messa_ForIco
 *
 * 
 *
 ***************************************************************************************/
int messa_ForIco (ico, message)
Ico *ico;
Messa_ev *message;
{
    if (ico == NULL) return FAUX;
    return messa_ForIcoInd (ico->index_creat, message);
}

int messa_ForIcoInd (ico_ind, message)
int ico_ind;
Messa_ev *message;
{
    int i;
    for (i = 0; i < message->ico_ind_n; i++)
	if (ico_ind == message->ico_ind_v[i]) 
	    return VRAI;
    return FAUX;
}
