/***************************************************************************************/
/* method.c                                                   I.P.S  v 3.1  18/11/1993 */
/*                                                                                     */
/* Declaration des methodes des objets pour icobox.c     Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"

static Menu menu_image, menu_oper, menu_telop, menu_macro, menu_matri, menu_vor2d;


/***************************************************************************************
 *
 *
 ***************************************************************************************/
void  method_init (ico)
Ico  *ico;
{
  switch (ico->nat_obj)
  {
    case IPS_IMAGE : ico->menu = menu_image;
                     ico->obj  = (Image *) pcp_alloc (sizeof(Image));
                     break;
    case IPS_OPER  : ico->menu = menu_oper;
                     ico->obj  = (Oper *) pcp_alloc (sizeof(Oper));
                     break;
    case IPS_TELOP : ico->menu = menu_telop;
                     ico->obj  = (Telop *) pcp_alloc (sizeof(Telop));
                     break;
    case IPS_MACRO : ico->menu = menu_macro;
                     ico->obj  = (MaCom *) pcp_alloc (sizeof(MaCom));
                     break;
    case IPS_MATRI : ico->menu = menu_matri;
                     ico->obj  = (Matri *) pcp_alloc (sizeof(Matri));
                     break;
    case IPS_VOR2D : ico->menu = menu_vor2d;
                     ico->obj  = (Vor2d *) pcp_alloc (sizeof(Vor2d));
                     break;
    case IPS_TOOL :  ico->menu = menu_matri;
                     ico->obj  = (Tool *) pcp_alloc (sizeof(Tool));
                     break;
    default : pcp_error ("method_init #01","unknown object type", 0);
  }
}

/***************************************************************************************
 *
 *
 ***************************************************************************************/
void  method_free (ico)
Ico  *ico;
{
  switch (ico->nat_obj)
  {
    case IPS_IMAGE : image_CloseAllWin (ico);
                     image_free ((Image *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    case IPS_OPER  : oper_CloseAllWin (ico);
                     oper_free ((Oper *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    case IPS_TELOP : telop_CloseAllWin (ico);
		     telop_maj_inout (ico);
                     telop_free ((Telop *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    case IPS_MACRO : macro_CloseAllWin (ico);
                     macro_free ((MaCom *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    case IPS_MATRI : matri_CloseAllWin (ico);
                     matri_free ((Matri *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    case IPS_VOR2D : vor2d_CloseAllWin (ico);
                     vor2d_free ((Vor2d *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    case IPS_TOOL  : tool_CloseAllWin (ico);
                     tool_free ((Tool *) ico->obj);
                     pcp_free (ico->obj);
                     break;
    default : pcp_error ("method_free #01","unknown object type", 0);
  }
}

/***************************************************************************************
 *
 *
 ***************************************************************************************/
void  method_duplic (ico_new, ico_old)
Ico  *ico_new, *ico_old;
{
  method_init (ico_new);

  switch (ico_old->nat_obj)
  {
    case IPS_IMAGE : image_duplic (ico_new, ico_old);
                     break;
    case IPS_OPER  : oper_duplic (ico_new, ico_old);
                     break;
    case IPS_TELOP : telop_duplic (ico_new, ico_old);
                     break;
    case IPS_MACRO : macro_duplic (ico_new, ico_old);
                     break;
    case IPS_MATRI : matri_duplic (ico_new, ico_old);
                     break;
    case IPS_VOR2D : vor2d_duplic (ico_new, ico_old);
                     break;
    case IPS_TOOL  : tool_duplic (ico_new, ico_old);
                     break;
    default : pcp_error ("method_duplic #01","unknown object type", 0);
  }
}

/***************************************************************************************
 *
 *
 ***************************************************************************************/
void  method_affi_ico (ico)
Ico  *ico;
{
  switch (ico->nat_obj)
  {
    case IPS_IMAGE : image_affi_ico (ico);
                     break;
    case IPS_OPER  : oper_affi_ico (ico);
                     break;
    case IPS_TELOP : telop_affi_ico (ico);
                     break;
    case IPS_MACRO : macro_affi_ico (ico);
                     break;
    case IPS_MATRI : matri_affi_ico (ico);
                     break;
    case IPS_VOR2D : vor2d_affi_ico (ico);
                     break;
    case IPS_TOOL  : tool_affi_ico (ico);
                     break;
    default : pcp_error ("method_affi_ico #01","unknown object type", 0);
  }
}

/***************************************************************************************
 * Grise ou change un nom dans un menu contextuel
 *
 ***************************************************************************************/
void  method_menu_contextuel (ico)
Ico  *ico;
{
  method_MenuGrise (ico, "Clear",     ico->lock & LOCK_CLEAR);
  method_MenuGrise (ico, "Duplicate", ico->lock & LOCK_DUPLIC);

  switch (ico->nat_obj)
  {
    case IPS_IMAGE : 
	method_MenuGrise (ico, "Properties", ico->lock & LOCK_MODIF);
	break;
    case IPS_OPER  : {
	Oper *op = (Oper *) ico->obj;
	method_MenuBascule (ico, "Attach", "Detach", (op->attach_n) ? 2 : 1);
      } break;
    case IPS_TELOP : {
	Telop *op = (Telop *) ico->obj;
	int lequel = (op->led_pourcent == LED_SLEEP) ? 1 : 2;
	method_MenuBasculeGrise (ico, "Run", "Stop", lequel, 
			(lequel == 1) ? ico->lock & LOCK_RUN : FAUX);
	method_MenuBascule (ico, "Attach", "Detach", (op->attach_n) ? 2 : 1);
      } break;
    case IPS_MACRO : break;
    case IPS_MATRI : break;
    case IPS_VOR2D : break;
    case IPS_TOOL  : break;
    default : pcp_error ("method_menu_contextuel #01","unknown object type", 0);
  }
}

/***************************************************************************************
 * Fonctions de manipulation de menus, appelees dans method_menu_contextuel.
 *
 ***************************************************************************************/
void method_MenuGrise (ico, nom, grise)
Ico *ico;
char *nom;
int grise;
{
    Menu_item item = (Menu_item) xv_find (ico->menu, MENUITEM,
	MENU_STRING, nom, XV_AUTO_CREATE, FALSE, 0);
    if (item != 0) 
	xv_set (item, MENU_INACTIVE, (grise) ? TRUE : FALSE, 0);
}

void method_MenuBascule (ico, nom1, nom2, lequel)
Ico *ico;
char *nom1, *nom2;
int lequel;
{
    char *cherche, *remplace;
    Menu_item item;
    switch (lequel) {
	case 1 : cherche = nom2; remplace = nom1; break;
	case 2 : cherche = nom1; remplace = nom2; break;
    }
    item = (Menu_item) xv_find (ico->menu, MENUITEM,
	MENU_STRING, cherche, XV_AUTO_CREATE, FALSE, 0);
    if (item != 0) 
	xv_set (item, MENU_STRING, remplace, 0);
}

void method_MenuBasculeGrise (ico, nom1, nom2, lequel, grise)
Ico *ico;
char *nom1, *nom2;
int lequel, grise;
{
    char *cherche, *remplace;
    Menu_item item;
    switch (lequel) {
	case 1 : cherche = nom2; remplace = nom1; break;
	case 2 : cherche = nom1; remplace = nom2; break;
    }
    item = (Menu_item) xv_find (ico->menu, MENUITEM,
	MENU_STRING, cherche, XV_AUTO_CREATE, FALSE, 0);
    if (item != 0) 
	xv_set (item, MENU_STRING, remplace, 
		MENU_INACTIVE, (grise) ? TRUE : FALSE, 0);
    else method_MenuGrise (ico, remplace, grise);
}

/***************************************************************************************
 * Avertis l'Icone que au prochain affichage il se redessinera completement.
 *
 ***************************************************************************************/
void  method_obsolete (ico)
Ico  *ico;
{
  switch (ico->nat_obj)
  {
    case IPS_IMAGE : image_obsolete ((Image *) ico->obj);
                     break;
    case IPS_OPER  :
                     break;
    case IPS_TELOP :
                     break;
    case IPS_MACRO :
                     break;
    case IPS_MATRI :
                     break;
    case IPS_VOR2D :
                     break;
    case IPS_TOOL :
                     break;
    default : pcp_error ("method_obsolete #01","unknown object type", 0);
  }
}

/***************************************************************************************
 *
 *
 ***************************************************************************************/
void  method_nothing (ico)
Ico  *ico;
{
}

/***************************************************************************************
 *
 *
 ***************************************************************************************/
void  method_menu_decla ()
{
  menu_image = (Menu) xv_create (0, MENU,
      MENU_TITLE_ITEM,          "Image",
      MENU_STRINGS,             "Edit", "Properties",
                                "Duplicate", "Back", "Refresh", "Clear",
                                0,
      MENU_NOTIFY_PROC,         method_menu_proc,
      0); 

  menu_oper = (Menu) xv_create (0, MENU,
      MENU_TITLE_ITEM,          "Oper",
      MENU_STRINGS,             "Run", "Params", "Attach",
                                "Duplicate", "Back", "Refresh", "Clear",
                                0,
      MENU_NOTIFY_PROC,         method_menu_proc,
      0); 

  menu_telop = (Menu) xv_create (0, MENU,
      MENU_TITLE_ITEM,          "telOper",
      MENU_STRINGS,             "Run", "Params", "Attach",
                                "Duplicate", "Back", "Refresh", "Clear",
                                0,
      MENU_NOTIFY_PROC,         method_menu_proc,
      0); 

  menu_macro = (Menu) xv_create (0, MENU,
      MENU_TITLE_ITEM,          "Macro",
      MENU_STRINGS,             "Exec", "Memo",
				"Duplicate", "Back", "Refresh", "Clear", 
                                0,
      MENU_NOTIFY_PROC,         method_menu_proc,
      0); 

  menu_matri = (Menu) xv_create (0, MENU,
      MENU_TITLE_ITEM,          "Matrix",
      MENU_STRINGS,             "Duplicate", "Back", "Refresh", "Clear", 
                                0,
      MENU_NOTIFY_PROC,         method_menu_proc,
      0);

   menu_vor2d = (Menu) xv_create (0, MENU,
      MENU_TITLE_ITEM,          "Vor 2D",
      MENU_STRINGS,             "Duplicate", "Back", "Refresh", "Clear", 
                                0,
      MENU_NOTIFY_PROC,         method_menu_proc,
      0); 
}

/***************************************************************************************
 * event_proc d'un menu d'icone                                                        *
 *                                                                                     *
 ***************************************************************************************/
void
method_menu_proc (menu, menu_item)
Menu menu;
Menu_item menu_item;
{
  char  *choice = (char *) xv_get (menu_item, MENU_STRING);
  Ico   *clic   = (Ico *)  xv_get (menu, MENU_CLIENT_DATA);

  method_menu_exec_choice (choice, clic);
}

/***************************************************************************************
 * Action liee au double clic                                                          *
 *                                                                                     *
 ***************************************************************************************/
void
method_menu_dbl_clic (ico)
Ico  *ico;
{
  if (ico == NULL) pcp_error ("method_menu_dbl_clic #01","Ico *NULL", 0);

  else switch (ico->nat_obj)
  {
    case IPS_IMAGE : edIMA_edit_ico (ico, FAUX); /* ne regarde pas la selection */
                     break;
    case IPS_OPER  : if ( ((Oper *) ico->obj)->attach_n > 0 || ico_sel_get_nb () != 1)
                          method_menu_exec_choice ("Run", ico);
                     else method_menu_exec_choice ("Params", ico);
                     break;
    case IPS_TELOP : if ( ((Telop *) ico->obj)->attach_n > 0 || ico_sel_get_nb () != 1)
                          method_menu_exec_choice ("Run", ico);
                     else method_menu_exec_choice ("Params", ico);
                     break;
    case IPS_MACRO : if (ico_sel_get_nb () != 1)
                          method_menu_exec_choice ("Exec", ico);
                     else method_menu_exec_choice ("Memo", ico);
                     break;
    case IPS_MATRI : 
                     break;
    case IPS_VOR2D : 
                     break;
    case IPS_TOOL : 
                     break;
    default : pcp_error ("method_menu_dbl_clic #02","unknown object type", 0);
  }
}

/***************************************************************************************
 * Execution du choix dans un menu                                                     *
 *                                                                                     *
 ***************************************************************************************/
void
method_menu_exec_choice (choice, clic)
char *choice;
Ico  *clic;
{ 
    int    x  = (int) xv_get (glo.canvas, XV_X) + clic->xb + clic->xm/2;
    int    y  = (int) xv_get (glo.canvas, XV_Y) + clic->yb + clic->ym/2;

    if (strcmp (choice, "Front") == 0) {

	ico_posit_clic (clic, 0); /* Front */

    } else if (strcmp (choice, "Back") == 0) {

	ico_posit_clic (clic, 1); /* Back */

    } else if (strcmp (choice, "Clear") == 0 && (clic->lock & LOCK_CLEAR) == 0) {

	ico_posit_clic (clic, 2); /* Clear */

    } else if (strcmp (choice, "Refresh") == 0) {

	method_obsolete (clic);
	ico_posit_clic (clic, 3); /* Refresh */

    } else if (strcmp (choice, "Duplicate") == 0 && (clic->lock & LOCK_DUPLIC) == 0) {

	Ico *dup = ico_duplic (clic);
	ico_tete_pz (dup);
	dup->sel = FALSE;
	dup->xb += 10;
	dup->yb += 16;
	ico_posit_clic (dup, 0); /* Front */

    } else if (strcmp (choice, "Edit") == 0) {

	if (clic->nat_obj == IPS_IMAGE)
	    edIMA_edit_ico (clic, FAUX); /* ne regarde pas la selection */

    } else if (strcmp (choice, "Run") == 0 && (clic->lock & LOCK_RUN) == 0) {

	if (clic->nat_obj == IPS_OPER)
	    oper_check_run (clic);
	else if (clic->nat_obj == IPS_TELOP)
	    telop_check_run (clic, VRAI);

    } else if (strcmp (choice, "Stop") == 0) {

	if (clic->nat_obj == IPS_TELOP)
	    telop_stop (clic);

    } else if (strcmp (choice, "Params") == 0) {

	if (clic->nat_obj == IPS_OPER)
	    oper_parbox_creat (clic);
	else if (clic->nat_obj == IPS_TELOP)
	    telop_parbox_creat (clic);

    } else if (strcmp (choice, "Properties") == 0 && (clic->lock & LOCK_MODIF) == 0) {

	if (clic->nat_obj == IPS_IMAGE)
	    image_props_affi_frame (clic);

    } else if (strcmp (choice, "Attach") == 0) {

	if (clic->nat_obj == IPS_OPER) {
	    oper_attach (clic);
	} else if (clic->nat_obj == IPS_TELOP) {
	    telop_attach (clic);
	}

    } else if (strcmp (choice, "Detach") == 0) {

	if (clic->nat_obj == IPS_OPER) {
	    oper_detach (clic);
	} else if (clic->nat_obj == IPS_TELOP) {
	    telop_detach (clic);
	}

    } else if (strcmp (choice, "Exec") == 0) {

	if (clic->nat_obj == IPS_MACRO)
	    macro_check_run (clic);

    } else if (strcmp (choice, "Memo") == 0) {

	if (clic->nat_obj == IPS_MACRO)
	    macro_memo_affi_frame (clic);

    } else {

	pcp_footer ("Sorry, nothing to do", 0);
    }

}

/***************************************************************************************
 * Creation du bouton et du menu "New" du frame_principal
 *
 ***************************************************************************************/
Panel_item method_FramePcpButtCreat (panel)
Panel panel;
{
    Menu menu  = (Menu) xv_create (0, MENU,
	MENU_STRINGS,		"Image", "Macro", "Mask", "Voronoi", 0,
	MENU_NOTIFY_PROC,	method_FramePcpMenuProc,
	0);

    return (Panel_item) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"New",
	PANEL_ITEM_MENU,	menu,
	0);
}

/***************************************************************************************
 * Activation du choix dans le menu "New" du frame_principal
 *
 ***************************************************************************************/
void method_FramePcpMenuProc (menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    char *menu_str = (char *) xv_get (menu_item, MENU_STRING);

    if (!strcmp(menu_str, "Image")) {

	Ico *ico = ico_creat (glo.sx, glo.sy, IPS_IMAGE);
	ico_tete_pz (ico); 
	image_creat ((Image *) ico->obj, 0, 256, 256, NULL);
	strcpy(((Image *) ico->obj)->ch1, "Image");
	image_calc_hublot (ico);
	ico_douane (ico);
	ico_refresh_now (ico);
	image_props_affi_frame (ico);
    }
    else if (!strcmp(menu_str, "Macro")) {

	pcp_avertissement ("","Not available","");
#ifdef TEMPORAIREMENT_DESACTIVE
	Ico *ico = ico_creat (glo.sx, glo.sy, IPS_MACRO);
	ico_tete_pz (ico); 
	macro_creat ((MaCom *) ico->obj);
	macro_prepare_noms (ico, "Macro Name");
	ico_douane (ico);
	ico_refresh_now (ico);
	/* macro_memo_affi_frame (ico); */
#endif
    }
    else if (!strcmp(menu_str, "Mask")) {

	pcp_avertissement ("","Not available","");
    }
    else if (!strcmp(menu_str, "Voronoi")) {

	pcp_avertissement ("","Not available","");
#ifdef TEMPORAIREMENT_DESACTIVE
	Ico *ico = ico_creat (glo.sx, glo.sy, IPS_VOR2D);
	ico_tete_pz (ico); 
	vor2d_creat ((Vor2d *) ico->obj, 100, 0);
	vor2d_prepare_noms (ico, "Diagramme");
	ico_douane (ico);
	ico_refresh_now (ico);
	/* ajout : props */
#endif
    }

    glo.sx += 10; glo.sy += 16;
}

/***************************************************************************************
 * _messaProc commune aux icos
 * Attention les icos ne sont pas de vrais clients
 ***************************************************************************************/
void method_messaProcIco (ico, message)
Ico *ico;
Messa_ev *message;
{
    int repaint = FAUX;

#   ifdef DEBUG
    printf ("method_messaProcIco %d\n", ico->index_creat);
#   endif

    if ( messa_refresh(message)
      || messa_done(message)
      || messa_lut(message)
      || messa_name(message)
      || messa_redraw_rect(message) ) {
	method_obsolete (ico);
	repaint = VRAI;
    }

    if (repaint) 
	ico_posit_clic (ico, 3); /* refresh */
}
