/***************************************************************************************/
/* obj_telop.c                                                I.P.S  v 3.6  15/09/1995 */
/*                                                                                     */
/* gestion de l'objet telop (ico->nat_obj = ICO_TELOP)   Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include "inclus.h"

#define DEBUG_


/****************************************************************************************
 * FUNCTION  : 
 *
 * PURPOSE   : Initialisation du telop op
 *
 * CALLED IN : telop_MenuHop
 *
 ***************************************************************************************/
void   telop_init (op, idf, nomf, nom_aide, nom_host)
Telop  *op;
char   *idf, *nomf, *nom_aide, *nom_host;
{ 
  op->idf		= stg_Cpy (idf);	/* idf de la commande lue dans menu	*/
  op->nomf		= stg_Cpy (nomf);	/* nom du fichier executable		*/
  op->nom_aide		= stg_Cpy (nom_aide);	/* nom du fichier de l'aide		*/
  op->nom_host		= stg_Cpy (nom_host);	/* nom de la machine distante		*/
  op->led_pourcent	= LED_SLEEP;		/* LED_SLEEP, LED_RUN, ..		*/
  op->nb_image_in	= 0;			/* nb d'images en entree		*/
  op->nb_image_out	= 0;			/* nb d'images en sortie		*/
  op->same_size		= VRAI;			/* meme taille des images existantes	*/
  op->val_cadre		= IPS_BORDER_NOONE;	/* cadre autour images : IPS_BORDER_	*/
  op->mot_x		= 10;			/* largeur max des sous titres		*/
  op->param_nb		= 0;			/* nb de params				*/
  op->param_type	= NULL;			/* vecteur des types des parametres	*/
  op->param_adr		= NULL;			/* vecteur des adresses des parametres	*/
  op->titre		= NULL;			/* titre du telop			*/
  op->lgd_images	= NULL;			/* Legendes pr les images en in		*/
  op->frame_parbox	= 0;			/* Frame de la boite de dialogue 	*/
  op->attach_n		= 0;			/* aucun telop attache			*/
  op->attach_v		= NULL;			/* vect des num icos attaches a op	*/
  op->ctx		= -1;			/* contexte ; -1 pour aucun		*/
  op->fig		= NULL;			/* fig windows, cf figw.c		*/
}
 
/****************************************************************************************
 * FUNCTION  : telop_free ( Telop *op );
 *
 * PURPOSE   : Suppression du telop op
 *
 * CALLED IN : method_free
 *
 ***************************************************************************************/
void   telop_free (op)
Telop *op;
{
  int i, t, n, k;

  /* contexte */
  if (op->ctx != -1) cs_kill_ctx (op->ctx);

  /* fig windows */
  fig_free (op->fig);

  /* strings */
  pcp_free (op->idf);
  pcp_free (op->nomf);
  pcp_free (op->nom_aide);
  pcp_free (op->nom_host);
  pcp_free (op->titre);

  /* Legendes pr les images en in,out */
  n = op->nb_image_in + op->nb_image_out;
  for (i = 0; i < n; i++) pcp_free (op->lgd_images[i]);
  if (n > 0) pcp_free (op->lgd_images);

  /* attachement */
  if (op->attach_n > 0) pcp_free (op->attach_v);
 
  if (op->param_nb > 0)
  {
  /* vecteur des adresses des parametres */
    for (i = 0; i < op->param_nb; i++)
    switch (t = op->param_type[i])
    {
      case IPS_PAR_REAL : {
          Telop_par_real *para = (Telop_par_real *) op->param_adr[i];
          pcp_free (para->legende);
        } break;

      case IPS_PAR_CHK_BOX :
      case IPS_PAR_SLIDER :
      case IPS_PAR_INT : {
          Telop_par_int *para = (Telop_par_int *) op->param_adr[i];
          pcp_free (para->legende);
        } break;

      case IPS_PAR_TEXT : {
          Telop_par_text *para = (Telop_par_text *) op->param_adr[i];
          pcp_free (para->legende);
          pcp_free (para->defaut);
          pcp_free (para->valeur);
        } break;

      case IPS_PAR_CH_ABR :
      case IPS_PAR_CHOICE : {
          Telop_par_choice *para = (Telop_par_choice *) op->param_adr[i];
          for (k = 0; k < para->nchoice; k++) pcp_free (para->choices[k]);
          pcp_free (para->choices);
          pcp_free (para->legende);
        } break;

      case 0  : break; /* param non attribue' */
      default : pcp_error ("telop_free #01", "type not implemented", NULL);
    }

    for (i = 0; i < op->param_nb; i++) pcp_free (op->param_adr[i]);
    pcp_free (op->param_adr);

    /* vecteur des types des parametres */
    pcp_free (op->param_type);
  }
}
 
/****************************************************************************************
 * FUNCTION  : telop_duplic ( Ico *ico_new, *ico_old );
 *
 * PURPOSE   : Duplication de l'icone IPS_TELOP : ico_n := ico_o
 *
 * CALLED IN : method_duplic
 *
 ***************************************************************************************/
void  telop_duplic (ico_n, ico_o)
Ico  *ico_n, *ico_o;
{ 
  Telop *op_o = (Telop *) ico_o->obj;
  Telop *op_n = (Telop *) ico_n->obj;
  int i, t, n, k;

  memcpy (op_n, op_o, sizeof(Telop));

  /* strings */
  op_n->idf = stg_Cpy (op_o->idf);
  op_n->nomf = stg_Cpy (op_o->nomf);
  op_n->nom_aide = stg_Cpy (op_o->nom_aide);
  op_n->nom_host = stg_Cpy (op_o->nom_host);
  op_n->titre = stg_Cpy (op_o->titre);

  /* Legendes pr les images en in,out */
  n = op_o->nb_image_in + op_o->nb_image_out;
  if (n > 0) 
  { op_n->lgd_images = (char **) pcp_alloc (n * sizeof(char *));
    for (i = 0; i < n; i++)
      op_n->lgd_images[i] = stg_Cpy (op_o->lgd_images[i]);
  }
 
  /* attachement */
  op_n->led_pourcent = LED_SLEEP;
  op_n->attach_n = 0;
  op_n->attach_v = NULL;

  /* contexte */
  op_n->ctx = -1;

  /* fig windows */
  op_n->fig = fig_duplic (op_o->fig);

  if (op_o->param_nb > 0)
  {
    /* vecteur des types des parametres */
    op_n->param_type = (int *) pcp_alloc (op_o->param_nb * sizeof(int));
    for (i = 0; i < op_o->param_nb; i++) 
    op_n->param_type[i] = op_o->param_type[i];

    /* vecteur des adresses des parametres */
    op_n->param_adr = (void **) pcp_alloc (op_o->param_nb * sizeof(void *));
    for (i = 0; i < op_o->param_nb; i++)
    switch (t = op_o->param_type[i])
    {
      case IPS_PAR_REAL : {
          Telop_par_real *par_o = (Telop_par_real *) op_o->param_adr[i];
          Telop_par_real *par_n = (Telop_par_real *) pcp_alloc (sizeof(Telop_par_real));

          (Telop_par_real *) op_n->param_adr[i] = par_n;

          memcpy (par_n, par_o, sizeof(Telop_par_real));

          par_n->legende = stg_Cpy (par_o->legende); 
        } break;

      case IPS_PAR_CHK_BOX :
      case IPS_PAR_SLIDER :
      case IPS_PAR_INT  : {
          Telop_par_int *par_o = (Telop_par_int *) op_o->param_adr[i];
          Telop_par_int *par_n = (Telop_par_int *) pcp_alloc (sizeof(Telop_par_int));

          (Telop_par_int *) op_n->param_adr[i] = par_n;

          memcpy (par_n, par_o, sizeof(Telop_par_int));

          par_n->legende = stg_Cpy (par_o->legende); 
        } break;

      case IPS_PAR_TEXT : {
          Telop_par_text *par_o = (Telop_par_text *) op_o->param_adr[i];
          Telop_par_text *par_n = (Telop_par_text *) pcp_alloc (sizeof(Telop_par_text));

          (Telop_par_text *) op_n->param_adr[i] = par_n;

          par_n->taille_max = par_o->taille_max;

          par_n->legende = stg_Cpy (par_o->legende);
          par_n->defaut  = stg_Cpy (par_o->defaut);
          par_n->valeur  = (char *) pcp_alloc (par_o->taille_max + 1);
          strcpy (par_n->valeur,  par_o->valeur); 
        } break;

      case IPS_PAR_CH_ABR :
      case IPS_PAR_CHOICE : {
          Telop_par_choice *par_o = (Telop_par_choice *) op_o->param_adr[i];
          Telop_par_choice *par_n = (Telop_par_choice *) pcp_alloc (sizeof(Telop_par_choice));

          (Telop_par_choice *) op_n->param_adr[i] = par_n;

          memcpy (par_n, par_o, sizeof(Telop_par_choice));

          par_n->legende = stg_Cpy (par_o->legende);
          par_n->choices = (char **) pcp_alloc (par_o->nchoice * sizeof (char *));
          for (k = 0; k < par_o->nchoice; k++)
            par_n->choices[k] = stg_Cpy (par_o->choices[k]);
        } break;

      case 0 : /* param non attribue' */
	  op_n->param_adr[i] = (Telop_par_text *) NULL;
	  break;
 
      default : pcp_error ("telop_duplic #01", "type not implemented", NULL);
    }
  }
  else { op_n->param_adr = NULL; op_n->param_type = NULL; }

  /* divers */
  op_n->frame_parbox = 0;
}
 
/****************************************************************************************
 * FUNCTION  : telop_affi_ico ( Ico *ico );
 *
 * PURPOSE   : Afficher l'icone IPS_TELOP
 *
 * CALLED IN : method_affi_ico
 *
 ***************************************************************************************/
void  telop_affi_ico (ico)
Ico  *ico;
{
  Telop *op = (Telop *) ico->obj;
  int    a1, a2, a3, x, y, l;

  stg_PrintCenter (ico->xb+ico->xm/2, ico->yb+2, op->mot_x, op->titre);

  if (ico->sel)
       { a1 = glo.c[4]; a2 = glo.c[1]; }
  else { a1 = glo.c[1]; a2 = glo.c[4]; }
  a3 = (op->led_pourcent == LED_WAIT) ? 6 :
       (op->led_pourcent >=  LED_RUN) ? 12 :
       (op->attach_n) ? 9 : 10;	/* attach : sleep */

  x = (op->ctx == -1) ? ico->xb+10 : ico->xb+(ico->xm-42)/2;
  y = ico->yb+ico->ym - 14;
  l = (op->ctx == -1) ? 12 : 42;
  ico_affi_rect_3d (x, y, l, 6, a1, a2);

  XSetForeground (glo.dpy, glo.gc, coul_pic[a3]);
  XFillRectangle (glo.dpy, glo.xwin, glo.gc, x+1, y+1, l-2, 4);
  if (op->led_pourcent < LED_RUN_LO || op->led_pourcent > LED_RUN_HI) return;
  XSetForeground (glo.dpy, glo.gc, coul_pic[14]);
  XFillRectangle (glo.dpy, glo.xwin, glo.gc, 
	x+1+op->led_pourcent, y-2, 2, 10);
}

/****************************************************************************************
 * FUNCTION  : telop_prepare_noms
 *
 * PURPOSE   : Calcule les coords des sous titres de l'IPS_TELOP
 *
 * CALLED IN : telop_MenuHop
 *
 ****************************************************************************************/
void  telop_prepare_noms (ico)
Ico  *ico;
{
    Telop *ob = (Telop *) ico->obj;
    char bla[500];
    int nb;

    if (ob->titre == NULL) ob->titre = stg_Cpy ("WARNING: Oper without name");

    /* Normalise espaces */
    stg_NormSpace (ob->titre, bla);
    pcp_free (ob->titre);
    ob->titre = stg_Cpy (bla);

    /* Repartit les mots */
    stg_RepartiMots (bla, &(ob->mot_x), &nb);

    /* calcul de ico->xm, ico->ym */
    ico->ym = 3 + nb*(glo.font_h+2) + 16+6;
    ico->xm = max (ob->mot_x * glo.font_l + 14, 16+42);
}


/****************************************************************************************
 * FUNCTION  : telop_decode_def 
 *
 * PURPOSE   : Decode la DEF du telop, encodee par tp_encode_def
 *
 * CALLED IN : cs_input
 *
 ****************************************************************************************/
void   telop_decode_def (ico, buf, tt)
Ico   *ico;
char  *buf;  /* le buffer ou chaque element est separe par un '\0' */
int   tt;    /* la taille du buffer ; il est termine' par un double '\0' */
{
  Telop   *op = (Telop *) ico->obj;
  int      ordre, i, n, num, param_type[TELOP_VECM], k, vasy = VRAI;
  void    *param_adr[TELOP_VECM];
  char    *lgd_images[TELOP_VECM], *legende, *p;

  /* init */
  p = buf;
  op->param_nb = op->nb_image_in = op->nb_image_out = 0;
  for (i = 1; i < TELOP_VECM; i++)
  { param_type[i] = 0; param_adr[i] = NULL; lgd_images[i] = NULL; }

  /* lecture arguments */
  while (vasy && (ordre = atoi (stg_buf_rd (&p))) != 0)
  switch (ordre) /* #1 */
  {  
    case IPS_NAME : {
        char *pligne = stg_buf_rd (&p);
	pcp_free (op->titre);
        op->titre = stg_Cpy (pligne); 
      } break;

    case IPS_IMAGE_IN  : 
    case IPS_IMAGE_OUT :
        num     = atoi (stg_buf_rd (&p));
        legende = stg_buf_rd (&p);

        if (num < 1 || num > OPER_VECM-1)
        { pcp_error ("telop_def #02","image number is not correct", NULL); break; }
        if (lgd_images[num] != NULL)
        { pcp_error ("telop_def #03","image number is already used", NULL); break; }

        switch (ordre) /* #3 */
        {
          case IPS_IMAGE_IN : 
              if (op->nb_image_out > 0)
              { pcp_error ("telop_def #04","IMAGE_IN defined after IMAGE_OUT", NULL); num = 0; }
              if (num > op->nb_image_in) op->nb_image_in = num;
              break;
          case IPS_IMAGE_OUT : 
              if (num > op->nb_image_out) op->nb_image_out = num;
              break;
        } /* #3 */

        if (num == 0) break;
        lgd_images[num] = stg_Cpy (legende);
        break;

    case IPS_FIG_IMAGE : {
	int num_fig = atoi (stg_buf_rd (&p));
	int num_ima = atoi (stg_buf_rd (&p));
        char *legen = stg_buf_rd (&p);
	fig_decode_def (op, num_fig, num_ima, legen);
    } break;

    case IPS_BORDER_ANY :
    case IPS_BORDER_POSIT :
    case IPS_BORDER_NOONE :
        op->val_cadre = ordre;
        break;
    case IPS_BORDER :
        op->val_cadre = atoi (stg_buf_rd (&p));
        break;
    case IPS_SAME_SIZE :
        op->same_size = atoi (stg_buf_rd (&p));
        break;

    case IPS_PAR_REAL    : 
    case IPS_PAR_CHK_BOX :
    case IPS_PAR_SLIDER  :
    case IPS_PAR_INT     : 
    case IPS_PAR_TEXT    : 
    case IPS_PAR_CH_ABR  :
    case IPS_PAR_CHOICE  : 
        num = atoi (stg_buf_rd (&p));

        if (num < 1 || num > OPER_VECM-1)
        { pcp_error ("telop_def #05","param number is not correct", NULL); num = 0; }
        if (param_type[num] != 0)
        { pcp_error ("telop_def #06","param number is already used", NULL); num = 0; }
        param_type[num] = ordre;
        if (num > op->param_nb) op->param_nb = num;

        switch (ordre) /* #2 */
        {
          case IPS_PAR_REAL : {
              char		*legende	= stg_buf_rd (&p);
              double		borne_inf	= atof (stg_buf_rd (&p));
              double		borne_sup	= atof (stg_buf_rd (&p));
	      double		val_defaut	= atof (stg_buf_rd (&p));
              Telop_par_real	*para;

              if (num == 0) break;

              para = (Telop_par_real *) pcp_alloc (sizeof(Telop_par_real));
              (Telop_par_real *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->inf = borne_inf;
              para->sup = borne_sup;
              para->defaut = para->valeur = val_defaut;
#             ifdef DEBUG
                printf ("IPS_PAR_REAL num.%2d \"%s\" %f %f %f\n", 
                        num, legende, borne_inf, borne_sup, val_defaut);
#             endif
            } break;

          case IPS_PAR_SLIDER :
          case IPS_PAR_INT : {
              char		*legende	= stg_buf_rd (&p);
              int		borne_inf	= atoi (stg_buf_rd (&p));
              int		borne_sup	= atoi (stg_buf_rd (&p));
              int		val_defaut	= atoi (stg_buf_rd (&p));
              Telop_par_int	*para;

              if (num == 0) break;

              para = (Telop_par_int *) pcp_alloc (sizeof(Telop_par_int));
              (Telop_par_int *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->inf = borne_inf;
              para->sup = borne_sup;
              para->defaut = para->valeur = val_defaut;
#             ifdef DEBUG
                printf ("IPS_PAR_INT  num.%2d \"%s\" %d %d %d\n", 
                        num, legende, borne_inf, borne_sup, val_defaut);
#             endif
            } break;
 
          case IPS_PAR_CHK_BOX : {
              char		*legende	= stg_buf_rd (&p);
              int		val_defaut	= atoi (stg_buf_rd (&p));
              Telop_par_int	*para;

              if (num == 0) break;

              para = (Telop_par_int *) pcp_alloc (sizeof(Telop_par_int));
              (Telop_par_int *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->inf = 0;
              para->sup = 1;
              para->defaut = para->valeur = val_defaut;
#             ifdef DEBUG
                printf ("IPS_PAR_CHK_BOX  num.%2d \"%s\" %d\n", 
                        num, legende, val_defaut);
#             endif
            } break;

          case IPS_PAR_TEXT : {
              char		*legende	= stg_buf_rd (&p);
              int		taille_max	= atoi (stg_buf_rd (&p));
              char		*val_defaut	= stg_buf_rd (&p);
              Telop_par_text	*para;

              if (num == 0) break;

              para = (Telop_par_text *) pcp_alloc (sizeof(Telop_par_text));
              (Telop_par_text *) param_adr[num] = para;

              para->legende = stg_Cpy (legende); 
              para->defaut  = stg_Cpy (val_defaut);
              para->taille_max = taille_max;
              para->valeur = (char *) pcp_alloc (taille_max + 1);
              strcpy (para->valeur, val_defaut);
#             ifdef DEBUG
                printf ("IPS_PAR_TEXT num.%2d \"%s\" %d \"%s\"\n", 
                        num, legende, taille_max, val_defaut);
#             endif
            } break;
 
          case IPS_PAR_CH_ABR :
          case IPS_PAR_CHOICE : {
              char		*legende	= stg_buf_rd (&p);
              int		val_defaut	= atoi (stg_buf_rd (&p));
              Telop_par_choice	*para;
              char              *nom, *choices[TELOP_VECM];

              if (num == 0) break;

              para = (Telop_par_choice *) pcp_alloc (sizeof(Telop_par_choice));
              (Telop_par_choice *) param_adr[num] = para;

              para->legende = stg_Cpy (legende);
              para->defaut = para->valeur = val_defaut;

              para->nchoice = 0;
              while ((nom = stg_buf_rd (&p))[0] != 0)
                choices[para->nchoice++] = stg_Cpy (nom);
              para->choices = (char **) pcp_alloc (para->nchoice * sizeof (char *));
              for (k = 0; k < para->nchoice; k++) para->choices[k] = choices[k];
#             ifdef DEBUG
                printf ("IPS_PAR_CHOICE num.%2d \"%s\" %d %d\n", 
                        num, legende, val_defaut, para->nchoice);
#             endif
            } break;
 
        } /* switch (ordre) #2 */
        break;

    case IPS_IDF : {
        char *pligne = stg_buf_rd (&p);
        op->idf = stg_Cpy (pligne); 
      } break;

    default : 
	pcp_error ("telop_def #07", op->idf, "error in definition", NULL);
	vasy = FAUX;
  } /* switch (ordre) #1 */

  /* enregistre params - attention, decalage dans numeros */
  if (op->param_nb > 0)
  {
    op->param_adr  = (void **) pcp_alloc (op->param_nb * sizeof(void *));
    op->param_type = (int *)   pcp_alloc (op->param_nb * sizeof(int));
    for (i = 0; i < op->param_nb; i++) 
    { op->param_adr[i] = param_adr[i+1];
      op->param_type[i]= param_type[i+1];
    }
  }

  /* enregistre lgd_images */
  if (op->nb_image_out > 0) op->nb_image_out -= op->nb_image_in;
  n = op->nb_image_in + op->nb_image_out;
  if (n > 0) op->lgd_images = (char **) pcp_alloc (n * sizeof(char *));
  for (i = 0; i < n; i++) op->lgd_images[i] = lgd_images[i+1];

  /* La DEF est maintenant complete.
   * - Probleme : si une parbox avait ete cre'e'e entre-temps, cf remarque vers la fin 
   *   de telop_parbox_creat.
   * - Solution : on la detruit puis on la reconstruit (mise-a-jour).
  */
  if (op->frame_parbox != 0) {
	telop_parbox_kill (op->frame_parbox);
	telop_parbox_creat (ico);
  }

  /* on enleve la protection mise a la creation de l'icone par telop_MenuHop */
  ico->lock = LOCK_NO;
}


/****************************************************************************************
 * FUNCTION  : telop_decode_cho 
 *
 * PURPOSE   : Decode resultat de section CHOICE du telop, encodee par otp_encode_CHO
 *
 * CALLED IN : cs_input
 *
 ****************************************************************************************/
void   telop_decode_cho (ico, buf, tt)
Ico   *ico;
char  *buf;  /* le buffer ou chaque element est separe par un '\0' */
int   tt;    /* la taille du buffer ; il est termine' par un double '\0' */
{
    Telop *op = (Telop *) ico->obj;
    int      ordre, num, gri;
    char    *p;
    Telop_parbox *p_b = NULL;

    /* init */
    p = buf;
    if (op->frame_parbox != 0)
	p_b = (Telop_parbox *) xv_get (op->frame_parbox, WIN_CLIENT_DATA);

    /* lecture arguments */
    while ((ordre = atoi (stg_buf_rd (&p))) != 0)
    switch (ordre) { /* #1 */

	case IPS_PAR_ACTIVE :
	case IPS_PAR_REAL :
	case IPS_PAR_TEXT :
	case IPS_PAR_CHK_BOX :
	case IPS_PAR_SLIDER :
	case IPS_PAR_INT :
	case IPS_PAR_CH_ABR :
	case IPS_PAR_CHOICE : {

            num = atoi (stg_buf_rd (&p));

	    if (num < 0 || num >= op->param_nb) { 
		pcp_error ("telop_decode_cho #02",
			"param number is not correct", NULL); 
		break;
	    }
	    if (op->param_type[num] == 0) { 
		pcp_error ("telop_decode_cho #03",
			"this param does not exist", NULL); 
		break;
	    }

	    switch (ordre) { /* #2 */

		case IPS_PAR_ACTIVE : {
        	    gri = atoi (stg_buf_rd (&p));
		    if (p_b != NULL)
			xv_set (p_b->adr[num], PANEL_INACTIVE, (gri == 2) ? 0 : 1, 0); 
		} break;
		case IPS_PAR_REAL : {
		    Telop_par_real *para;
		    char *bla;
		    para = (Telop_par_real *) op->param_adr[num];
		    para->valeur = atof (bla = stg_buf_rd (&p));
		    if (p_b != NULL)
			xv_set (p_b->adr[num], PANEL_VALUE, bla, 0);
		} break;
		case IPS_PAR_TEXT : {
		    Telop_par_text *para;
		    para = (Telop_par_text *) op->param_adr[num];
		    strcpy (para->valeur, stg_buf_rd (&p));
		    if (p_b != NULL)
			xv_set (p_b->adr[num], PANEL_VALUE, para->valeur, 0);
		    } break;
		case IPS_PAR_CHK_BOX :
		case IPS_PAR_SLIDER :
		case IPS_PAR_INT : {
		    Telop_par_int *para;
		    para = (Telop_par_int *) op->param_adr[num];
		    para->valeur = atoi (stg_buf_rd (&p));
		    if (p_b != NULL)
			xv_set (p_b->adr[num], PANEL_VALUE, para->valeur, 0);
		} break;
		case IPS_PAR_CH_ABR :
		case IPS_PAR_CHOICE : {
		    Telop_par_choice *para;
		    para = (Telop_par_choice *) op->param_adr[num];
		    para->valeur = atoi (stg_buf_rd (&p));
		    if (p_b != NULL)
			xv_set (p_b->adr[num], PANEL_VALUE, para->valeur, 0);
		} break;

	    } /* switch (ordre) #2 */
	} break;

	default : 
	    pcp_error ("telop_decode_cho #07", op->idf, "erreur d'encodage", NULL);
	    return;
    } /* #1 */
}


/****************************************************************************************
 * FUNCTION  : void telop_encode_cho ()
 *
 * PURPOSE   : le buf sera decode' par tp_decode_cho ()
 *
 * CALLED IN : cs_input
 *
 ****************************************************************************************/
void   telop_encode_cho (ico, buf, bufsize, bufmax)
Ico *ico;
char *buf;
int *bufsize, bufmax;
{
    Telop *op = (Telop *) ico->obj;
    char *p;
    int i;

    /* prologue */
    p = buf; *bufsize = 0;

    /* encodage parametres */
    for (i = 0; i < op->param_nb; i++) 
    if (op->param_type[i] != 0) {

	stg_buf_wr_int (&p, op->param_type[i]);  /* toujours != 0 final */
	stg_buf_wr_int (&p, i);

	switch (op->param_type[i]) {

	    case IPS_PAR_REAL : {
	        Telop_par_real *para = (Telop_par_real *) op->param_adr[i];
	        stg_buf_wr_dbl (&p, para->valeur);
	    } break;
	    case IPS_PAR_TEXT : {
	        Telop_par_text *para = (Telop_par_text *) op->param_adr[i];
		stg_buf_wr_int (&p, para->taille_max);
	        stg_buf_wr_str (&p, para->valeur);
	    } break;
	    case IPS_PAR_CHK_BOX :
	    case IPS_PAR_SLIDER :
	    case IPS_PAR_INT : {
	        Telop_par_int *para = (Telop_par_int *) op->param_adr[i];
	        stg_buf_wr_int (&p, para->valeur);
	    } break;
	    case IPS_PAR_CH_ABR :
	    case IPS_PAR_CHOICE : {
	        Telop_par_choice *para = (Telop_par_choice *) op->param_adr[i];
	        stg_buf_wr_int (&p, para->valeur);
	    } break;
	}
    }

    /* NULL final */
    stg_buf_wr_str (&p, "0");

    /* taille buffer */
    *bufsize = p - buf;
    if (*bufsize > bufmax) {
	printf ("ERROR 109 in section CHOICE : bufsize too large\n");
	*bufsize = 0;
    }

    /* CONTROLE */
#   ifdef DEBUG
    stg_buf_affi (buf, *bufsize);
#   endif
}

/****************************************************************************************
 * retrouve la selection, tri et tests
 *
 * Retourne 0 pour succes, -1 pour erreur
 ***************************************************************************************/
int telop_retrouve_selec (ico, v_ico, n_ico)
Ico *ico, **v_ico;
int *n_ico;
{
    Telop *op  = (Telop *) ico->obj;
    Ico *cur;
    int i;

    /*
     * init
    */
    for (i = 0; i < TELOP_VECM; i++) v_ico[i] = NULL;
    v_ico[0] = ico; *n_ico = 0;

    /*
     * retrouver la selection, tri et tests
    */
    if (op->attach_n == 0) {

	for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) 
	if (cur->sel) { 
	    if (IPS_IS_DATA(cur->nat_obj)) {
		if (++(*n_ico) >= TELOP_VECM) {
		    pcp_avertis_ico ("WARNING !",
			"Too much datas", "were selected", ico);
		    return -1;
		}
		ico_tri_selec (cur, v_ico, 1, *n_ico);  /* tri par insertion */
	    } else if (cur->index_creat != ico->index_creat) {
		pcp_avertis_ico ("WARNING !","Only datas",
		    "must be selected", cur);
		return -1;
	    }
	}
    }

    /*
     * retrouver les icones attaches
    */
    else for (i = 0; i < op->attach_n; i++) {

	    cur = ico_get_from_index_creat (op->attach_v[i]);
	    if (cur != (Ico *) NULL)
		v_ico[++(*n_ico)] = cur;
	    else {
		pcp_avertis_ico ("WARNING !","An attached icon", "is lost", ico);
		return -1;
	    }
    }

    /*
     * TESTE le nombre d'images
    */
    if (*n_ico < op->nb_image_in || 
        *n_ico > op->nb_image_in + op->nb_image_out) {
	char bla[200];
	sprintf (bla, "%d in and %d out\n", op->nb_image_in, op->nb_image_out);
	pcp_avertis_ico ("WARNING !","Number of datas needed by this telop :",bla, ico);
	return -1;
    }

    return 0;
}

/****************************************************************************************
 * retrouve icones et si possible lance la requete
 *
 * Renvoie 0 pour succes, -1 pour erreur.
 ***************************************************************************************/
int telop_check_run (ico, lancer_requete)
Ico *ico;
int lancer_requete;
{
    Ico		*v[TELOP_VECM], *cur;
    Telop	*op  = (Telop *) ico->obj;
    int		n, i;
    Telop_data_cs *data;

    if (op->led_pourcent > LED_SLEEP) {
	cs_footer (op->ctx, "Sorry, already running");
	return -1;
    }

    /*
     * Retrouve la selection, tests et tri
    */
    if (telop_retrouve_selec (ico, v, &n) < 0) return -1;

    /*
     * test si les icones sont proteges par LOCK_MODIF
    */
    for (i = 1; i <= n; i++) if (v[i]->lock & LOCK_MODIF) {
	pcp_avertis_ico ("WARNING !","That icon", "is locked", v[i]);
	return -1;
    }

    /*
     * Appel du tel_oper :
     * 
     * selon lancer_requete,
     *  - on lance la requete,
     *  - ou on est deja dans un dialogue, et on va en 300 depuis cs_input()
    */
    telop_change_led (ico, LED_WAIT);
    if (lancer_requete) {
	char bla[200];
	sprintf (bla, "300 %s", op->idf); /* DIM_EXE_ACT */
	if (cs_requete (ico, bla) < 0) return -1; 
    }

    /* 
     * stockage par cs_set_data2 ; le free est dans telop_maj_inout()
    */
    data = (Telop_data_cs *) pcp_alloc (sizeof (Telop_data_cs));
    memset (data, 0, sizeof (Telop_data_cs));
    if (data == NULL) return -1;
    data->nb = n;
    data->n1 = op->nb_image_in;
    data->n2 = op->nb_image_in + op->nb_image_out;
    memcpy (data->v, v, sizeof (Ico *) * OPER_VECM);
    cs_set_data2 (op->ctx, data);

    /*
     * Protection des IN/OUT jusqu'a l'appel de telop_decode_dim
     * dans lequel les OUT manquants seront cre'es.
    */
    for (i = 1; i <= data->nb; i++) if (v[i] != NULL)
	v[i]->lock = LOCK_CLEAR | LOCK_MODIF;

    return 0; /* succes */
}


/****************************************************************************************
 * FUNCTION  : 
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
void   telop_encode_dim (ico, buf, bufsize, bufmax)
Ico *ico;
char *buf;
int *bufsize, bufmax;
{
    Telop *op = (Telop *) ico->obj;
    Telop_data_cs *data = (Telop_data_cs *) cs_get_data2 (op->ctx);
    Ico **v = data->v;
    char *p, bla[256];
    int i;

    /*
     * choix taille cadre
    */
    switch (op->val_cadre) { 
	case IPS_BORDER_ANY   : 
	    data->cmin = (data->nb == 0) ? 0 : ((Image *)v[1]->obj)->b.cmin; break;
	case IPS_BORDER_POSIT : 
	    data->cmin = (data->nb == 0) ? 2 : ((Image *)v[1]->obj)->b.cmin; 
	    if (data->cmin == 0) data->cmin = 2; break;
	case IPS_BORDER_NOONE : 
	    data->cmin = 0; break;
	default : data->cmin = op->val_cadre;
    }

    /*
     * prologue encodage
    */
    p = buf; *bufsize = 0;

    /* infos generales CONSULTEES */
    stg_buf_wr_int (&p, IPS_IMAGE_INFOS);
    stg_buf_wr_int (&p, data->n2);		/* nb total d'images */
    stg_buf_wr_int (&p, data->cmin);		/* val cadre */

    /* encodage infos MODIFIABLES */
    for (i = 1; i <= data->n2; i++) {

	stg_buf_wr_int (&p, IPS_IMAGE);  
	stg_buf_wr_int (&p, i);

	/* taille en x */
	stg_buf_wr_int (&p, (data->nb == 0) 
	    ? 256 : (op->same_size || i > data->nb)
	    ? ((Image *)v[1]->obj)->b.xsiz : ((Image *)v[i]->obj)->b.xsiz );  

	/* taille en y */
	stg_buf_wr_int (&p, (data->nb == 0)
	    ? 256 : (op->same_size || i > data->nb)
	    ? ((Image *)v[1]->obj)->b.ysiz : ((Image *)v[i]->obj)->b.ysiz );  
    }

    /* NULL final */
    stg_buf_wr_str (&p, "0");

    /* taille buffer */
    *bufsize = p - buf;
    if (*bufsize > bufmax) {
	printf ("ERROR 109 in section CHOICE : bufsize too large\n");
	*bufsize = 0;
    }

    /* CONTROLE */
#   ifdef DEBUG
    stg_buf_affi (buf, *bufsize);
#   endif
}

/****************************************************************************************
 * FUNCTION  : 
 *
 * PURPOSE   : 
 *
 * CALLED IN : 
 *
 ***************************************************************************************/
int   telop_decode_dim (ico, buf, tt)
Ico *ico;
char  *buf;  /* le buffer ou chaque element est separe par un '\0' */
int   tt;    /* la taille du buffer ; il est termine' par un double '\0' */
{
    Telop *op = (Telop *) ico->obj;
    Telop_data_cs *data = (Telop_data_cs *) cs_get_data2 (op->ctx);
    Ico **v = data->v;
    Image *ima[OPER_VECM];
    char *p, ima_name[OPER_VECM][IMAGE_CH1LEN], bla[256];
    int i, ordre, num, exec_poss = VRAI, 
	ima_x[OPER_VECM], ima_y[OPER_VECM], ima_lut[OPER_VECM];

    /* 
     * init
    */
    p = buf;
    for (i = 1; i < OPER_VECM; i++) {
	ima_x[i] = ima_y[i] = 256;
	ima_lut[i] = 0;
	ima_name[i][0] = 0;
    }
    for (i = 1; i <= data->nb; i++) ima[i] = (Image *) v[i]->obj;

    /*
     * Deprotection des IN/OUT, qu'on reprotege a la fin,
     * i.e si tout s'est bien passe'.
    */
    for (i = 1; i <= data->n2; i++) if (v[i] != NULL)
    v[i]->lock = LOCK_NO;

    /*
     * lecture arguments
    */
    while (exec_poss && (ordre = atoi (stg_buf_rd (&p))) != 0)
    switch (ordre) { /* #1 */

	case IPS_IMAGE : {
            num = atoi (stg_buf_rd (&p));
            ima_x[num] = atoi (stg_buf_rd (&p));
            ima_y[num] = atoi (stg_buf_rd (&p));
            ima_lut[num] = atoi (stg_buf_rd (&p));
            strncpy (ima_name[num], stg_buf_rd (&p), IMAGE_CH1LEN);

	    /* Test pour securite */
	    if (ima_x[num] < 8 || ima_x[num] > 16384) ima_x[num] = 256;
	    if (ima_y[num] < 8 || ima_y[num] > 16384) ima_y[num] = 256;
	} break;

	default : 
	    pcp_error ("telop_dim #07", op->idf, "erreur d'encodage", NULL);
	    return (exec_poss = FAUX);
    } /* switch (ordre) #1 */

    /*
     * TESTE si les images IN ont la bonne dimension
    */
    if (data->nb > 0) {
	for (i = 1; i <= data->n1; i++)
	    exec_poss = (exec_poss && ima[i]->b.xsiz == ima_x[i] 
			 && ima[i]->b.ysiz == ima_y[i]);
	if (!exec_poss) {
	    pcp_avertis_ico ("WARNING !","Image size problem", "with this telOper", ico);
	    return FAUX;
	}
    }

    /*
     * ima_name et ima_lut des images OUT, avec test si change's dans section DIM.
    */
    for (i = data->n1+1; i <= data->n2; i++) {

	if (ima_name[i][0] == 0)
	    if (op->lgd_images[i-1][0] == 0)
		 sprintf (ima_name[i], "Out %d", i);
	    else strncpy (ima_name[i], op->lgd_images[i-1], IMAGE_CH1LEN);
	if (data->n1 > 0) {
	    sprintf (bla, "%s \\%d", ima_name[i], v[1]->index_creat);
	    strncpy (ima_name[i], bla, IMAGE_CH1LEN);
	}

	ima_lut[i] = (ima_lut[i] >= IPS_COUL)
	    ? ima_lut[i] - IPS_COUL
	    : (data->nb > 0)
	    ? glo_get_attr_lut (ima[1]->coul_tab)
	    : IPS_COUL_16C;
    }

    /*
     * ajustement du cadre, taille et palette des images, test si realloc ok
    */
    if (data->nb > 0) {

	/* images IN */
	for (i = 1; i <= data->n1; i++)
	    if (ima[i]->b.cmin != data->cmin) {
		if (!image_chg_cad (v[i], data->cmin, 
			ima[i]->b.xsiz, ima[i]->b.ysiz))
		    return FAUX;
		messa_Send (
		    MESSA_FROM_ICO, v[i],
		    MESSA_TO_ICO, v[i],
		    MESSA_RESCAN, VRAI,
		    NULL);
	    }

	/* images OUT existantes, test si realloc ok */
	for (i = data->n1+1; i <= data->nb; i++)
	    if (!image_props_direct (v[i], 1, data->cmin,
		    ima_x[i], ima_y[i], ima_lut[i], ima_name[i])) 
		return FAUX;

    } /* data->nb > 0 */

    /*
     * creation eventuelle des resultats, et positionnement des icones
    */
    if (data->nb < data->n2) {

	int pos_x = ico->xb, pos_y = ico->yb,
	    lar_x = ico->xm, lar_y = ico->ym,
	    mul_x = 1, mul_y = 0;

	/* calcul mul_y, mul_x suivant zone */
	if (data->nb != 0) {

	    int nr = data->nb;  /* numero de l'icone de reference / telOper */
	    int acote = FAUX;

	    /* on teste dans cet ordre l'intersection de l'icone v[nr] avec :
	     * telOper, zone a gauche, zone a droite, zone en haut, zone en bas
	    */
	    if (ico_inter (v[nr], ico)) {
		/* les valeurs par defaut sont ok */
	    } else if (ico_inter_zone (v[nr], ico->xb-50, ico->yb, 50, ico->ym)) {
		pos_y = v[data->nb]->yb; mul_x = 1;
	    } else if (ico_inter_zone (v[nr], ico->xb+ico->xm, ico->yb, 50, ico->ym)) {
		pos_y = v[data->nb]->yb; mul_x = -1;
	    } else if (ico_inter_zone (v[nr], ico->xb, ico->yb-50, ico->xm, 50)) {
		pos_x = v[data->nb]->xb; mul_x = 0; mul_y = 1;
	    } else if (ico_inter_zone (v[nr], ico->xb, ico->yb+ico->ym, ico->xm, 50)) {
		pos_x = v[data->nb]->xb; mul_x = 0; mul_y = -1;
	    } else acote = VRAI;

	    if (acote || data->n1 == 0) {
		mul_x = 1; mul_y = 0;
		lar_x = v[data->nb]->xm; lar_y = v[data->nb]->ym;
		pos_x = v[data->nb]->xb; pos_y = v[data->nb]->yb;
	    }

#ifdef ancienancienancien

	    int nr = (data->n1 == 0) ? data->nb : data->n1;  /* num ico de ref / ico */

	    if (v[nr]->yb + v[nr]->ym < ico->yb) mul_y = 1;
	    else if (v[nr]->yb < ico->yb + ico->ym) 
		{ mul_y = 0; pos_y = v[data->nb]->yb; }
	    else mul_y = -1;

	    if (v[nr]->xb + v[nr]->xm < ico->xb) mul_x = 1;
	    else if (v[nr]->xb < ico->xb + ico->xm) 
	    if (mul_y == 0) mul_x = 1;
	    else { mul_x = 0; pos_x = v[data->nb]->xb; }
	    else mul_x = -1;

	    if (data->n1 == 0) {
		mul_x = -mul_x; mul_y = -mul_y;
		lar_x = v[data->nb]->xm; lar_y = v[data->nb]->ym;
		pos_x = v[data->nb]->xb; pos_y = v[data->nb]->yb;
	    }
#endif
	} /* if */

	for (i = data->nb+1; i <= data->n2; i++) {

	    /* New Image */
	    v[i] = ico_creat (0, 0, IPS_IMAGE);
	    image_creat ((Image *) v[i]->obj, data->cmin, ima_x[i], ima_y[i], NULL);
	    ima[i] = (Image *) v[i]->obj;

	    /* teste si tout a bien pu etre cree */
	    if (ima[i]->tab == NULL) {
		ico_free(v[i]); v[i] = NULL; exec_poss = FAUX; break;
	    }

	    ico_tete_pz (v[i]);
	    strncpy (ima[i]->ch1, ima_name[i], IMAGE_CH1LEN);
	    image_calc_hublot (v[i]);

	    /* table des couleurs */
	    ima[i]->coul_tab = glo_get_ctab_lut (ima_lut[i]);

	    /* coordonnees */
	    if (mul_x < 0) lar_x = v[i]->xm;
	    if (mul_y < 0) lar_y = v[i]->ym;
	    pos_x += mul_x * (10 + lar_x);
	    pos_y += mul_y * (10 + lar_y);
	    lar_x = v[i]->xm; lar_y = v[i]->ym;
	    v[i]->xb = pos_x; v[i]->yb = pos_y;

	    ico_bascule_xor ();
	    v[i]->sel = VRAI; ico_index_selec (v[i]);
	    ico_douane (v[i]);
	    ico_affi_ico (v[i]);
	    ico_bascule_xor ();

	} /* for i */

    } /* data->nb < data->n2 */

    /*
     * Protection des IN/OUT jusqu'a l'appel de telop_maj_inout
    */
    for (i = 1; i <= data->n2; i++) if (v[i] != NULL)
    v[i]->lock = LOCK_CLEAR | LOCK_MODIF;

    XFlush (glo.dpy);
    return exec_poss;
}


/****************************************************************************************
 *
 ****************************************************************************************/
void telop_parbox_change_titre (op)
Telop *op;
{
    char bla[256];

    if (op->frame_parbox == 0) return;
    if (op->nom_host[0])
         sprintf (bla, "%s  [%s]", op->titre, op->nom_host);
    else sprintf (bla, "%s", op->titre);
    xv_set (op->frame_parbox, FRAME_LABEL, bla, 0);
}

/****************************************************************************************
 * FUNCTION  : telop_parbox_creat (Ico *ico);
 *
 * PURPOSE   :
 *
 * CALLED IN : ico_menu_ico_proc
 *
 ****************************************************************************************/
void   telop_parbox_creat (ico)
Ico   *ico;
{
  Telop         *op = (Telop *) ico->obj;
  Telop_parbox  *p;
  Panel          panel;
  int            mx = (int) xv_get (glo.frame_principal, XV_X) + ico->xb;
  int            my = (int) xv_get (glo.frame_principal, XV_Y) + ico->yb;
  int            i, n, n1, n2, ygap, m, k, prems = VRAI;
  char           bla1[256], bla2[256];
  Panel_item     pan_inout[TELOP_VECM], pan_image[TELOP_VECM];

  if (op->frame_parbox != 0)
  { 
    /*  Telop_parpox *p existe deja ; on le recupere
     */
    p = (Telop_parbox *) xv_get (op->frame_parbox, WIN_CLIENT_DATA);
  }
  else
  { 
    /*  Telop_parpox *p n'existe pas ; on le cree
     */

    p = (Telop_parbox *) pcp_alloc (sizeof(Telop_parbox));

    p->ico = ico; p->telop = op;

    op->frame_parbox = (Frame) xv_create (glo.frame_principal, FRAME_CMD,
	FRAME_CMD_PUSHPIN_IN,	TRUE,
        WIN_CLIENT_DATA,	p,
        FRAME_DONE_PROC,	telop_parbox_kill,
        0);
    telop_parbox_change_titre (op);

    panel = (Panel) xv_get (op->frame_parbox, FRAME_CMD_PANEL);

    /*  Images IN et OUT
     */
    ygap = (int) xv_get (panel, PANEL_ITEM_Y_GAP);
    xv_set (panel, PANEL_ITEM_Y_GAP, 6, NULL);

    n1 = op->nb_image_in;
    n2 = op->nb_image_out;
    n = n1 + n2;

    for (i = 1; i <= n; i++)
    {
      if (i <= n1)
	   sprintf (bla1, "IN image %d", i); 
      else sprintf (bla1, "OUT image %d",i); 
      sprintf (bla2, op->lgd_images[i-1]);

      xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, NULL);
      pan_inout[i] = (Panel_item) xv_create (panel, PANEL_MESSAGE,
		PANEL_LABEL_BOLD,	TRUE,
		PANEL_LABEL_STRING,	bla1, 
		0);
      if (prems) { xv_set (pan_inout[i], XV_Y, 10, NULL); prems = FAUX; }
      xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);
      pan_image[i] = (Panel_item) xv_create (panel, PANEL_MESSAGE, PANEL_LABEL_STRING, bla2, 0);
   }

    /*  Allignement images IN et OUT
     */
    n1 = 0;
    for (i = 1; i <= n; i++)
    { n2 = (int) xv_get (pan_inout[i], XV_WIDTH);
      if (n2 > n1) n1 = n2;
    }
    if (n > 0) n1 += xv_get (pan_inout[1], XV_X);
    for (i = 1; i <= n; i++) 
    xv_set (pan_inout[i], XV_X, n1 - xv_get (pan_inout[i], XV_WIDTH), NULL);

    n1 = 0;
    for (i = 1; i <= n; i++)
    { n2 = (int) xv_get (pan_image[i], XV_X);
      if (n2 > n1) n1 = n2;
    }
    for (i = 1; i <= n; i++) xv_set (pan_image[i], XV_X, n1, NULL);

    xv_set (panel, PANEL_LAYOUT, PANEL_VERTICAL, PANEL_ITEM_Y_GAP, ygap-2, NULL);

    /*  Parametres
     */
    if (op->param_nb > 0) 
         p->adr = (int *) pcp_alloc (op->param_nb * sizeof(int));
    else p->adr = NULL;

    for (i = 0; i < op->param_nb; i++) {
      switch (op->param_type[i]) {

      case IPS_PAR_INT : {
          Telop_par_int *para = (Telop_par_int *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_NUMERIC_TEXT,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_MIN_VALUE,	para->inf,
		PANEL_MAX_VALUE,	para->sup,
		0);
        } break;

      case IPS_PAR_SLIDER : {
          Telop_par_int *para = (Telop_par_int *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_SLIDER,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_MIN_VALUE,	para->inf,
		PANEL_MAX_VALUE,	para->sup,
		PANEL_SLIDER_WIDTH,	100,
		0);
        } break;

      case IPS_PAR_CHK_BOX : {
          Telop_par_int *para = (Telop_par_int *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_CHECK_BOX,
		PANEL_LABEL_STRING,	para->legende,
		/*PANEL_CHOICE_STRINGS,	para->legende, NULL,*/
		PANEL_NOTIFY_PROC,	telop_parbox_par_choice_proc,
		XV_KEY_DATA,		101, p,
		XV_KEY_DATA,		102, i+1,
		0);
        } break;

      case IPS_PAR_REAL : {
          Telop_par_real *para = (Telop_par_real *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_TEXT,
		PANEL_LABEL_STRING,		para->legende,
		PANEL_VALUE_DISPLAY_LENGTH,	15,
		PANEL_VALUE_STORED_LENGTH,	30,
		0);
        } break;

      case IPS_PAR_TEXT : {
          Telop_par_text *para = (Telop_par_text *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_TEXT,
		PANEL_LABEL_STRING,		para->legende,
		PANEL_VALUE_DISPLAY_LENGTH,	25,
		PANEL_VALUE_STORED_LENGTH,	para->taille_max,
		0);
        } break;

      case IPS_PAR_CHOICE : {
          Telop_par_choice *para = (Telop_par_choice *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_CHOICE,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_LAYOUT,		PANEL_HORIZONTAL,
		PANEL_NOTIFY_PROC,	telop_parbox_par_choice_proc,
		XV_KEY_DATA,		101, p,
		XV_KEY_DATA,		102, i+1,
		0);
          for (k = 0; k < para->nchoice; k++)
          xv_set (p->adr[i], PANEL_CHOICE_STRING, k, para->choices[k], NULL);
        } break;

       case IPS_PAR_CH_ABR : {
          Telop_par_choice *para = (Telop_par_choice *) op->param_adr[i];

          p->adr[i] = (int) xv_create (panel, PANEL_CHOICE_STACK,
		PANEL_LABEL_STRING,	para->legende,
		PANEL_LAYOUT,		PANEL_HORIZONTAL,
		PANEL_NOTIFY_PROC,	telop_parbox_par_choice_proc,
		XV_KEY_DATA,		101, p,
		XV_KEY_DATA,		102, i+1,
		0);
          for (k = 0; k < para->nchoice; k++)
          xv_set (p->adr[i], PANEL_CHOICE_STRING, k, para->choices[k], NULL);
        } break;

      default : p->adr[i] = 0; /* tres important */
      } /* switch */

      if (prems && p->adr[i] != 0)
	    { xv_set (p->adr[i], XV_Y, 10, NULL); prems = FAUX; }
    } /* for */

    /*  Allignement parametres
     *
    */
    n1 = 0;
    for (i = 0; i < op->param_nb; i++) if (p->adr[i])
    { n2 = (int) xv_get ((Panel_item) p->adr[i], PANEL_VALUE_X);
      if (n2 > n1) n1 = n2;
    }
    for (i = 0; i < op->param_nb; i++) if (p->adr[i])
    xv_set ((Panel_item) p->adr[i], PANEL_VALUE_X, n1, NULL);


    /*  Bouton Figure window
     */
    if (op->fig != NULL) {
	(void) xv_create (panel, PANEL_BUTTON,   
	    PANEL_LABEL_STRING, "Show Figure Windows",
	    PANEL_NOTIFY_PROC,  telop_parbox_butt_panel_proc,
	    PANEL_CLIENT_DATA,  p,
	    0);
    } 


    /*  Creation des boutons Help, Run, etc
     */

      (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,    "Help",
	PANEL_ITEM_MENU,       (Menu) xv_create (0, MENU,
		MENU_STRINGS,		"Help", "Edit help", NULL,
		MENU_NOTIFY_PROC,	telop_parbox_butt_menu_proc,
		MENU_CLIENT_DATA,	p,
		0),
	0);

    xv_set (panel, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);

    if (op->param_nb > 0)
    {

      (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"Apply & Run",
	PANEL_NOTIFY_PROC,       telop_parbox_butt_panel_proc,
	PANEL_CLIENT_DATA,	p,
	0);

      (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,    "Apply",
	PANEL_ITEM_MENU,       (Menu) xv_create (0, MENU,
		MENU_STRINGS,		"Apply", "Cancel", "Default", NULL,
		MENU_NOTIFY_PROC,	telop_parbox_butt_menu_proc,
		MENU_CLIENT_DATA,	p,
		0),
	0);
    }
    else (void) xv_create (panel, PANEL_BUTTON,   
	PANEL_LABEL_STRING,	"Run",
	PANEL_NOTIFY_PROC,      telop_parbox_butt_panel_proc,
	PANEL_CLIENT_DATA,	p,
	0);


    window_fit (panel);
    window_fit (op->frame_parbox);
  }

  /*  Affectation des champs PANEL_VALUE des parametres
   *  - Probleme : si affi parbox avant fin DEF, re-affi recupere p
   *  et arrive directement ici --> crash !
   *  - Solution : appeler telop_parbox_kill a la fin de decode_def si p existe.
   */
  telop_parbox_butt_direct (p, 3);  /* 3 = bouton Cancel */

  /*  Affichage de parbox
   */
  xv_set (op->frame_parbox, XV_X, mx, XV_Y, my, 
	XV_SHOW, TRUE, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
}


/****************************************************************************************
 * FUNCTION  :
 *
 * PURPOSE   :
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
int         telop_parbox_butt_panel_proc (item, event)
Panel_item  item;
Event      *event;

{
  char          *choice = (char*)         xv_get (item, PANEL_LABEL_STRING);
  Telop_parbox  *p      = (Telop_parbox*) xv_get (item, PANEL_CLIENT_DATA);
  int            value;

  if      (!strcmp(choice, "Run"         )) value = 4;
  else if (!strcmp(choice, "Apply & Run" )) value = 5;
  else if (!strcmp(choice, "Show Figure Windows" )) value = 6;
  else value = 4;

# ifdef DEBUG
  printf ("telop_parbox_butt_panel_proc %s %d\n", choice, value);
# endif

  if (value == 4 || value == 5 || value == 6) if (p->ico->lock & LOCK_RUN)
      return XV_OK;

  telop_parbox_butt_direct (p, value);

  return XV_OK;
}

/****************************************************************************************
 * FUNCTION  :
 *
 * PURPOSE   :
 *
 * CALLED IN : 
 *
 ****************************************************************************************/
void        telop_parbox_butt_menu_proc (menu, menu_item)
Menu menu;
Menu_item menu_item;

{
  char          *choice = (char*)         xv_get (menu_item, MENU_STRING);
  Telop_parbox  *p      = (Telop_parbox*) xv_get (menu, MENU_CLIENT_DATA);
  int            value;

  if      (!strcmp(choice, "Apply"           )) value = 1;
  else if (!strcmp(choice, "Default"         )) value = 2;
  else if (!strcmp(choice, "Cancel"          )) value = 3;
  else if (!strcmp(choice, "Help"            )) value = 11;
  else if (!strcmp(choice, "Edit help"       )) value = 12;
  else value = 1;

# ifdef DEBUG
  printf ("telop_parbox_butt_menu_proc %s %d\n", choice, value);
# endif

  telop_parbox_butt_direct (p, value);
}

/****************************************************************************************
 * Callback des boutons Apply, Default, Cancel, Run, Help de la parbox.
 *
 * Attention : on suppose que op->frame_parbox EXISTE.
 ****************************************************************************************/
void           telop_parbox_butt_direct (p, value)
Telop_parbox  *p;
int            value;
{
  Telop  *op = p->telop;
  int     i;
  char    bla1[256];
  int     lancer = 0, fermer = FAUX;

  struct { int num, val; } li_cho[OPER_VECM];
  int nb_cho = 0;

  switch(value)
  {
    case 1 : /* Apply */
        /* fermer = VRAI; */
	 break;
    case 2 : /* Default */
        break;
    case 3 : /* Cancel */
        break;
    case 4 : /* Run */
	lancer = 1;
        break; 
    case 5 : /* Apply & Run */
	value = 1; lancer = 1;
        break; 
    case 6 : /* Show Figure Windows */
	value = 1; lancer = 2;
        break; 
    case 11 : /* Help */
	aide_affi_text (op->idf, op->nom_aide, "B_HELP", "E_HELP", 
	    "Help on Operator ", op->idf);
        return;
    case 12 : /* Edit help */
	lmt_edit_file (op->nom_aide, "B_HELP", op->idf, " %s %s");
        return;
  }

  /* 
   * Default/Apply/Cancel des parametres dans la boite
   */
  for (i = 0; i < op->param_nb; i++)
  switch (op->param_type[i])
  {
    case IPS_PAR_CHK_BOX :
    case IPS_PAR_SLIDER :
    case IPS_PAR_INT : {
        Telop_par_int *para = (Telop_par_int *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->defaut, NULL);
              if (op->param_type[i] == IPS_PAR_CHK_BOX)
		{ li_cho[nb_cho].num = i+1; li_cho[nb_cho++].val = para->defaut; }
              /* no break; */
          case 1 : /* Apply */
              para->valeur = (int) xv_get (p->adr[i], PANEL_VALUE);
              break;
          case 3 : /* Cancel */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->valeur, NULL);
              if (op->param_type[i] == IPS_PAR_CHK_BOX)
		{ li_cho[nb_cho].num = i+1; li_cho[nb_cho++].val = para->valeur; }
              break;
        }
      } break;

    case IPS_PAR_REAL : {
        Telop_par_real *para = (Telop_par_real *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              sprintf (bla1, "%f", para->defaut);
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, bla1, NULL);
              /* no break; */
          case 1 : /* Apply */
              para->valeur = atof ((char*) xv_get (p->adr[i], PANEL_VALUE));
              if (para->valeur > para->sup) 
                   para->valeur = para->sup;
              else if (para->valeur < para->inf) 
                   para->valeur = para->inf;
              else break;
          case 3 : /* Cancel */
              sprintf (bla1, "%f", para->valeur);
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, bla1, NULL);
              break;
        }
      } break;

    case IPS_PAR_TEXT : {
        Telop_par_text *para = (Telop_par_text *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->defaut, NULL);
              /* no break; */
          case 1 : /* Apply */
              strcpy (para->valeur, (char*) xv_get (p->adr[i], PANEL_VALUE));
              break;
          case 3 : /* Cancel */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->valeur, NULL);
              break;
        }
      } break;

    case IPS_PAR_CH_ABR :
    case IPS_PAR_CHOICE : {
        Telop_par_choice *para = (Telop_par_choice *) op->param_adr[i];

        switch(value)
        {
          case 2 : /* Default */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->defaut, NULL);
	      li_cho[nb_cho].num = i+1; li_cho[nb_cho++].val = para->defaut;
              /* no break; */
          case 1 : /* Apply */
              para->valeur = (int) xv_get (p->adr[i], PANEL_VALUE);
              break;
          case 3 : /* Cancel */
              xv_set ((Panel_item) p->adr[i], PANEL_VALUE, para->valeur, NULL);
	      li_cho[nb_cho].num = i+1; li_cho[nb_cho++].val = para->valeur;
              break;
        }
      } break;

  }

  /* Requete MULTI ACT_CHOICE (uniquement lorsque lancer == 0, sinon on enchainerait
   * 2 appele a cs_requete, ce qui est interdit).
   * Pour chaque param de type choice, une sequence "num val" est ajoutee a 
   * la requete (pour Default et Cancel ; jamais pour Apply --> jamais quand lancer > 0).
   * Le champs init est mis a 1 ; on s'en sert dans l'op ChamferMask.
   * Format requete : "ordre idf is_init nb_cho   num val num val ..."
  */
  if (nb_cho > 0) { /* alors lancer == 0 */
	char requete[2048], *d = requete;

	sprintf (d, "200 %s 1 %d", op->idf, nb_cho); /* CHO_ACT */
	d += strlen (d);
	for (i = 0; i < nb_cho; i++) {
	    sprintf (d, " %d %d", li_cho[i].num, li_cho[i].val);
	    d += strlen (d);
	}
	cs_requete (p->ico, requete);
  }

  if (fermer) xv_set (op->frame_parbox,
	FRAME_CMD_PUSHPIN_IN, FALSE, XV_SHOW, FALSE, NULL);

  switch (lancer) {
      case 1 : telop_check_run (p->ico, VRAI); break;
      case 2 : fig_show_windows (p->ico); break;
  }
}

/****************************************************************************************
 * FUNCTION  : telop_parbox_par_choice_proc ()
 *
 * PURPOSE   :
 *
 ****************************************************************************************/
void        telop_parbox_par_choice_proc (item, value, event)
Panel_item  item;
int         value;
Event      *event;
{
  Telop_parbox  *p  = (Telop_parbox *) xv_get (item, XV_KEY_DATA, 101);
  int            n  = (int)            xv_get (item, XV_KEY_DATA, 102);
  Telop         *op = p->telop;
  char		requete[256];

# ifdef DEBUG
    printf ("telop_parbox_par_choice_proc  param %d  value %d\n", n, value);
# endif
  
  /* Format requete : "ordre idf is_init nb_cho num val" */
  sprintf (requete, "200 %s 0 1 %d %d", op->idf, n, value); /* CHO_ACT */
  cs_requete (p->ico, requete);
}

/****************************************************************************************
 * FUNCTION  : telop_parbox_kill (Frame frame);
 *
 * PURPOSE   : FRAME_DONE_PROC
 *
 ****************************************************************************************/
void  telop_parbox_kill (frame)
Frame frame;
{
  Telop_parbox *p = (Telop_parbox *) xv_get (frame, WIN_CLIENT_DATA);

  if (xv_destroy_safe(frame) != XV_OK) pcp_error ("telop_parbox_kill #01", NULL);

  p->telop->frame_parbox = 0;
  if (p->adr != NULL) pcp_free (p->adr);
  pcp_free(p);
}

/****************************************************************************************
 * FUNCTION  : telop_attach (Ico *ico);
 *
 * PURPOSE   : attache une selection au telop
 *
 ****************************************************************************************/
int  telop_attach (ico)
Ico  *ico;
{
  Telop *op = (Telop *) ico->obj;
  Ico *p[TELOP_VECM], *cur;
  int n, i;
  int n1 = op->nb_image_in;
  int n2 = op->nb_image_in + op->nb_image_out;
  int test = VRAI;

  if (op->attach_n > 0 || op->attach_v != NULL)
      pcp_error ("telop_attach #01", NULL);

  /* memorisation des ico selectes, tri et tests */
  n = 0;
  cur = glo.l_ico_a;
  while (cur != NULL) {
      if (cur->sel) 
          if (IPS_IS_DATA(cur->nat_obj)) {
              ico_tri_selec (cur, p, 0, n++);  /* tri par insertion */
              if (n > n2) break;
          }
          else if (cur->index_creat != ico->index_creat) {
	      pcp_avertis_ico ("WARNING !","Only datas",
	          "must be attached", cur);
              test = FAUX;
          }
      cur = cur->pa;
  }

  if (!test) return FAUX;

  if (n < n1 || n > n2) {
    char bla[100];
    sprintf (bla, "%d in and %d out\n", op->nb_image_in, op->nb_image_out);
    pcp_avertis_ico ("WARNING !","Number of datas needed by this telop :",bla, ico);
    return FAUX;
  }

  if (n > 0) {
    op->attach_v = (int *) pcp_alloc (n * sizeof(int));
    if (op->attach_v == NULL) return;
    for (i = 0; i < n; i++) op->attach_v[i] = p[i]->index_creat;
  }
  op->attach_n = n;
  ico_init_raf ();
  ico->sel = FAUX;
  ico->raf = VRAI;
  for (i = 0; i < n; i++) { /* on deselectionne ce qui est attache */
	p[i]->sel = FAUX; p[i]->raf = VRAI;
  }
  ico_inter_crible ();
  ico_affi_raf ();

  return VRAI;
}

/****************************************************************************************
 * FUNCTION  : telop_detach (Ico *ico);
 *
 * PURPOSE   : detache une selection au telop
 *
 ****************************************************************************************/
void  telop_detach (ico)
Ico  *ico;
{
    Telop *op = (Telop *) ico->obj;
    int i;
    Ico *cur;

    if (op->attach_n == 0 || op->attach_v == NULL)
	pcp_error ("telop_detach #01", NULL);

    else { 
	/* on deselectionne tout le monde et init ->raf */
	for (cur = glo.l_ico_a; cur != NULL; cur = cur->pa) {
	    cur->raf = cur->sel; cur->sel = FAUX;
	}
	/* on reselectionne ce qui etait attache */
	for (i = 0; i < op->attach_n; i++) {
	    cur = ico_get_from_index_creat (op->attach_v[i]);
	    if (cur != (Ico *) NULL) {
		cur->sel = cur->raf = VRAI;
		ico_index_selec (cur);
	    }
	}
    }
    pcp_free (op->attach_v);
    op->attach_v = NULL;
    op->attach_n = 0;
    ico->sel = FAUX;
    ico->raf = VRAI;
    ico_inter_crible ();
    ico_affi_raf ();
}


/****************************************************************************************
 * Activation du choix dans le menu "Telop" du frame_principal
 *
 * - lecture du nom du bouton et de l'idf du telop selectionne
 * - creation complete de l'IPS_TELOP correspondant
 *
 ****************************************************************************************/
void telop_MenuHop (m_idf, m_fic, m_help, m_hlist)
char *m_idf, *m_fic, *m_help, *m_hlist;
{
    Ico   *ico;
    Telop *ob;
    char  requete[256];

#   ifdef DEBUG
    printf ("telop_MenuHop %s  %s\n", m_idf, m_fic);
#   endif

    /*
     * New telop
    */
    ico = ico_creat (glo.sx, glo.sy, IPS_TELOP); 
    glo.sx += 10; glo.sy += 16;
    ico_tete_pz (ico);
    ob = (Telop *) ico->obj;
    telop_init (ob, m_idf, m_fic, m_help, ho_get_host(m_hlist));
    ob->titre = stg_Cpyf ("Waiting DEF on %s", 
	(ob->nom_host[0]) ? ob->nom_host : glo.host_name);
    telop_prepare_noms (ico);
    ico_douane (ico);
    ico_refresh_now (ico);

    /* cette protection est ensuite enlevee dans telop_decode_def */
    ico->lock = LOCK_DUPLIC | LOCK_RUN; 

    /*
     * Appel du tel_oper
    */
    sprintf (requete, "100 %s", m_idf); /* DEF_ACT */
    cs_requete (ico, requete);
}

/****************************************************************************************
 * FUNCTION  : telop_CloseAllWin (Ico *ico);
 *
 * CALLED IN : method_free
 *
 ****************************************************************************************/
void telop_CloseAllWin (ico)
Ico *ico;
{
    Telop *op = (Telop *) ico->obj;
  
    /* Boite de parametres */
    if (op->frame_parbox != 0)
	telop_parbox_kill (op->frame_parbox);

    /* message KILL aux abonnes de ico */
    messa_Send (
	MESSA_FROM_ICO, ico, /* ico ne le recevra pas */
	MESSA_TO_ICO, ico,
	MESSA_KILL, VRAI,
	NULL);
}


/****************************************************************************************
 * EFFET : - Unlock des icones IN/OUT, 
 *         - Mise a jour des icones OUT, 
 *         - Free de leur liste.
 *
 * ATTENTION : - led_pourcent doit etre != LED_SLEEP
 *             - le ctx doit encore exister pour recuperer data !
 ***************************************************************************************/
void telop_maj_inout (ico)
Ico *ico;
{
    Telop *op = (Telop *) ico->obj;
    Telop_data_cs *data;
    Ico **v;
    int i;

    if (op->led_pourcent == LED_SLEEP) return;

    data = (Telop_data_cs *) cs_get_data2 (op->ctx);
    if (data == (Telop_data_cs *) NULL) return;
    v = data->v;

    /*
     * Unlock icones IN et OUT
    */
    for (i = 1; i <= data->n2; i++)  if (v[i] != NULL)
	v[i]->lock = LOCK_NO;

    /*
     * affichage des resultats OUT.
    */
    for (i = data->n1 + 1; i <= data->n2; i++) if (v[i] != NULL) {
	v[i]->raf = VRAI; method_obsolete (v[i]);
    }
    ico_inter_crible ();
    ico_bascule_xor ();
    ico_affi_raf ();
    ico_bascule_xor ();
    /* Envoi du refresh general : aux OUT, ou aux IN+OUT si il y a des figures */
    for (i = (op->fig == NULL) ? data->n1 + 1 : 1; i <= data->n2; i++) if (v[i] != NULL)
	messa_Send (
	    MESSA_FROM_ICO, v[i],
	    MESSA_TO_ICO, v[i],
	    MESSA_REFRESH, VRAI,
	    NULL);

    /*
     * Libere data
    */
    pcp_free (data);
    cs_set_data2 (op->ctx, NULL);
}

/****************************************************************************************
 * Change la led de l'icone : LED_SLEEP, LED_WAIT, LED_RUN, ..
 *
 ****************************************************************************************/
void telop_change_led (ico, etat)
Ico *ico;
int etat;
{
    Telop *op = (Telop *) ico->obj;

    /* Icone */
    op->led_pourcent = etat; 
    ico_refresh_now (ico);

    /* Fenetre de parametres */
    /* if (op->frame_parbox != 0)
	xv_set (op->frame_parbox, FRAME_BUSY, (etat == 0) ? 0 : 1, 0); */
}

/****************************************************************************************
 * Stoppe le run en cours en tuant le process, et met tout a jour.
 *
 ****************************************************************************************/
void telop_stop (ico)
Ico *ico;
{
    Telop *op = (Telop *) ico->obj;

    if (op->led_pourcent == LED_SLEEP) return;

    cs_footer (op->ctx, "Interrupted");
    telop_maj_inout (ico); /* appele' avant de changer led ou ctx */
    if (op->ctx != -1) cs_kill_ctx (op->ctx);
    telop_change_led (ico, LED_SLEEP);
}


