/***************************************************************************************/
/* stg.c                                                      I.P.S  v 3.2  19/02/1994 */
/*                                                                                     */
/* module des chaines de caracteres                      Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"


/***************************************************************************************
 * Malloc et copy de string
 *
 ***************************************************************************************/
char *stg_Cpy (nom)
char *nom;
{
    char *c;

    if (nom == NULL) return NULL;

    c = (char *) pcp_alloc (strlen(nom)+1);
    strcpy (c, nom);
    return c;
}


/***************************************************************************************
 * Malloc et copy de string ; s'emploie comme un printf
 *
 ***************************************************************************************/
char *stg_Cpyf (char *format, ...)
{
    va_list ap;
    char tmp[1024];

    va_start (ap, format);
    vsprintf (tmp, format, ap);
    va_end (ap);

    return stg_Cpy(tmp);
}


/***************************************************************************************
 * Recoit le chemin complet n1 et le scinde en un chemin n2 et un nom de fichier n3
 *
 ***************************************************************************************/
void stg_ScindeChemNomFi (n1, n2, n3)
char *n1, *n2, *n3;
{
    int i, j;

    /* initialisation a des chaines vides */
    n2[0] =  n3[0] = 0;

    if (n1[0] != 0) {   /* la chaine n'est pas vide */

	/* recherche du dernier '/' */
	i = 0; j = 0;
	while (n1[i] != 0) {
	    if (n1[i] == '/') j = i;
	    i++;
	}

	/* recopie dans n2 et n3 */
	for (i = 0; i < j; i++) n2[i] = n1[i]; 
	n2[j] = 0; if (n1[j] == '/') j++; i = 0;
	while (n1[j] != 0) { n3[i] = n1[j]; j++; i++; } 
	n3[i] = 0;

#       ifdef DEBUG
	printf ("stg_ScindeChemNomFi `%s'\n", n1);
	printf ("n2 = `%s'\nn3 = `%s'\n", n2, n3); 
#       endif
    }
}


/***************************************************************************************
 * Recoit un chemin n1 et un nom de fichier n2, et le regroupe en un ch. complet n3
 *
 ***************************************************************************************/
void stg_RegroupChemNomFi (n1, n2, n3)
char *n1, *n2, *n3;
{
    if (n2[0] == '/') strcpy (n3, n2);
    else sprintf (n3, "%s/%s", n1, n2);

#   ifdef DEBUG
    printf ("stg_RegroupChemNomFi `%s' + `%s'\n", n1, n2);
    printf ("result = `%s'\n", n3);
#   endif
}


/***************************************************************************************
 * Recoit un nom de fichier n1, et renvoie la partie avant le '.' dans n2
 *
 ***************************************************************************************/
void stg_DebNomFi (n1, n2)
char *n1, *n2;
{ 
    int i = 0;
    while (n1[i] != 0 && n1[i] != '.') {n2[i] = n1[i]; i++; }
    n2[i] = 0;
}

/***************************************************************************************
 * Recoit un buffer et une position offset, recherche fin de ligne a partir de offset
 * Renvoie caractere de fin de ligne '\n' ou NULL, et met offset au caractere suivant.
 *
 ***************************************************************************************/
char stg_NextLine (buffer, offset)
char *buffer;
int  *offset;
{
    while (buffer[*offset] != '\n' && buffer[*offset] != 0) (*offset)++;
    return buffer[(*offset)++];
}


/***************************************************************************************
 * Recherche des mots, met 1ere lettre en majuscule et suppr espaces
 *
 * EX: 'salut jeSuis  ici' --> 'SalutJeSuisIci'
 ***************************************************************************************/
void stg_FirstCap (ligne, resu)
char *ligne, *resu;
{
    int	dedans = FALSE, c = 0, i;

    for (i = 0; ligne[i] && i < 1024; i++)
	if (dedans) {
	    if (isspace(ligne[i])) dedans = FAUX;
	    else resu[c++] = ligne[i]; 
	}
	else if (!isspace(ligne[i])) {
	    dedans = VRAI;
	    resu[c++] = toupper(ligne[i]);
	}
    resu[c] = 0;
}

/***************************************************************************************
 * Recherche des mots, normalise espaces (un '|' vaut un espace)
 *
 * EX: ' salut je	suis  ici ' --> 'Salut je suis ici'
 ***************************************************************************************/
void stg_NormSpace (ligne, resu)
char *ligne, *resu;
{
    int	dedans = FALSE, c = 0, i;

    for (i = 0; ligne[i] && i < 1024; i++)
	if (dedans) {
	    if (isspace(ligne[i]) || ligne[i] == '|') 
		 dedans = FAUX;
	    else resu[c++] = ligne[i]; 
	}
	else if (!isspace(ligne[i]) && ligne[i] != '|') {
	    dedans = VRAI;
	    if (c > 0) resu[c++] = ' ';
	    resu[c++] = ligne[i];
	}
    resu[c] = 0;
}

/***************************************************************************************
 * Renvoie la taille du premier mot (prov. de stg_NormSpace)
 *
 * EX: 'Salut je suis ici' --> 5
 ***************************************************************************************/
int stg_WordLen (ligne)
char *ligne;
{
    int i = 0;

    while ( ligne[i] && i < 1024 && !isspace(ligne[i]) ) i++;

    return i;
}

/***************************************************************************************
 * Renvoie la taille du + grand bloc (prov. de Stg_NormSpace) contigu <= lar
 *
 * EX: 'Salut je suis ici',9 --> 8
 ***************************************************************************************/
int stg_BiggestLen (ligne, lar)
char *ligne;
int lar;
{
    int i, l = strlen (ligne);

    if (l <= lar) return l;
    for (i = lar; i > 0; i--) if (isspace(ligne[i])) break;
    return i;
}

/***************************************************************************************
 * ENTREE  : ligne prov. de Stg_NormSpace
 * RENVOIE : largeur des sous-titres, nb de sous_titres
 ***************************************************************************************/
void stg_RepartiMots (ligne, lar, nb)
char *ligne;
int *lar, *nb;
{
    int	mot[100], m, n, l, c, k;

    m = c = 0;
    l = strlen(ligne);
    *lar = 7; /* taille min */
    
    /* Longueur des mots */
    while (c < l && (mot[m] = stg_WordLen(ligne + c)) != 0) {
	if (mot[m] > *lar) *lar = mot[m];
	c += mot[m++] + 1;
    }

    /* Calcul de *lar tel que *nb <= 5 */
    do {
	*nb = 1; k = -1;
	for (n = 0; n < m; n++) {
	    if (k + 1 + mot[n] <= *lar)
		k += 1 + mot[n];
	    else { *nb += 1; k = mot[n]; } 
	}
	if (*nb > 5) *lar += 1;
    } while (*nb > 5);

}

/***************************************************************************************
 * Affiche la ligne centre'e avec largeur "lar" en nb de caracteres.
 *
 * nouv_yc = stg_PrintCenter ();
 ***************************************************************************************/
int stg_PrintCenter (xc, yc, lar, ligne)
int xc, yc, lar;
char *ligne;
{
    int i, c, l, x;

    XSetForeground (glo.dpy, glo.gc, glo.c[6]);

    i = 0; l = strlen(ligne);
    while ( i < l && (c = stg_BiggestLen (ligne+i, lar)) != 0) {
	x = xc - (c * glo.font_l) / 2; /* centre' */
	yc += glo.font_h + 2;
	XDrawString (glo.dpy, glo.xwin, glo.gc, x, yc, ligne+i, c);
	i += c+1;
    }

    return yc;
}

/***************************************************************************************
 * Affiche la ligne justifie'e a gauche avec largeur "xm" en nb de pixels.
 *
 * Attention yc est la coord du point en bas a gauche du mot
 ***************************************************************************************/
void stg_PrintLeft (xc, yc, xm, ligne)
int xc, yc, xm;
char *ligne;
{
    int l, m;

    XSetForeground (glo.dpy, glo.gc, glo.c[6]);

    l = strlen (ligne);
    m = xm / glo.font_l;
    if (l > m) l = m;

    XDrawString (glo.dpy, glo.xwin, glo.gc, xc, yc, ligne, l); 
}

/****************************************************************************************
 * Ecriture dans un buffer ; utilise'par *_buf_encode_* ()
 * Les strings sont range's a la queue leu leu, separes par des '\0'
 *
 * APPEL :
 *	stg_buf_wr_str (&p, "salut");
 *	stg_buf_wr_int (&p, 1234);
 *	stg_buf_wr_dbl (&p, 3.14);
 *	stg_buf_wr_dblv (&p, "%.2f", 3.14);
 *
 * RQ : sur Linux, sprintf return strlen ; sur Sun, sprintf return s !!
 ****************************************************************************************/
void stg_buf_wr_str (p, s)
char **p, *s;
{
    sprintf (*p, "%s", s);
    *p += 1 + strlen (*p);
}

void stg_buf_wr_int (p, d)
char **p;
int d;
{
    sprintf (*p, "%d", d);
    *p += 1 + strlen (*p);
}

void stg_buf_wr_dbl (p, f)
char **p;
double f;
{
    sprintf (*p, "%f", f);
    *p += 1 + strlen (*p);
}

void stg_buf_wr_dblv (p, u, f)
char **p, *u;
double f;
{
    sprintf (*p, u, f);
    *p += 1 + strlen (*p);
}

/****************************************************************************************
 * Lecture dans un buffer ; utilise' dans *_buf_decode_* ().
 * Les strings sont range's a la queue leu leu, separes par des '\0'
 *
 * APPEL :
 *    char   *ligne = stg_buf_rd (&p);
 *    int     x = atoi (stg_buf_rd (&p));
 *    double  y = atof (stg_buf_rd (&p));
 *
 ****************************************************************************************/
char *stg_buf_rd (p)
char **p;
{
    char *q = *p;
    *p += 1 + strlen (*p);
    return q;
}


/****************************************************************************************
 * Affichage d'un buffer
 * Les strings sont range's a la queue leu leu, separes par des '\0'
 * Les '\0' sont repre'sente's par des '~'
 *
 ****************************************************************************************/
void stg_buf_affi (buf, bufsize)
char *buf;
int bufsize;
{
    int i;

    printf ("Taille = %d\n", bufsize);
    printf ("DEBUT------------\n");
    { int i; for (i = 0; i < bufsize; i++) 
	if (buf[i] == '\0') putchar ('~');
	else putchar (buf[i]); }
    printf ("\nFIN--------------\n");
}

/****************************************************************************************
 * Concatenation de deux chemins
 *
 * Exemple :	s1 = /baba/bob/gloub/file1   s2 = ../truc/hop/file2
 *	    -->	s3 = /baba/bob/truc/hop/file2
 *
 ****************************************************************************************/
void stg_concat_2files (s1, s2, s3)
char *s1, *s2, *s3;
{
    int i, m1;

    if (s2[0] == 0)
	s3[0] = 0; /* s2 est vide */
    else if (s2[0] == '/')
	memcpy (s3, s2, strlen(s2)+1); /* s2 est absolu */
    else {

	/* suppr nom fichier dans s1 */    
	for (i = strlen(s1)-1; i >= 0; i--) if (s1[i] == '/') break;
	m1 = i+1;

	/* copie de s1+s2 dans s3 */
	memcpy (s3, s1, m1);
	memcpy (s3+m1, s2, strlen(s2)+1); /* avec le 0 final */
    }

    /* simplification des "../" */
    while (stg_simplify_path (s3));
}

/****************************************************************************************
 * Simplification d'un chemin
 *
 * Exemple :	s = /toto/baba/bob/../toz/../../gloub/file1
 *	    -->	s = /toto/gloub/file1
 *
 ****************************************************************************************/
int stg_simplify_path (s)
char *s;
{
    int i, k, j, n;

    if (s[1] == 0) return FAUX;

    n = strlen (s);
    for (i = 1; i < n; i++)
	if (s[i] == '.' && s[i+1] == '.' && s[i+2] == '/') {
	    for (k = i-2; k >= 0; k--) if (s[k] == '/') break;
	    for (j = i+3; j <= n; j++) s[j+k-i-2] = s[j];
	    return VRAI;	    
	}
    return FAUX;
}



