/***************************************************************************************/
/* tool_itml.c                                                I.P.S  v 3.5  19/06/1995 */
/*                                                                                     */
/* Interpreteur ITML (IPS Tool Meta Langage)             Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include "inclus.h"

/* #define DEBUG */

#define LG_LIGNE	256
#define LG_MOT		256
#define LG_GROS		16384


/*---------------------------------------------------------------------------------------
 *		Definitions pour les lexemes
 *-------------------------------------------------------------------------------------*/

enum LexemeType { Lx_rien, Lx_string, Lx_mot, Lx_num, Lx_ponct };


typedef struct {
	int	p,		/* position du debut du lexeme dans le source */
		n,		/* taille du lexeme */
		type;		/* type du lexeme */
	} LexInfo;


#define EST_ALPHA(c)	(('a'<=c && c<='z') || ('A'<=c && c<='Z') || c == '_')
#define EST_NUM(c)	('0'<=c && c<='9')

/*
 * couples :  ++ -- << >> == && || != += -= <= >= 
*/
#define EST_PONCT2(c,d)	((strchr ("+-<>=&|", c) && d == c) || \
			 (strchr ("!+-<>", c) && d == '=') )

/*
 * simplex :  ! - + * / % < > & ^ | ? : = , ( ) ; { } [ ]
*/
#define EST_PONCT1(c)	(strchr ("!-+*/%<>&^|?:=,();{}[]", c))


/*---------------------------------------------------------------------------------------
 *				Definition des tokens
 *-------------------------------------------------------------------------------------*/

enum TokenType {			/* -------------------------- GRAMMAIRE */
	Tk_null,			/* null					*/
	Tk_corp,			/* DEFGRAM : corps du programme		*/
	Tk_proc,			/* DEFGRAM : fonction			*/
	Tk_bloc,			/* DEFGRAM : bloc			*/
	Tk_expr,			/* DEFGRAM : expression			*/
	MAX_DEFGRAM,			/* placer tous les DEFGRAM avant ici	*/
	Tk_dup1,			/* le dernier : duplicable 		*/
	Tk_dup2,			/* 2 derniers : idem			*/
					/* ----------------------- PONCTUATIONS */
	Tk_accG, Tk_accD,		/* accolades { }			*/
	Tk_parG, Tk_parD,		/* parentheses ( )			*/
	Tk_croG, Tk_croD,		/* crochets [ ]				*/
	Tk_virg, Tk_ptvi,		/* virgule ,  point-virgule ;		*/
	Tk_afec,			/* affectation = += -=			*/
	Tk_unai, Tk_bina,		/* op unaire, op binaire		*/
	Tk_u_b,				/* op unaire ou binaire	+ -		*/
	Tk_ter1, Tk_ter2,		/* ops ternaires ? :			*/
					/* -------------------------- MOTS CLES */
	Tk_sub ,			/* fonction sub				*/
	Tk_if  , Tk_else,		/* test if else				*/
	Tk_whil, Tk_do  ,		/* boucle while do			*/
	Tk_for ,			/* boucle for				*/
					/* ----------------------------- AUTRES */
	Tk_mot ,			/* mot non cle				*/
	Tk_num ,			/* nombre				*/
	Tk_stng				/* string				*/
	};				/* ------------------------------------ */


#define GRAMTYPENB  1024

typedef struct {
	int nb;
	int **tk;
	int gdeb[MAX_DEFGRAM], gfin[MAX_DEFGRAM]; 
	} GramType;

static GramType itml_gram;


/*---------------------------------------------------------------------------------------
 *				Definition de la grammaire
 *  DEFGRAM xxxx
 *  CORP  = corps du prog		MOT   = mot
 *  PROC  = fonction SUB		'st'  = string
 *  EXPR  = expression			1234  = nombre
 *  BLOC  = bloc			DUP1  = le dernier est duplicable
 *  STOP  = fin du stgram_		DUP2  = les 2 derniers
 *
 *-------------------------------------------------------------------------------------*/

static char *itml_stgram[] = {

    "DEFGRAM CORP",

	"PROC DUP1",
	
    "DEFGRAM PROC",							/* types ? */

	"SUB  MOT  (                     )  {              }",
	"SUB  MOT  (  MOT                )  {              }",
	"SUB  MOT  (  MOT  ,  MOT  DUP2  )  {              }",
	"SUB  MOT  (                     )  {  BLOC  DUP1  }",
	"SUB  MOT  (  MOT                )  {  BLOC  DUP1  }",
	"SUB  MOT  (  MOT  ,  MOT  DUP2  )  {  BLOC  DUP1  }",

    "DEFGRAM BLOC",

	"{              }",
	"{  BLOC  DUP1  }",

	"MOT  =  DUP2  EXPR  ;",			/* lvaleur, affectation	*/
	"MOT  (                      )  ;",		/* fonction		*/
	"MOT  (  EXPR                )  ;",
	"MOT  (  EXPR ,  EXPR  DUP2  )  ;",

	"IF  (  EXPR  )  BLOC",
	"IF  (  EXPR  )  BLOC  ELSE  BLOC",

	"WHILE  (  EXPR  )  BLOC",

	"DO  BLOC  WHILE  (  EXPR  )  ;",

	"FOR  (  EXPR  ;  EXPR  ;  EXPR  )  BLOC",

    "DEFGRAM EXPR",		/* Qqch qui a 1 valeur - attention au type	*/
				/* pas de { } ;		le meme nb de ( )	*/
	"MOT",
	"MOT  +  EXPR",					/* binaire + -		*/
	"MOT  *  EXPR",					/* binaire pur		*/
	"MOT  (                       )",		/* fonction		*/
	"MOT  (  EXPR                 )",
	"MOT  (  EXPR  ,  EXPR  DUP2  )",

	"'st'",
	"'st'  +  EXPR",				/* binaire + -		*/
	"'st'  *  EXPR",				/* binaire pur		*/

	"1234",
	"1234  +  EXPR",				/* binaire + -		*/
	"1234  *  EXPR",				/* binaire pur		*/

	"+ EXPR",					/* unaire + -		*/
	"! EXPR",					/* unaire ! ++ --	*/

	"(  EXPR  )",
	"(  EXPR  )  ?  EXPR  :  EXPR",			/* ternaire		*/

    "STOP",
    };


/*---------------------------------------------------------------------------------------
 *		Fonctions du module
 *-------------------------------------------------------------------------------------*/

int itml_FindSub ();
int itml_AfterBracket ();
int itml_NextLexeme ();
int itml_CtrlBasNiveau ();
void itml_AffiLexemes ();
void itml_AffiLigneErreur ();
int itml_Lex2Token ();
int itml_initGram ();
int itml_initGeneral ();
char *itml_CreateVariabFromBox ();
int itml_GetVariab ();


/***************************************************************************************
 * Recherche "SUB nom_sub" et avance au prochain lexeme
 *
 * RETURN
 * -1 si erreur
 *  0 si s2 trouve mais e vide
 *  1 si s2 trouve et   e ok
 **************************************************************************************/
int itml_FindSub (nom_sub, source, e)
char *nom_sub, *source;
LexInfo *e;
{
    /* init */
    e->p = -1;

    while (itml_NextLexeme (source, e) > 0) {
	if (strncmp (source+e->p, "SUB", e->n) != 0) continue;
	if (itml_NextLexeme (source, e) <= 0) return -1;
	if (strncmp (source+e->p, nom_sub, e->n) != 0) continue;
	return itml_NextLexeme (source, e);
    }
    return -1;
}


/***************************************************************************************
 * Saute le bloc s1 .. s2
 *
 * EXEMPLES pour s1,s2 :  "(",")"  "[","]"  "{","}"  "BEGIN","END"
 *
 * A l'appel, e == s1 ; au retour, e est le lexeme qui suit s2 .
 *
 * RETURN
 * -1 si erreur
 *  0 si s2 trouve mais e vide
 *  1 si s2 trouve et   e ok
 **************************************************************************************/
int itml_AfterBracket (s1, s2, source, e)
char *s1, *s2, *source;
LexInfo *e;
{
    int nb = 1;
    char *s;

    while (itml_NextLexeme (source, e) > 0) {
	s = source+e->p;
	if (strncmp (s, s2, e->n) == 0) if (--nb == 0)
	    return itml_NextLexeme (source, e);
	if (strncmp (s, s1, e->n) == 0) nb++;
    }
    return -1;
}


/***************************************************************************************
 * Extraction du prochain lexeme, en sautant le lexeme actuel.
 * 
 * Si e.p < 0 la recherche debute en 0 (initialisation : e.p = -1)
 * Si e.p >= 0 et e.n == 0 alors erreur
 * Si e.p >= 0 alors e.p += e.n 
 *
 * RESULTAT
 * e.p est au debut du lexeme
 * e.type est le LexemeType
 * e.n est la taille du lexeme
 *
 * RETURN
 * -1 si erreur              (e.n peut etre > 0)
 *  0 si aucun lexeme trouve (e.n = 0)
 *  1 succes                 (e.n > 0)
 *
 * USAGE
 *     LexInfo e;
 *     e.p = -1;
 *     while (itml_NextLexeme (source, &e) > 0) {
 *         strncmp (source + e.p, "motcle", e.n);
 *     }
 **************************************************************************************/
int itml_NextLexeme (source, e)
char *source;
LexInfo *e;
{
    char c, d;

    /* Saut du lexeme actuel, si il existe */
    if (e->p < 0) e->p = 0;
    else if (e->n == 0) return -1;
    else e->p += e->n;

    /* Initialisation du nouveau lexeme */
    e->n = 0;
    e->type = Lx_rien;

    while (c = source[e->p]) {

	/* on saute les blancs */
	if (isspace (c)) {
	    e->p++;
	    continue;
	}

	/* on saute les commentaires */
	if (c == '#') {
	    do e->p++; while ((d = source[e->p]) && d != '\n');
	    if (d) e->p++;
	    continue;
	}

	/* c'est un string */
	if (c == '\'') {
	    e->type = Lx_string;
	    do e->n++; while ((d = source[e->p+e->n]) && d != '\'');
	    if (d == '\'') { e->n++; return 1; }
	    return -1;
	    /* Attention les "'" sont stockees avec les string */
	    /* Attention les "\'" ne sont pas gere'es dans le string */
	}

	/* c'est un mot */
	if (EST_ALPHA(c)) {
	    e->type = Lx_mot;
	    do e->n++; while ((d = source[e->p+e->n]) && 
		(EST_ALPHA(d) || EST_NUM(d) || d == '.') );
	    return 1;
	    /* Attention le cas ".." n'est pas traite' */
	}

	/* c'est un nombre */
	if (EST_NUM(c)) {
	    e->type = Lx_num;
	    do e->n++; while ((d = source[e->p+e->n]) && EST_NUM(d));
	    if (d == '.') {
		e->n++;
		if (!EST_NUM(source[e->p+e->n])) return -1;
		while ((d = source[e->p+e->n]) && EST_NUM(d)) e->n++;
	    }
	    if (toupper(d) == 'E') {
		e->n++;
		if ((d = source[e->p+e->n]) == '+' || d == '-') e->n++;
		if (!EST_NUM(source[e->p+e->n])) return -1;
		while ((d = source[e->p+e->n]) && EST_NUM(d)) e->n++;
	    }
	    return 1;
	    /* Attention on n'a pas teste les depassements de capacite */
	}

	/* c'est une ponctuation : couple ou simplex */
	d = source[e->p+e->n+1];
	if (EST_PONCT2 (c,d)) { e->type = Lx_ponct; return (e->n = 2); }
	if (EST_PONCT1 (c)  ) { e->type = Lx_ponct; return (e->n = 1); }

	/* erreur */
	return -1;
    }

    /* Rien d'interessant n'est lu */
    return 0;
}


/***************************************************************************************
 * Controle bas niveau de la syntaxe :
 *
 * - Regularite des lexemes
 * - Imbrication des () [] et {}
 * - SUB en dehors des {}
 **************************************************************************************/
int itml_CtrlBasNiveau (source)
char *source;
{
#   define TAB_PAR_CR0 1024
    int nb_par_cro = 0, tab_par_cro[TAB_PAR_CR0];
    enum { UNE_PAR, UNE_CRO };
    int r, nb_acc = 0;
    LexInfo e;
    char *s;

    e.p = -1;
    while ((r = itml_NextLexeme (source, &e)) > 0) {

	s = source + e.p;
	if (strncmp (s, "(", e.n) == 0) {
	    if (nb_par_cro >= TAB_PAR_CR0-1) { r = -102; break; }
	    tab_par_cro[nb_par_cro++] = UNE_PAR;
	}
	else if (strncmp (s, "[", e.n) == 0) {
	    if (nb_par_cro >= TAB_PAR_CR0-1) { r = -102; break; }
	    tab_par_cro[nb_par_cro++] = UNE_CRO;
	}
	else if (strncmp (s, ")", e.n) == 0) {
	    if (--nb_par_cro < 0) { r = -101; break; }
	    if (tab_par_cro[nb_par_cro] != UNE_PAR) { r = -104; break; }
	}
	else if (strncmp (s, "]", e.n) == 0) {
	    if (--nb_par_cro < 0) { r = -101; break; }
	    if (tab_par_cro[nb_par_cro] != UNE_CRO) { r = -104; break; }
	}
	else if (strncmp (s, "{", e.n) == 0) {
	    nb_acc++;
	    if (nb_par_cro > 0) { r = -108; break; }
	}
	else if (strncmp (s, "}", e.n) == 0) {
	    if (--nb_acc < 0) { r = -101; break; }
	    if (nb_par_cro > 0) { r = -104; break; }
	}
	else if (strncmp (s, "SUB", e.n) == 0) {
	    if (nb_par_cro > 0 || nb_acc > 0) { r = -107; break; }
	}
    }

    if (r >= 0 && nb_acc > 0) r = -105;
    if (r < 0) itml_AffiLigneErreur (source, &e, r);
    return r;
}


/***************************************************************************************
 * Affiche les lexemes du source
 **************************************************************************************/
void itml_AffiLexemes (source)
char *source;
{
    LexInfo e;
    int i, r;

    e.p = -1;
    while ((r = itml_NextLexeme (source, &e)) > 0) {

	switch (e.type) {
	    case Lx_rien   : printf ("RIEN   \""); break;
	    case Lx_string : printf ("STRING \""); break;
	    case Lx_mot    : printf ("MOT    \""); break;
	    case Lx_num    : printf ("NUM    \""); break;
	    case Lx_ponct  : printf ("PONCT  \""); break;
	}

	for (i = 0; i < e.n; i++) printf ("%c", source[e.p+i]);
	printf ("\"\n");
     }

    if (r < 0) itml_AffiLigneErreur (source, &e, -1);
}


/***************************************************************************************
 * Affiche la ligne et la position de l'erreur
 **************************************************************************************/
void itml_AffiLigneErreur (source, e, r)
char *source;
LexInfo *e;
int r;
{
    int li_p = -1, li_n = 1, i;
    char c;

    for (i = 0; i < e->p; i++)
	if (source[i] == '\n') { li_p = i; li_n++; }
    printf ("\nERROR %d in line %d : ", r, li_n);

    switch (r) {
	case     -1 : printf ("Syntax\n"); break;
	case   -101 : printf ("Too many '),],}'\n"); break;
	case   -102 : printf ("Over 1024 '(,['\n"); break;
	case   -104 : printf ("')' or ']' expected\n"); break;
	case   -105 : printf ("'}' expected\n"); break;
	case   -107 : printf ("Badly placed 'SUB'\n"); break;
	case   -108 : printf ("Badly placed '{'\n"); break;
	default     : printf ("\n");
    }

    for (i = li_p+1; (c = source[i]) && c != '\n'; i++)
	printf ("%c", c);
    printf ("\n");
    for (i = li_p+1; i < e->p; i++)
	printf ("%c", (isspace (c = source[i])) ? c : ' ');
    for (i = 0; i < e->n; i++)
	printf ("^");
    printf ("^\n\n");
}


/***************************************************************************************
 * Determine TokenType a partir du lexeme
 **************************************************************************************/
int itml_Lex2Token (source, e)
char *source;
LexInfo *e;
{
    switch (e->type) {

	case Lx_mot : {
	    char *s = source + e->p;
	    if      (strncmp (s, "SUB"  , e->n) == 0) return Tk_sub;
	    else if (strncmp (s, "IF"   , e->n) == 0) return Tk_if;
	    else if (strncmp (s, "ELSE" , e->n) == 0) return Tk_else;
	    else if (strncmp (s, "WHILE", e->n) == 0) return Tk_whil;
	    else if (strncmp (s, "DO"   , e->n) == 0) return Tk_do;
	    else if (strncmp (s, "FOR"  , e->n) == 0) return Tk_for;
	    else return Tk_mot;
	}
	case Lx_ponct : {
	    char *s = source + e->p;
	    if      (strncmp (s, "{" , e->n) == 0) return Tk_accG;
	    else if (strncmp (s, "}" , e->n) == 0) return Tk_accD;
	    else if (strncmp (s, "(" , e->n) == 0) return Tk_parG;
	    else if (strncmp (s, ")" , e->n) == 0) return Tk_parD;
	    else if (strncmp (s, "[" , e->n) == 0) return Tk_croG;
	    else if (strncmp (s, "]" , e->n) == 0) return Tk_croD;
	    else if (strncmp (s, "," , e->n) == 0) return Tk_virg;
	    else if (strncmp (s, ";" , e->n) == 0) return Tk_ptvi;
	    else if (strncmp (s, "?" , e->n) == 0) return Tk_ter1;
	    else if (strncmp (s, ":" , e->n) == 0) return Tk_ter2;
	    else if (strncmp (s, "=" , e->n) == 0 ||
		     strncmp (s, "+=", e->n) == 0 ||
		     strncmp (s, "-=", e->n) == 0 ) return Tk_afec;
	    else if (strncmp (s, "!" , e->n) == 0 ||
		     strncmp (s, "++", e->n) == 0 ||
		     strncmp (s, "--", e->n) == 0 ) return Tk_unai;
	    else if (strncmp (s, "+" , e->n) == 0 ||
		     strncmp (s, "-" , e->n) == 0 ) return Tk_u_b;
	    else return Tk_bina;
	}
	case Lx_string : return Tk_stng;
	case Lx_num    : return Tk_num;
	default        : return Tk_null;
    }
}


/***************************************************************************************
 * Initialisation de la grammaire
 *
 **************************************************************************************/
int itml_initGram (stgram, gram)
char *stgram[];
GramType *gram;
{
    LexInfo e;
    int i, j, r, k, v[GRAMTYPENB], *tk[GRAMTYPENB], curg;
    char *s;

    /* init index des DEFGRAM */
    for (i = 0; i < MAX_DEFGRAM; i++) gram->gdeb[i] = gram->gfin[i] = 0;
    curg = Tk_null;

#   ifdef DEBUG
    printf ("\nVOICI LA GRAMMAIRE\n\n");
#   endif

    for (gram->nb = 0; 1; gram->nb++) {

#	ifdef DEBUG
	printf ("%4d   %s\n", gram->nb, stgram[gram->nb]);
#	endif

	e.p = -1; j = 0;

	while ((r = itml_NextLexeme (stgram[gram->nb], &e)) > 0) {
	    s = stgram[gram->nb] +e.p;

	    if ((k = itml_Lex2Token (stgram[gram->nb], &e)) == Tk_mot) {

		if (strncmp (s, "DEFGRAM", e.n) == 0) {
		    gram->gfin[curg] = gram->nb;
		    if (itml_NextLexeme (stgram[gram->nb], &e) <= 0) return -8;
		    s = stgram[gram->nb] +e.p;
		    if      (strncmp (s, "CORP", e.n) == 0) curg = Tk_corp;
		    else if (strncmp (s, "PROC", e.n) == 0) curg = Tk_proc;
		    else if (strncmp (s, "BLOC", e.n) == 0) curg = Tk_bloc;
		    else if (strncmp (s, "EXPR", e.n) == 0) curg = Tk_expr;
		    else return -7;
		    gram->gdeb[curg] = gram->nb+1;
		    break;
		}

		if (strncmp (s, "STOP", e.n) == 0) {
		    if ((gram->tk = (int **) malloc (gram->nb*sizeof(int*))) == NULL) 
			return -3;
		    memcpy (gram->tk, tk, gram->nb*sizeof(int*));
		    gram->gfin[curg] = gram->nb;
#		    ifdef DEBUG
		    printf ("\nDEBUT ET FIN DEFGRAM\n\n");
		    for (i = 0; i < MAX_DEFGRAM; i++)
		    printf ("%3d  %3d %3d\n", i, gram->gdeb[i], gram->gfin[i]);
#		    endif
		    return 0;
		}

		if      (strncmp (s, "PROC", e.n) == 0) k = Tk_proc;
		else if (strncmp (s, "BLOC", e.n) == 0) k = Tk_bloc;
		else if (strncmp (s, "EXPR", e.n) == 0) k = Tk_expr;
		else if (strncmp (s, "DUP1", e.n) == 0) k = Tk_dup1;
		else if (strncmp (s, "DUP2", e.n) == 0) k = Tk_dup2;
	    }
	    if (j >= GRAMTYPENB-1) return -2;
	    v[j++] = k;
	}

	if (r < 0) return -1; /* erreur de lexeme */
	if (j >= GRAMTYPENB-1) return -2;
	v[j++] = Tk_null;
	if ((tk[gram->nb] = (int *) malloc (j * sizeof(int))) == NULL) return -3;
	memcpy (tk[gram->nb], v, j * sizeof(int));
    }
    return -5;  /* le STOP pas trouve' */
 }


/***************************************************************************************
 * Initialisation generale du module
 **************************************************************************************/
int itml_initGeneral ()
{
    int r;

    /* chargement de la grammaire */
    if ((r = itml_initGram (itml_stgram, &itml_gram)) < 0) return r;

    return 0;
}


/***************************************************************************************
 * 
 **************************************************************************************/
char *itml_CreateVariabFromBox (source)
char *source;
{
    int g;
    char mot[LG_MOT], gros[LG_GROS];
    LexInfo e;

    g = 0;
    if (itml_FindSub ("BOX", source, &e) <= 0) return (char *) NULL;

    if (strncmp (source+e.p, "(", e.n) == 0)
    if (itml_AfterBracket ("(", ")", source, &e) <= 0) return (char *) NULL;

    do {
	if (strncmp (source+e.p, "TITLE", e.n) != 0) continue;
	if (itml_NextLexeme (source, &e) <= 0) break;
	if (strncmp (source+e.p, "=", e.n) != 0) continue;
	if (itml_NextLexeme (source, &e) <= 0) break;
	sprintf (gros+g, "VAL TITLE "); g += 10;
	strncpy (gros+g, source+e.p+1, e.n-2); g += e.n-2;
	sprintf (gros+g, "\n"); g += 1;
	break;
    } while (itml_NextLexeme (source, &e) > 0);

    gros[g] = 0;
    return stg_Cpy (gros);
}

/***************************************************************************************
 * Parcourt la zone variab, cherche une ligne du genre "mot1 mot2 ???".
 * 
 * Si trouve', renvoie ??? dans resu et return VRAI.
 *
 * USAGE :
 *     char resu[256];
 *     if (itml_GetVariab (variab, "VAL", "TITLE", resu)) ;
 *
 **************************************************************************************/
int itml_GetVariab (variab, mot1, mot2, resu)
char *variab, *mot1, *mot2, *resu;
{
    int p = 0, i, j;
    char c, q[LG_LIGNE];

    /* debut de la ligne a chercher */
    sprintf (q, "%s %s ", mot1, mot2);
#   ifdef DEBUG
    printf ("itml_GetVariab \"%s\"\n", q);
#   endif

    /* recherche */
    while (variab[p]) {

	/* comparaison */
	i = 0;
	while ((c = variab[p+i]) && c == q[i]) i++;
	p += i;

	/* q trouve */
	if (q[i] == 0) {
	    j = 0;
	    while ((c = variab[p+j]) && c != '\n') resu[j++] = c;
	    resu[j] = 0;
#	    ifdef DEBUG
	    printf ("  OK \"%s\"\n", resu);
#	    endif
	    return VRAI;
	} 

	/* q pas trouve, goto ligne suivante */
	while ((c = variab[p]) && c != '\n') p++;
	if (c) p++; 
    }

    /* q non trouve */
    resu[0] = 0;
    return FAUX;
}


/*****************************************************************************************/

#ifdef CaServiraPlusTard

enum TokenType {
	Tk_not, Tk_pp, Tk_mm,		/* ! ++ --		unaire		*/
	Tk_pu, Tk_mu, Tk_fct, 		/* + - fonction 	unaire		*/
	Tk_mul, Tk_div, Tk_mod, 	/* * / %		multiplicatif	*/
	Tk_add, Tk_sous, 		/* + -			additif		*/
	Tk_decG, Tk_decD, 		/* << >>		decalage	*/
	Tk_inf, Tk_sup, 		/* < >			relationnel	*/
	Tk_infE, Tk_supE, 		/* <= >=		relationnel	*/
	Tk_ega, Tk_egaN, 		/* == !=		egalite		*/
	Tk_etB, Tk_xorB, Tk_ouB, 	/* & ^ |		niveau bit	*/
	Tk_et, Tk_ou,			/* && ||		logique		*/
	Tk_ter1, Tk_ter2, 		/* ? :			conditionnel	*/
	Tk_setE, Tk_setP, Tk_setM, 	/* = += -=		affectation	*/
	Tk_virgu, Tk_parG, Tk_parD, 	/* , ( )		ponctuation	*/
	Tk_Lval, Tk_Rval		/* 			lvaleur rvaleur	*/
	};

/*
 * ATTENTION respecter l'ordre donne' dans TokenType
*/

static int itml_priorite[] = {
	13, 13, 13,			/* ! ++ --		unaire		*/
	13, 13, 13, 			/* + - fonction 	unaire		*/
	12, 12, 12, 			/* * / %		multiplicatif	*/
	11, 11, 			/* + -			additif		*/
	10, 10, 			/* << >>		decalage	*/
	9, 9, 				/* < >			relationnel	*/
	9, 9, 				/* <= >=		relationnel	*/
	8, 8, 				/* == !=		egalite		*/
	7, 6, 5, 			/* & ^ |		niveau bit	*/
	4, 3,				/* && ||		logique		*/
	2, 2, 				/* ? :			conditionnel	*/
	1, 1, 1, 			/* = += -=		affectation	*/
	0, 0, 0, 			/* , ( )		ponctuation	*/
	0, 0				/* 			lvaleur rvaleur	*/
	};


typedef char Motscle[16];

static Motscle itml_motscle[] = {
	"IF", "ELSE", "WHILE", "DO", "FOR",
	"SUB", "BOX", "SEL",
	"NEW_ITEM", "NULL", "ID", "CAT", "TYPE", "NAME", 
	"MIN", "MAX", "VAL", "SIZE", "SLIST", "PROC",
	"IN", "OUT", "IMAGE",
	"PAR", "DOUBLE", "INT", "TEXT", "MENUH", "MENUV", "SLIDER", "ONOFF",
	"CENTER", "LEFT", "RIGHT"
	};

#define NB_MOTSCLE	(sizeof(itml_motscle) / sizeof(Motscle))

#define GRAM_LIGNE  16
typedef Gram_ligne[GRAM_LIGNE];
#define GRAM_SIZE(x)  (sizeof(x) / sizeof(Gram_ligne))

Gram_ligne gram_corp[] = {
	{ Tk_fonc , Tk_dup1, Tk_null },
	};

Gram_ligne gram_fonc[] = {
	{ Tk_sub , Tk_mot , 
	  Tk_parG, Tk_mot , Tk_virg, Tk_dup2, Tk_parD,
	  Tk_accG, Tk_bloc, Tk_dup1, Tk_accD,
	  Tk_null },
	};

Gram_ligne gram_bloc[] = {
	 { Tk_accG, Tk_bloc, Tk_dup1, Tk_accD, Tk_null },
	 { Tk_mot , Tk_afec, Tk_mot , Tk_afec, Tk_dup2, Tk_expr, Tk_ptvi, Tk_null },
	 { Tk_mot , Tk_parG, Tk_expr, Tk_virg, Tk_dup2, Tk_parD, Tk_ptvi, Tk_null },
	 { Tk_if  , Tk_parG, Tk_expr, Tk_parD, Tk_bloc, Tk_null },
	 { Tk_if  , Tk_parG, Tk_expr, Tk_parD, Tk_bloc, Tk_else, Tk_bloc, Tk_null },
	 { Tk_whil, Tk_parG, Tk_expr, Tk_parD, Tk_bloc, Tk_null },
	 { Tk_do  , Tk_bloc,
	   Tk_whil, Tk_parG, Tk_expr, Tk_parD, Tk_ptvi, Tk_null },
	 { Tk_for , Tk_parG, Tk_expr, Tk_ptvi, Tk_expr, Tk_ptvi, Tk_expr, Tk_parD, 
	   Tk_bloc, Tk_null },
	};

Gram_ligne gram_expr[] = {
	 { Tk_mot , Tk_null },
	 { Tk_mot , Tk_bina, Tk_expr, Tk_null },
	 { Tk_mot , Tk_u_b , Tk_expr, Tk_null },
	 { Tk_mot , Tk_parG, Tk_expr, Tk_virg, Tk_dup2, Tk_parD, Tk_null },
	 { Tk_stng, Tk_null },
	 { Tk_stng, Tk_bina, Tk_expr, Tk_null },
	 { Tk_stng, Tk_u_b , Tk_expr, Tk_null },
	 { Tk_num , Tk_null },
	 { Tk_num , Tk_bina, Tk_expr, Tk_null },
	 { Tk_num , Tk_u_b, Tk_expr, Tk_null },
	 { Tk_unai, Tk_expr, Tk_null },
	 { Tk_u_b , Tk_expr, Tk_null },
	 { Tk_parG, Tk_expr, Tk_parD, Tk_null },
	 { Tk_parG, Tk_expr, Tk_parD, Tk_ter1, Tk_expr, Tk_ter2, Tk_expr, Tk_null },
	};



#endif
