/***************************************************************************************/
/* tool_menu.c                                                I.P.S  v 3.5  30/01/1995 */
/*                                                                                     */
/* Lecture des menus `tool'                              Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/

#include "inclus.h"


/*
 * Nombre, clef et nom des menus root
*/
#define		 NB_MENU_ROOT		3
static char *	ltm_MENU_ROOT_clef[] = { "GLOBAL", "PERSO", "HELP" };
static char *	ltm_MENU_ROOT_nom [] = { "Global", "Perso", "Help" };


#define LG_STG_CHAR	1024*16

#define LG_LIGNE	255+1
#define LG_MOT		255+1

#define MAX_PROFONDEUR	20

typedef struct Lmt_stock_t {
	char c[LG_STG_CHAR];			/* chaines NON RELOGEABLES */
	struct Lmt_stock_t *next;
	} Lmt_stock;

typedef struct {
	char *this_file, *help_file, *oper_f_c, *oper_f_exe, 
	     *host_list; 
	} Lmt_info_files;

static Lmt_stock *lmtStg_v[NB_MENU_ROOT];		
static int lmtStg_p[NB_MENU_ROOT],		/* position libre dans chaine */
           lmtStg_n;				/* numero actif */

static int lmtErreur,
           lmtProfondeur;
static char lmt_ipsrc[256];

static Menu lmt_menu_error;
static Menu lmt_menus[NB_MENU_ROOT];
static Panel_item lmt_boutons[NB_MENU_ROOT];

static Menu lmt_menu_clic;    			/* memo pour lmt_ClickNow */
static Menu_item lmt_menu_item_clic;


void  lmt_FramePcpButtCreat ();
void  lmt_FramePcpButtReread ();
void  lmt_relecture ();
void  lmt_destroy_menu ();
Menu  lmt_menu_root ();
char *lmt_stg ();
void *lmt_adr ();
int   lmt_stg_place ();
char *lmt_mot_suiv ();
Menu  lmt_menu ();
void  lmt_FramePcpMenuProc ();
void  lmt_edit_file ();
int   lmt_trouve_num_ligne ();
void  lmt_this_menu_empty ();
void  lmt_ClickNow ();

/***************************************************************************************
 * Creation des boutons root et leurs menus dans le frame_principal
 *
 ***************************************************************************************/
void lmt_FramePcpButtCreat (panel)
Panel panel;
{
    int n;

    /* init pour lmt_ClickNow */
    lmt_menu_clic = 0;
    lmt_menu_item_clic = 0;

    lmt_menu_error = (Menu) xv_create (0, MENU,
	MENU_STRINGS, "This menu is empty, click to correct", NULL,
	MENU_NOTIFY_PROC, lmt_this_menu_empty,
	0);

    sprintf (lmt_ipsrc, "%s/.ipsrc", glo.home_dir);
    lmtErreur = FAUX;

    for (n = 0; n < NB_MENU_ROOT; n++) {
	lmt_menus[n] = lmt_menu_root (lmt_ipsrc, ltm_MENU_ROOT_clef[n], n);
	if (lmt_menus[n] == (Menu) NULL)
	    lmt_menus[n] = lmt_menu_error;
	lmt_boutons[n] = (Panel_item) xv_create (panel, PANEL_BUTTON,   
	    PANEL_LABEL_STRING,	ltm_MENU_ROOT_nom[n],
	    PANEL_ITEM_MENU,	lmt_menus[n],
	    NULL);
    }

    if (lmtErreur) 
	 pcp_footer ("Errors while reading \"", lmt_ipsrc, "\" : see stderr.", NULL);
    else pcp_footer_n ();
}

/***************************************************************************************
 * Relecture des menus root dans le frame_principal
 *
 ***************************************************************************************/
void lmt_FramePcpButtReread ()
{
    int n;

    /* init pour lmt_ClickNow */
    lmt_menu_clic = 0;
    lmt_menu_item_clic = 0;

    lmtErreur = FAUX;

    for (n = 0; n < NB_MENU_ROOT; n++)
	(void) lmt_relecture (lmt_ipsrc, ltm_MENU_ROOT_clef[n], n);

    if (lmtErreur) 
	 pcp_footer ("Errors while reading \"", lmt_ipsrc, "\" : see stderr.", NULL);
    else pcp_footer_n ();
}

/***************************************************************************************
 * Relecture d'un menu racine avec ses sous-menus.
 *
 **************************************************************************************/
void lmt_relecture (nom_f, nom_menu, n)
char *nom_f, *nom_menu;
int n;
{
    Lmt_stock *v1, *v2;

    pcp_footer ("Destroying menus ...", NULL);

    /* dereference d'abord le menu avant de le detruire */
    xv_set (lmt_boutons[n], PANEL_ITEM_MENU, NULL, NULL);
    if (lmt_menus[n] != lmt_menu_error)
	lmt_destroy_menu (lmt_menus[n]);

    /* Liberation des strings */ 
    v1 = lmtStg_v[n];
    while (v1 != (Lmt_stock *) NULL) {
	v2 = v1->next;
	free (v1);
	v1 = v2;
    }

    /* lit le menu et attache a bouton */
    lmt_menus[n] = lmt_menu_root (nom_f, nom_menu, n);
    if (lmt_menus[n] == (Menu) NULL)
	lmt_menus[n] = lmt_menu_error;
    xv_set (lmt_boutons[n], PANEL_ITEM_MENU, lmt_menus[n], NULL);
}


/***************************************************************************************
 * Destruction recursive d'un menu.
 *
 * Le pb vient de xview, qui ne libere pas les MENU_PULLRIGHT.
 **************************************************************************************/
void lmt_destroy_menu (menu)
Menu menu;
{
    int i;
    Frame subf;

    if (menu == 0) return;

    for (i = (int) xv_get (menu, MENU_NITEMS); i > 0; i--) {

	Menu_item item = (Menu_item) xv_get (menu, MENU_NTH_ITEM, i);
	Menu subm = (Menu) xv_get (item, MENU_PULLRIGHT);

	if (subm != 0) {
	    /* dereference d'abord le menu avant de le detruire */
	    xv_set (item, MENU_PULLRIGHT, NULL, NULL);
	    lmt_destroy_menu (subm);
	}
    }

    subf = (Frame) xv_get (menu, MENU_PIN_WINDOW);
    if (subf != 0) xv_destroy (subf);
    xv_destroy (menu); 
}

/***************************************************************************************
 * Creation d'un menu racine avec ses sous-menus.
 *   nom_f	nom du fichier contenant le menu racine
 *   n		numero de stockage < NB_MENU_ROOT
 ***************************************************************************************/
Menu  lmt_menu_root (nom_f, nom_menu, n)
char *nom_f, *nom_menu;
int n;
{
    Menu menu;

    pcp_footer ("Reading menus from ", nom_f, NULL);

    /* Init du bloc de memorisation */
    lmtStg_n = n;
    lmtStg_v[n] = (Lmt_stock *) malloc (sizeof(Lmt_stock));
    if (lmtStg_v[n] == (Lmt_stock *) NULL) {
	pcp_error ("lmt_menu_root #01", "can't allocate memory", NULL);
	pcp_footer_n ();
	return (Menu) NULL;
    }
    lmtStg_v[n]->next = (Lmt_stock *) NULL;
    lmtStg_v[n]->c[0] = 0;
    lmtStg_p[n] = 0;

    /* Lecture recursive du menu racine */
    lmtProfondeur = 0;
    menu = lmt_menu ("", nom_f, nom_menu);

    /* Retour */
    return menu;
}

/***************************************************************************************
 * Copie s1 dans l'espace de stockage et renvoie l'adresse.
 *
 * On stocke dans la meme structure des string et des adresses.
 * Or sur certaines architectures une adresse doit absolument etre un multiple de 4.
 * On procede donc a un re'alignement de la longueur (on ajoute au maximum 3).
 ***************************************************************************************/
char *lmt_stg (s1)
char *s1;
{
    char *s2;
    int l1 = strlen(s1)+1, r = l1 % 4;
    if (r > 0) l1 += 4-r;

    if (lmt_stg_place (l1))
	return (char *) NULL;
    s2 = lmtStg_v[lmtStg_n]->c + lmtStg_p[lmtStg_n];
    strcpy (s2, s1);
    lmtStg_p[lmtStg_n] += l1;
    return s2;
}

/***************************************************************************************
 * Reserve len dans l'espace de stockage et renvoie l'adresse.
 *
 * On fait le realignement, cf lmt_stg().
 ***************************************************************************************/
void *lmt_adr (len)
int  len;
{
    void *p;
    int r = len % 4;
    if (r > 0) len += 4-r;

    if (lmt_stg_place (len))
	return (void *) NULL;
    (char *) p = lmtStg_v[lmtStg_n]->c + lmtStg_p[lmtStg_n];
    lmtStg_p[lmtStg_n] += len;
    return p;
}

/***************************************************************************************
 * Degage un bloc contigu de `taille' octets dans l'espace de stockage lmtStg_v
 *
 ***************************************************************************************/
int lmt_stg_place (taille)
int taille;
{
    int n = lmtStg_n;

    if (lmtStg_p[n] + taille > LG_STG_CHAR) {
	Lmt_stock *v = (Lmt_stock *) malloc (sizeof(Lmt_stock));
	if (v == (Lmt_stock *) NULL) {
	    fprintf (stderr, "ERROR can't allocate memory for menus\n");
	    lmtErreur = VRAI;
	    return VRAI;
	}
	v->next = lmtStg_v[n];
	lmtStg_v[n] = v;
	lmtStg_v[n]->c[0] = 0;
	lmtStg_p[n] = 0;
    }
    return FAUX;
}

/****************************************************************************************
 * Renvoie le mot suivant
 *
 * On fait le realignement, cf lmt_stg().
 ***************************************************************************************/
char *lmt_mot_suiv (mot)
char *mot;
{
    int i, r;
    for (i = 0; mot[i++] && i < LG_MOT; );
    r = i % 4;
    if (r > 0) i += 4-r;
    return mot + i;
}

/***************************************************************************************
 * Lecture du menu `nom_menu' dans le fichier `nom_f_rel', 
 * avec appels recursifs et detection de boucle.
 *
 * Le chemin de nom_f_rel est recalcule avec nom_f_pere
 * Attention les autres chemins sont stocke's tels quels.
 *
 * PAS ENCORE GERE' : le saut de ligne [\] et les variable $TOTO.
 ***************************************************************************************/
Menu  lmt_menu (nom_f_pere, nom_f_rel, nom_menu)
char *nom_f_pere, *nom_f_rel, *nom_menu;
{
    char ligne[LG_LIGNE], mot[LG_MOT], *suite, *sm, *st, nom_f[256], 
         nom_tmp[256],
	 help_file[256], oper_f_c[256], oper_f_exe[256], 
	 host_list[256]; 
    FILE *f;
    int n_ligne, motp, trouve, grise, i, k;
    Menu menu = (Menu) NULL;
    Menu_item  mi;
    Lmt_info_files *info_adr;

    /*
     * Le chemin de nom_f_rel est recalcule avec nom_f_pere
    */
    stg_concat_2files (nom_f_pere, nom_f_rel, nom_f);

    /*
     * Test de profondeur
    */
    if (++lmtProfondeur > MAX_PROFONDEUR) {
	fprintf (stderr, "ERROR for menu %s : loop detected in %s\n", 
	    nom_menu, nom_f);
	lmtErreur = VRAI;
	lmtProfondeur--; return (Menu) NULL;	
    }

    /*
     * Ouverture du fichier 
    */
    if ((f = fopen (nom_f, "r")) == NULL) {
	fprintf (stderr, "ERROR for menu %s : can't open the file %s\n", 
	    nom_menu, nom_f);
	lmtErreur = VRAI;
	lmtProfondeur--; return (Menu) NULL;
    }

    /*
     * Initialisations
    */
    n_ligne = 0; trouve = 0;

    /*
     * Recherche de "B_MENU nom_menu" dans le fichier
    */
    while ( fgets(ligne , LG_LIGNE , f ) != NULL ) {

	n_ligne++;
	for (i = 0; ligne[i]; i++) 
	if (ligne[i] == '#') { 
	    ligne[i] = 0; break; 
	}

	if (sscanf (ligne, " B_MENU %s%n", mot, &motp) == 1) {
	    if (strcmp (nom_menu, mot) == 0) { trouve = 1; break; }
	} else
	if (sscanf (ligne, " L_MENU %s%n", mot, &motp) == 1) {
	    if (strcmp (nom_menu, mot) == 0) { trouve = 2; break; }
	}
    }

    /* Ni B_MENU ni L_MENU n'ont ete trouves */
    if (trouve == 0) {
	fprintf (stderr, "ERROR can't find the menu %s in file %s\n",
	    nom_menu, nom_f);
	lmtErreur = VRAI;
	fclose (f);
	lmtProfondeur--; return menu;
    }

    /* L_MENU */
    if (trouve == 2) {
	char s_fic[LG_MOT];

	if ( sscanf (ligne+motp, "%s", s_fic) == 1)
	    menu = lmt_menu (nom_f, s_fic, nom_menu);
	else {
	    fprintf (stderr, "ERROR line %d in file %s : L_MENU without file name\n", 
		n_ligne, nom_f);
	    lmtErreur = VRAI;
	    menu = (Menu) NULL;
	}
	fclose (f);
	lmtProfondeur--; return menu;
    }


    /*
     * Lecture du nom du menu
    */
    if (sscanf (ligne+motp, " \"%[^\"]\"", mot) == 1)
	sm = mot; /* attention sm est volatile */
    else {
	sm = "No name";
	fprintf (stderr, "ERROR line %d in file %s : menu without name\n", 
	    n_ligne, nom_f);
	lmtErreur = VRAI;
    }

    /* Assure que les 4 prochaines adresses seront contigues
     * Pour chaque string, compter strlen(s)+1+3
     * Pour chaque adr, compter sizeof(adr)+3
     * (les +3 cause re'alignement, cf lmt_stg() ).
    */
    if ( lmt_stg_place (
	    strlen("MENU")+strlen(sm)+strlen(nom_menu)+3*(1+3)
	    +sizeof(Lmt_info_files)+3 )
	) { fclose (f); lmtProfondeur--; return menu; }
    st = lmt_stg ("MENU");
    lmt_stg (nom_menu);
    sm = lmt_stg (sm); /* voila sm est prote'ge' */
    info_adr = (Lmt_info_files *) lmt_adr (sizeof(Lmt_info_files));    

    /*
     * Creation du menu et inits
    */
    menu = (Menu) xv_create (0, MENU,
	MENU_TITLE_ITEM,	sm,
	MENU_CLIENT_DATA,	st,
	MENU_GEN_PIN_WINDOW,	glo.frame_principal, sm,
	MENU_NOTIFY_PROC,	lmt_FramePcpMenuProc,
	0);
    strcpy (help_file, nom_f);
    oper_f_c[0] = oper_f_exe[0] = host_list[0] = 0;

    /*
     * Lecture de corps du menu
    */
    while ( fgets(ligne , LG_LIGNE , f ) != NULL ) {

	n_ligne++;
	for (i = 0; ligne[i]; i++) 
	if (ligne[i] == '#') { 
	    ligne[i] = 0; break; 
	}

	grise = (ligne[0] == '?');
	if (grise) ligne[0] = ' ';

	if ( sscanf (ligne, "%s%n", mot, &motp) != 1 )
	    continue; /* on saute la ligne */

	suite = ligne + motp;

	if (strcmp (mot, "OPER") == 0 || 
	    strcmp (mot, "TEXT") == 0 || strcmp (mot, "TOOL") == 0) {

	    char s_idf[LG_MOT], s_tit[LG_MOT], s_fic[LG_MOT], 
		*m_mot, *m_idf, *m_tit, *m_fic;
	    Lmt_info_files **adr;

	    k = sscanf (suite, "%s \"%[^\"]\" %s", s_idf, s_tit, s_fic);
	    if (k < 3) {
		if (strcmp (mot, "TEXT") == 0) strcpy (s_fic, nom_f);
		else s_fic[0] = 0;
	    }

	    /* Assure que les 5 prochaines adresses seront contigues
	     * Pour chaque string, compter strlen(s)+1+3
	     * Pour chaque adr, compter sizeof(adr)+3
	     * (les +3 cause re'alignement, cf lmt_stg() ).
	    */
	    if ( lmt_stg_place (
		    strlen(mot)+strlen(s_idf)+strlen(s_tit)+strlen(s_fic)
		    +4*(1+3) +sizeof(info_adr)+3 )
		) continue;
	    m_mot = lmt_stg (mot);
	    m_idf = lmt_stg (s_idf);
	    m_tit = lmt_stg (s_tit);
	    m_fic = lmt_stg (s_fic);
	    adr = lmt_adr (sizeof(info_adr)); *adr = info_adr;

	    mi = (Menu_item) xv_create (0, MENUITEM,
		MENU_STRING,		m_tit,
		MENU_CLIENT_DATA,	m_mot,
		MENU_RELEASE,
		MENU_INACTIVE,		grise,
		0);
	    xv_set (menu, MENU_APPEND_ITEM, mi, NULL);
	}

	else if (strcmp (mot, "LOAD") == 0) {

	    char s_tit[LG_MOT], s_fic[LG_MOT], 
		*m_mot, *m_tit, *m_fic;
	    Lmt_info_files **adr;

	    k = sscanf (suite, " \"%[^\"]\" %s", s_tit, s_fic);
	    if (k < 2) s_fic[0] = 0;

	    /* Assure que les 4 prochaines adresses seront contigues
	     * Pour chaque string, compter strlen(s)+1+3
	     * Pour chaque adr, compter sizeof(adr)+3
	     * (les +3 cause re'alignement, cf lmt_stg() ).
	    */
	    if ( lmt_stg_place (
		    strlen(mot)+strlen(s_tit)+strlen(s_fic)
		    +3*(1+3) +sizeof(info_adr)+3 )
		) continue;
	    m_mot = lmt_stg (mot);
	    m_tit = lmt_stg (s_tit);
	    m_fic = lmt_stg (s_fic);
	    adr = lmt_adr (sizeof(info_adr)); *adr = info_adr;

	    mi = (Menu_item) xv_create (0, MENUITEM,
		MENU_STRING,		m_tit,
		MENU_CLIENT_DATA,	m_mot,
		MENU_RELEASE,
		MENU_INACTIVE,		grise,
		0);
	    xv_set (menu, MENU_APPEND_ITEM, mi, NULL);
	}

	else if (strcmp (mot, "EXEC") == 0) {

	    char s_tit[LG_MOT], s_exec[LG_MOT],
		*m_mot, *m_tit, *m_exec;
	    Lmt_info_files **adr;

	    k = sscanf (suite, " \"%[^\"]\" %[^\n]", s_tit, s_exec);
	    if (k < 2) s_exec[0] = 0;

	    /* Assure que les 4 prochaines adresses seront contigues
	     * Pour chaque string, compter strlen(s)+1+3
	     * Pour chaque adr, compter sizeof(adr)+3
	     * (les +3 cause re'alignement, cf lmt_stg() ).
	    */
	    if ( lmt_stg_place (
		    strlen(mot)+strlen(s_tit)+strlen(s_exec)
		    +3*(1+3) +sizeof(info_adr)+3 )
		) continue;
	    m_mot = lmt_stg (mot);
	    m_tit = lmt_stg (s_tit);
	    m_exec = lmt_stg (s_exec);
	    adr = lmt_adr (sizeof(info_adr)); *adr = info_adr;

	    mi = (Menu_item) xv_create (0, MENUITEM,
		MENU_STRING,		m_tit,
		MENU_CLIENT_DATA,	m_mot,
		MENU_RELEASE,
		MENU_INACTIVE,		grise,
		0);
	    xv_set (menu, MENU_APPEND_ITEM, mi, NULL);
	}

	else if (strcmp (mot, "MENU") == 0) {

	    char s_idf[LG_MOT], s_tit[LG_MOT], s_fic[LG_MOT];

	    k = sscanf (suite, "%s \"%[^\"]\" %s", s_idf, s_tit, s_fic);
	    if (k < 3) strcpy (s_fic, nom_f);

	    mi = (Menu_item) xv_create (0, MENUITEM,
		MENU_STRING,		lmt_stg (s_tit),
		MENU_PULLRIGHT,		lmt_menu (nom_f, s_fic, s_idf),
		MENU_RELEASE,
		MENU_INACTIVE,		grise,
		0);
	    xv_set (menu, MENU_APPEND_ITEM, mi, NULL);
	}

	else if (strcmp (mot, "SEPARATOR") == 0) {

	    mi = (Menu_item) xv_create (0, MENUITEM_SPACE, MENU_RELEASE, 0);
	    xv_set (menu, MENU_APPEND_ITEM, mi, 0);
	}

	else if (strcmp (mot, "TEL_OPER_EXE") == 0) {

	    if (sscanf (suite, "%s", oper_f_exe) != 1) oper_f_exe[0] = 0;
	}

	else if (strcmp (mot, "TEL_OPER_C") == 0) {

	    if (sscanf (suite, "%s", oper_f_c) != 1) oper_f_c[0] = 0;
	}

	else if (strcmp (mot, "HELP_FILE") == 0) {

	    if (sscanf (suite, "%s", help_file) != 1) help_file[0] = 0;
	}
	else if (strcmp (mot, "HOST_LIST") == 0) {

	    if (sscanf (suite, "%s", host_list) != 1)
		 host_list[0] = 0;
	    else strncpy (host_list, suite, 255);
	}

	else if (strcmp (mot, "B_MENU") == 0) {

	    /* On ne fait rien, les lignes suivantes sont partages
	     * avec un autre menu imbriqu. 
	    */
	}

	else if (strcmp (mot, "E_MENU") == 0) {

	    /* Si on atteint la fin du menu, on sort du while avec un break.
	     * Cas  distinguer :
	     *
	     *		E_MENU			fin atteinte
	     *		E_MENU nom_menu		fin atteinte
	     *		E_MENU autre_nom	fin d'un autre menu imbriqu.
	    */

	    char s_idf[LG_MOT];

	    k = sscanf (suite, "%s", s_idf);
	    if (k < 1 || strcmp (s_idf, nom_menu) == 0)
		break; /* sortie du while ; sinon on continue la lecture */
	}

	else { /* Erreur de Syntaxe */

	    fprintf (stderr, "SYNTAX ERROR line %d in file %s\n", 
		n_ligne, nom_f);
	    lmtErreur = VRAI;
	    break;
	}

    } /* of while */	

	    
    /*
     * Si le menu est vide, ajoute un MENUITEM_SPACE pr eviter erreur xview
    */
    if ((int) xv_get (menu, MENU_NITEMS) <= 1) {
    	mi = (Menu_item) xv_create (0, MENUITEM_SPACE, MENU_RELEASE, 0);
    	xv_set (menu, MENU_APPEND_ITEM, mi, 0);
    }
 
    /*
     * stockage des strings pour info_adr
    */
    info_adr->this_file  = lmt_stg (nom_f);
    info_adr->help_file  = lmt_stg (help_file);
    info_adr->oper_f_c   = lmt_stg (oper_f_c);
    info_adr->oper_f_exe = lmt_stg (oper_f_exe);
    info_adr->host_list  = lmt_stg (host_list);


    /* 
     * Fermeture
    */
    fclose (f);
    lmtProfondeur--; return (menu);
}


/****************************************************************************************
 * Appel direct pour aide_action_proc
 *
 ***************************************************************************************/
void lmt_ClickNow ()
{
    if (lmt_menu_clic != 0 && lmt_menu_item_clic != 0)
	lmt_FramePcpMenuProc (lmt_menu_clic, lmt_menu_item_clic);
}

/****************************************************************************************
 * Callback des menus
 *
 ***************************************************************************************/
void lmt_FramePcpMenuProc (menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    char *m_mot;

    /* memo pour lmt_ClickNow */
    lmt_menu_clic = menu;
    lmt_menu_item_clic = menu_item;

    m_mot = (char *) xv_get (menu_item, MENU_CLIENT_DATA);
    if (m_mot == (char *) NULL)
	m_mot = (char *) xv_get (menu, MENU_CLIENT_DATA);

    if (strcmp (m_mot, "TEXT") == 0) {

	char *m_idf = lmt_mot_suiv (m_mot),
	     *m_tit = lmt_mot_suiv (m_idf),
	     *m_fic = lmt_mot_suiv (m_tit);
	Lmt_info_files *i_adr = *((Lmt_info_files **) lmt_mot_suiv (m_fic));

	/* chemin absolu */
	char text_file[256];

	/* Attention tous les chemins stocke's sont relatifs
	 * Il faut les recalculer par rapport au fichier de menus qui est absolu.
	*/
	stg_concat_2files (i_adr->this_file, m_fic, text_file);

	switch (aide_get_action ()) {

	    case AIDE_IS_DISPLAY_HELP :
	    case AIDE_IS_CREATE_ICON :
		aide_affi_text (m_idf, text_file, "B_TEXT", "E_TEXT", "Text", "");
		break;

	    case AIDE_IS_EDIT_MENU : 
		lmt_edit_file (i_adr->this_file, m_mot, m_idf, " %s %s");
		break;

	    case AIDE_IS_EDIT_HELP :
		lmt_edit_file (text_file, "B_TEXT", m_idf, " %s %s");
		break;

	    default :
		aide_nothing_to_do ();
	}
    }
    else if (strcmp (m_mot, "TOOL") == 0) {

	char *m_idf = lmt_mot_suiv (m_mot),
	     *m_tit = lmt_mot_suiv (m_idf),
	     *m_fic = lmt_mot_suiv (m_tit);
	Lmt_info_files *i_adr = *((Lmt_info_files **) lmt_mot_suiv (m_fic));

	switch (aide_get_action ()) {

	    case AIDE_IS_DISPLAY_HELP :
		aide_affi_tool (m_idf, m_fic);
		break;

	    case AIDE_IS_CREATE_ICON : {

		int error = tool_create_tool_from_exec (m_idf, m_fic);
		if (error < 0) fprintf (stderr, 
		    "ERROR %d in tool_create_tool_from_exec\n", error);
	    } break;

	    default :
		aide_nothing_to_do ();
	}
    }

    else if (strcmp (m_mot, "OPER") == 0) {

	/* lecture dans l'ordre des arguments */
	char *m_idf = lmt_mot_suiv (m_mot),
	     *m_tit = lmt_mot_suiv (m_idf),
	     *m_fic = lmt_mot_suiv (m_tit);
	Lmt_info_files *i_adr = *((Lmt_info_files **) lmt_mot_suiv (m_fic));

	/* chemins absolus */
	char help_file[256], oper_f_c[256], oper_f_exe[256];

	/* Attention tous les chemins stocke's sont relatifs
	 * Il faut les recalculer par rapport au fichier de menus qui est absolu.
	*/
	stg_concat_2files (i_adr->this_file, i_adr->help_file, help_file);
	stg_concat_2files (i_adr->this_file, i_adr->oper_f_c, oper_f_c);
	stg_concat_2files (i_adr->this_file,
		(m_fic[0] == 0) ? i_adr->oper_f_exe : m_fic, oper_f_exe);

#	ifdef DEBUG
	printf ("\n");
	printf ("this_file  = %s\n", i_adr->this_file);
	printf ("m_idf      = %s\n", m_idf);
	printf ("m_tit      = %s\n", m_tit);
	printf ("m_fic      = %s\n", m_fic); /* relatif */
	printf ("oper_f_exe = %s\n", oper_f_exe);
	printf ("oper_f_c   = %s\n", oper_f_c);
	printf ("help_file  = %s\n", help_file);
	printf ("host_list  = %s\n", i_adr->host_list);
#	endif

	switch (aide_get_action ()) {

	    case AIDE_IS_DISPLAY_HELP :
		aide_affi_text (m_idf, help_file, "B_HELP", "E_HELP",
		    "Help on Operator ", m_idf);
		break;

	    case AIDE_IS_CREATE_ICON :
		telop_MenuHop (m_idf, oper_f_exe, help_file, i_adr->host_list);
		break;

	    case AIDE_IS_EDIT_MENU : 
		lmt_edit_file (i_adr->this_file, m_mot, m_idf, " %s %s");
		break;

	    case AIDE_IS_EDIT_TEL_OPER_C :
		lmt_edit_file (oper_f_c, "CO_BEGIN_CONNECT", m_idf, " %s ( \"%[^\"]\"");
		break;

	    case AIDE_IS_EDIT_HELP :
		lmt_edit_file (help_file, "B_HELP", m_idf, " %s %s");
		break;

	    default :
		aide_nothing_to_do ();
	}
    }

    else if (strcmp (m_mot, "LOAD") == 0) {

	/* lecture dans l'ordre des arguments */
	char *m_tit = lmt_mot_suiv (m_mot),
	     *m_fic = lmt_mot_suiv (m_tit);
	Lmt_info_files *i_adr = *((Lmt_info_files **) lmt_mot_suiv (m_fic));

	/* chemins absolus */
	char load_file[256];

	/* Attention tous les chemins stocke's sont relatifs
	 * Il faut les recalculer par rapport au fichier de menus qui est absolu.
	*/
	stg_concat_2files (i_adr->this_file, m_fic, load_file);

#	ifdef DEBUG
	printf ("\n");
	printf ("this_file  = %s\n", i_adr->this_file);
	printf ("m_tit      = %s\n", m_tit);
	printf ("m_fic      = %s\n", m_fic); /* relatif */
	printf ("load_file  = %s\n", load_file);
#	endif

	switch (aide_get_action ()) {

	    case AIDE_IS_CREATE_ICON :
		fichier_drop (load_file);
		break;

	    case AIDE_IS_EDIT_MENU :
		lmt_edit_file (i_adr->this_file, m_mot, m_tit, " %s \"%[^\"]\"");
		break;

	    case AIDE_IS_DISPLAY_HELP : {
		char leblabla[4096];
		sprintf (leblabla,
"			FILE WHICH IS LOADED : \n\
\n\
\n\
  Title         = %s \n\n\
  Menu file     = %s \n\n\
  Relative path = %s \n\n\
  Computed path = %s \n\n\
\n\
", m_tit, i_adr->this_file, m_fic, load_file); 
		aide_affi_message (leblabla);
	    } break;

	    default :
		aide_nothing_to_do ();
	}
    }

    else if (strcmp (m_mot, "EXEC") == 0) {

	/* lecture dans l'ordre des arguments */
	char *m_tit = lmt_mot_suiv (m_mot),
	     *m_exec = lmt_mot_suiv (m_tit);
	Lmt_info_files *i_adr = *((Lmt_info_files **) lmt_mot_suiv (m_exec));

	/* chemins absolus */
	char chemin[256], exec_line[256];

	/* Attention tous les chemins stocke's sont relatifs
	 * Il faut les recalculer par rapport au fichier de menus qui est absolu.
	*/
	stg_concat_2files (i_adr->this_file, ".", chemin);
        sprintf (exec_line, "cd %s ; %s &", chemin, m_exec);

#	ifdef DEBUG
	printf ("\n");
	printf ("this_file  = %s\n", i_adr->this_file);
	printf ("m_tit      = %s\n", m_tit);
	printf ("m_exec     = %s\n", m_exec); /* relatif */
	printf ("chemin     = %s\n", chemin); /* absolu */
	printf ("exec_line  = %s\n", exec_line);
#	endif

	switch (aide_get_action ()) {

	    case AIDE_IS_CREATE_ICON :
		system (exec_line);
		break;

	    case AIDE_IS_EDIT_MENU :
		lmt_edit_file (i_adr->this_file, m_mot, m_tit, " %s \"%[^\"]\"");
		break;

	    case AIDE_IS_DISPLAY_HELP : {
		char leblabla[4096];
		sprintf (leblabla,
"			COMMAND WHICH IS EXECUTED : \n\
\n\
\n\
  Title          = %s \n\n\
  Menu file      = %s \n\n\
  Command        = %s \n\n\
  Path from menu = %s \n\n\
  Executed line  = %s \n\n\
\n\
", m_tit, i_adr->this_file, m_exec, chemin, exec_line); 
		aide_affi_message (leblabla);
	    } break;

	    default :
		aide_nothing_to_do ();
	}
    }

    else if (strcmp (m_mot, "MENU") == 0) {

	char *m_idf = lmt_mot_suiv (m_mot),
	     *m_tit = lmt_mot_suiv (m_idf);
	Lmt_info_files *i_adr = (Lmt_info_files *) lmt_mot_suiv (m_tit);

#	ifdef DEBUG
	printf ("\n");
	printf ("this_file  = %s\n", i_adr->this_file);
	printf ("m_idf      = %s\n", m_idf);
	printf ("m_tit      = %s\n", m_tit);
#	endif

	switch (aide_get_action ()) {

	    case AIDE_IS_EDIT_MENU : 
		lmt_edit_file (i_adr->this_file, "B_MENU", m_idf, " %s %s");
		break;

	    default :
		aide_nothing_to_do ();
	}
    }
}

/****************************************************************************************
 * Appel de l'editeur
 *
 ***************************************************************************************/
void lmt_edit_file (nomf, mot1, mot2, format)
char *nomf, *mot1, *mot2, *format;
{
    if (nomf[0] == 0)
	aide_affi_message ("No file to edit.");
    else {
	char bla[256];
	int num_ligne = (mot1[0] == 0) 
		? 0 : lmt_trouve_num_ligne (nomf, mot1, mot2, format);
	if (num_ligne == 0)
	    pcp_avertissement ("WARNING", "This text field was not found:", mot2);
	sprintf (bla, "Edit %s", nomf);
	aide_affi_edit (bla, nomf, num_ligne);
    }
}

/***************************************************************************************
 * Recherche de la premiere occurence de "mot1 mot2" avec un format tel que " %s %s"
 * en tenant compte des # et ?
 *
 * Renvoie le numero de ligne ou 0
 ***************************************************************************************/
int lmt_trouve_num_ligne (nomf, nom1, nom2, format)
char *nomf, *nom1, *nom2, *format;
{
    char ligne[LG_LIGNE], mot1[LG_LIGNE], mot2[LG_LIGNE];
    int num = 0, i;
    FILE *f;

    if ((f = fopen (nomf, "r")) == NULL) {
	fprintf (stderr, "ERROR can't open the file %s\n", nomf);
	return 0;
    }

    while ( fgets(ligne , LG_LIGNE , f ) != NULL ) {

	for (i = 0; ligne[i]; i++) {
	    if (ligne[i] == '#')
		ligne[i] = 0;
	  /*  else if (ligne[i]=='?' || ligne[i]=='"' || ligne[i]=='(' || ligne[i]==')')
		ligne[i] = ' '; */
	}
	if (sscanf (ligne, format, mot1, mot2) == 2)
	if (strcmp (nom1, mot1) == 0 && strcmp (nom2, mot2) == 0) {
	    fclose (f); return num;
	}
	num++;
    }

    fclose (f); return 0;
}


/***************************************************************************************
 * Message d'aide quand un menu root n'est pas trouve'
 *
 ***************************************************************************************/
void lmt_this_menu_empty (menu, menu_item)
Menu menu;
Menu_item menu_item;
{
    aide_affi_message (
"			THIS IS HOW TO GET THE MENUS \
\
\
In your home directory, create or update the \".ipsrc\" file, then run in menu [Extra] \
the [Reread menus] option. \
\
This is a .ipsrc example, with dummy paths : \
\
	L_MENU	GLOBAL	path1/ips.menu_global \
	L_MENU	PERSO	path2/ips.menu_perso \
	L_MENU	HELP	path3/ips.menu_help \
\
When [Extra][Reread Menus], IPS will search theses menu-files, in which it should find \
the \"MENU GLOBAL\", \"MENU PERSO\" and \"MENU HELP\" items. \
\
Warning : if IPS doesn't find the \"MENU\" items, you will get an error ! \
 \
"); 
}

