/***************************************************************************************/
/* tel_protoc.c                                               I.P.S  v 3.7  18/11/1996 */
/*                                                                                     */
/* Module du protocole IPS_SOCKET (udp/ip)               Edouard.Thiel@lim.univ-mrs.fr */
/***************************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdarg.h>

#include "attributs.h"
#include "tel_protoc.h"
#include "tel_oper.h"

#define DEBUG1_

#define VRAI 1
#define FAUX 0

#define TP_BUFSIZE	2048	/* Taille systeme des buffers des socket */
#define TP_GROSBUF	10240	/* Taille buffer reception pour envoi par blocs */
#define TP_GRAIN	1728	/* Taille d'un bloc (ancien=1792 plante Linux 2.0.0) */
#define TP_ENTETE	32	/* Taille entete message */
#define TP_ORDBLOC	32	/* Position bloc apres la commande d'envoi */

#define TP_ALRM_TMAX	300
#define TP_ALRM_WAIT	12
#define TP_ALRM_ACQUIT	2	
#define TP_ALRM_SPY	1	
#define TP_ALRM_ACMAX	180	

enum { F_TMAX, F_WAIT, F_ACQUIT, F_AGAIN, F_SPY };

/*
 * Variables globales
*/

static int	tp_sLoc, 			/* socket locale */
		tp_pLoc, tp_pFar,		/* port local et distant */
		tp_sizeSnd, tp_sizeRcv,		/* taille buffer systeme pr socket */
		tp_ctx,				/* numero contexte */
		tp_lgFar,			/* longueur adrFar */
		tp_Buf1Size,			/* taille totale dans tp_Buf1 */
		tp_Buf1_nb,			/* nombre blocs pour retrouver tp_Buf1 */
		tp_Buf1_tb,			/* taille blocs pour retrouver tp_Buf1 */
		tp_Buf2Size,			/* taille dans tp_Buf2 */
		tp_BufDrawSize,			/* taille buffer tp_BufDraw */
		tp_nm_in, tp_nm_out,		/* num dernier mess recu et envoye' */
		tp_alrm_flag,			/* flag : nature du timeout */
		tp_alrm_sec,			/* duree timeout */
		tp_sendto_len,			/* on conserve len du dernier sendto */
		tp_ord_suiv,			/* ordre suivant, pour envoi de buffers */
		tp_num_data,			/* numero du data courant pour section EXE */
		*tp_spy_adr,			/* adresse de l'entier, val de debut et */
		tp_spy_deb, tp_spy_fin,		/*   de fin pour tp_spy_start () */
		tp_time_size,			/* pour le taux de transfert */
		tp_short_pareil;		/* booleen : meme representation des short */
static struct sockaddr_in tp_adrFar;		/* adresse distante */
static char 	tp_bufSnd[TP_BUFSIZE], 		/* buffer pr envoi */
		tp_bufRcv[TP_BUFSIZE],		/* buffer pr reception */
		*tp_mSnd, *tp_mRcv,		/* pointeurs apres entete dans buffers */
		*tp_Adr1,			/* adr non libere, pour REP/SEND/BLOC/DONE */
		tp_Buf1[TP_GROSBUF],		/* gros buffer pour tp_encode_* */
		tp_Buf2[TP_BUFSIZE],		/* petit buffer */
		*tp_BufDraw,			/* buffer pour dessins dans section FIG */
		*tp_BufDrawP;			/* lieu prochaine ecriture */
static Tel_oper_p tp_r;				/* le struct general pour tel_oper() */
static double	tp_time_exec,			/* pour stocker le temps d'execution */
		tp_time_rate;			/* pour le taux de transfert */


/*
 * Parametres de la ligne de commande
*/

typedef struct { 
	char	nom[32];	/* nom du parametre */
	int	nb, 		/* nombre d'arguments */
		stop,		/* stopper apres ce parametre ? */
		pos;		/* position dans argc ; init a 0 */
	} Tp_params;

Tp_params tp_params[] = {
	{"-help" , 0, 0, 0},
	{"-ips"  , 2, 0, 0},	/* num port, num contexte */
	{"-host" , 1, 0, 0},	/* nom du host distant */
	};

#define TP_IS_HELP	tp_params[0].pos
#define TP_IS_IPS	tp_params[1].pos
#define TP_IS_HOST	tp_params[2].pos

#define TP_TOT_PARAMS (sizeof(tp_params) / sizeof(Tp_params))


/*
 * Declaration fonctions
*/

int tp_decode_params ();
void tp_init ();
int tp_creer_socket ();
void tp_quit ();
int tp_recvfrom ();
int tp_sendto ();
int tp_sendto_str ();
int tp_sendto_again ();
void tp_hand_signal ();
void tp_timeout ();
double tp_gettime ();
void tp_wr_str ();
void tp_wr_int ();
void tp_wr_dbl ();
void tp_wr_dblv ();
char *tp_rd ();
void tp_buf_affi ();
void tp_encode_def (int ordre, ...);
void tp_decode_cho ();
void tp_encode_cho ();
void tp_free_cho ();
void tp_decode_dim ();
void tp_encode_dim ();
void tp_prepare_tp_r ();
void tp_appels_fig ();
void tp_fig_draw (int ordre, ...);
void tp_fig_init_draw ();
void tp_fig_free_draw ();
void tp_fig_realloc_draw ();
void tp_spy_start ();
void tp_spy_send ();
void tp_short_adapt ();
void tp_dialogue ();

#define tp_short_echange(x) \
	((unsigned short)((((unsigned short)(x) & 0x00ff) << 8) | \
			  (((unsigned short)(x) & 0xff00) >> 8)))


/***************************************************************************************
 * Comptage des parametres dans la ligne de commande
 *
 ***************************************************************************************/
int tp_decode_params (argc, argv, params, tot_params)
int argc;
char *argv[];
Tp_params params[];
int tot_params;
{
    int c, i, trouve, n = 0;

    /* Recherche des params */
    for (c = 1; c < argc; c++) {
	trouve = FAUX;

	for (i = 0; i < tot_params; i++)
	if (!strcmp(params[i].nom, argv[c])) {

	    if (params[i].pos) {
		printf ("ERROR: `%s' already set\n", params[i].nom);
		return -1;
	    }

	    params[i].pos = c;
	    c += params[i].nb;

	    if (c >= argc && params[i].nb > 0) {
		printf ("ERROR: `%s' needs %d args\n", params[i].nom, params[i].nb);
		return -1;
	    }

	    trouve = VRAI; n++; 
	    if (params[i].stop)
		return n;
	    break;
	}

	if (!trouve) {
	    printf ("ERROR: `%s' unknown\n", argv[c]);
	    return -1;
	}
    }

    return n;
}



/***************************************************************************************
 * Initialisation generale
 *
 * APPEL : machine = "" en localhost
 ***************************************************************************************/
void tp_init (port, ctx, machine)
int port, ctx;
char *machine;
{
    struct sigaction act;

    tp_ctx = ctx;
    tp_nm_in = tp_nm_out = 0;
    tp_mRcv = tp_bufRcv+TP_ENTETE;
    tp_mSnd = tp_bufSnd+TP_ENTETE;

    if (tp_creer_socket () < 0) exit (0);

    /* Preparation de l'adresse */
    memset ((char *) &tp_adrFar, 0, sizeof (tp_adrFar));
    tp_lgFar = sizeof(tp_adrFar);
    tp_pFar = port;
    tp_adrFar.sin_port = htons (tp_pFar);
    tp_adrFar.sin_family = AF_INET;

    /* Recherche de l'adresse internet de la machine */
    if (machine[0] == 0)
	tp_adrFar.sin_addr.s_addr = INADDR_ANY;  /* localhost */
    else {
	struct hostent *hp;

	if ((hp = gethostbyname (machine)) == NULL) {
	    fprintf (stderr, "`%s' : Host unknown\n", machine);
	    exit (0);
	}
	memcpy (&tp_adrFar.sin_addr, hp->h_addr, hp->h_length);
    }

    /* representation des short identique entre machine locale et distante */
    tp_short_pareil = VRAI;

    /* mise en place signaux et timeout 
     * ATTENTION : signal() n'agit pas de la meme facon sous BSD, sysV et POSIX !!
     * REMEDE : utiliser sigaction()
     * CE QU'ON VEUT : le signal recu est masque', le handler est conserve'
     * et les appels systeme (bloquants) sont repris
    */
#   ifndef SA_RESTART
#   define SA_RESTART 0x0000	/* Pour Sun 4.1.3 */
#   endif
    sigemptyset (&act.sa_mask);
    act.sa_handler = tp_hand_signal;
    act.sa_flags = SA_RESTART;
    sigaction (SIGALRM, &act, NULL);
    sigaction (SIGTERM, &act, NULL);
    tp_timeout (F_TMAX);
}


/***************************************************************************************
 * Creation de la socket
 *
 ***************************************************************************************/
int tp_creer_socket ()
{
    int lg;
    struct sockaddr_in addr;

    /* Creation de la socket */
    if ((tp_sLoc = socket (AF_INET, SOCK_DGRAM, 0)) == -1)  {
	perror ("Can't create socket");
	return -1;
    }

    /* Preparation de l'adresse */
    memset ((char *) &addr, 0, sizeof (addr));
    addr.sin_port = 0;
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = INADDR_ANY;

    if (bind (tp_sLoc, (struct sockaddr *) &addr, sizeof(addr))) {
	perror ("Can't bind socket");
	return -1;
    }

    lg = sizeof (addr);
    if (getsockname (tp_sLoc, (struct sockaddr *) &addr, &lg)) {
	perror ("Can't get socket name");
	return -1;
    }

    tp_pLoc = ntohs (addr.sin_port);
#   ifdef DEBUG
    printf ("tel%d Numero du port = %d\n", tp_ctx, tp_pLoc);
#   endif

    /* propriete socket : Rifflet p.287 */
    tp_sizeSnd = tp_sizeRcv = TP_BUFSIZE;
    lg = sizeof (tp_sizeSnd);
    setsockopt (tp_sLoc, SOL_SOCKET, SO_SNDBUF, &tp_sizeSnd, lg);
    setsockopt (tp_sLoc, SOL_SOCKET, SO_RCVBUF, &tp_sizeRcv, lg);
    getsockopt (tp_sLoc, SOL_SOCKET, SO_SNDBUF, &tp_sizeSnd, &lg);
    getsockopt (tp_sLoc, SOL_SOCKET, SO_RCVBUF, &tp_sizeRcv, &lg);

    return 0;
}


/***************************************************************************************
 * Suppression de la socket
 *
 ***************************************************************************************/
void tp_quit ()
{
    sprintf (tp_mSnd, "37"); /* BYE */
    tp_sendto_str (); /* On l'envoie 2 fois, c'est plus prudent */
    sleep (1);
    tp_sendto_str ();
    sleep (2); /* on attend un peu, pour que IPS puisse lire les BYE sans pb */

    close (tp_sLoc); 
#   ifdef DEBUG1
    printf ("tel%d Fermeture socket %d\n", tp_ctx, tp_pLoc);
#   endif
}


/***************************************************************************************
 * Reception message (bloquante)
 *
 * On teste le header du message.
 * On filtre les anciens messages avec tp_nm_in.
 *
 * BUG corrige' le 18/11/96 sur IRIX : auparavant on faisait l'appel bloquant : 
 *       recvfrom ( ..., &tp_lgFar)
 * avec tp_lgFar en global. 
 * PB pendant un SIGALRM, on interromp cet appel pour aller faire : 
 *       sendto ( ..., tp_lgFar)
 * Or de cet endroit, tp_lgFar, qui est un argument de l'appel bloquant recvfrom,
 * est temporairement mis a 0 par le systeme, d'ou l'erreur EINVAL dans l'appel
 * a sendto.
 * Solution : utilisation d'une variable tampon l_adr, qui est ensuite perdue
 * (le resultat de l_adr n'est d'aucune utilite')
 ***************************************************************************************/
int tp_recvfrom ()
{
    int n, k, nm_lu, l_adr = tp_lgFar;

    while (1) { /* Pour revenir sur recvfrom quand un message est ignore' */

	n = recvfrom (tp_sLoc, tp_bufRcv, TP_BUFSIZE, 0,
	    (struct sockaddr *) &tp_adrFar, &l_adr);

#	 ifdef DEBUG1
	 printf ("tel%d < IPS : %s\n", tp_ctx, tp_mRcv);
#	 endif

	if (n < 0) {
	    perror ("tel: ERROR recvfrom");
	} else if (sscanf (tp_bufRcv, "IPS %d %d", &k, &nm_lu) != 2) {
	    printf ("tel: ERROR recvfrom, bad header\n");
	} else if (n != k) {
	    printf ("tel: ERROR recvfrom, bad message size\n");
	} else if (nm_lu > tp_nm_in) {
	    /* Ce message n'a jamais ete recu */
	    tp_nm_in = nm_lu;
	    return n; /* ok */
	} else {
	    /* Ce message a deja ete recu.
	     * On ne fait rien, pour eviter un dedoublement du dialogue.
	     * On retourne au recvfrom bloquant.
	    */
#	    ifdef DEBUG
	    printf ("tel: BINGO, le message %d est ancien\n", nm_lu);
#	    endif
	    continue;
	}
	return -1;
    }
}


/***************************************************************************************
 * Emission message (non bloquante)
 *
 * Ecriture de l'entete et increment de tp_nm_out.
 *
 ***************************************************************************************/
int tp_sendto (len)
int len;
{
    int n;

#   ifdef DEBUG1
    printf ("tel%d > IPS : %s\n", tp_ctx, tp_mSnd);
#   endif

    tp_sendto_len = len += TP_ENTETE;
    sprintf (tp_bufSnd, "TEL %5d %5d %10d ", tp_ctx, len, ++tp_nm_out);

    n = sendto (tp_sLoc, tp_bufSnd, len, 0, 
	(struct sockaddr *) &tp_adrFar, tp_lgFar);

    if (n < 0) perror ("ERROR sendto");

    return n;
}


/***************************************************************************************
 * Idem tp_sendto + calcul taille message automatique (valable en cas de string).
 *
 ***************************************************************************************/
int tp_sendto_str ()
{
    return tp_sendto (strlen (tp_mSnd) + 1);
}


/***************************************************************************************
 * Re'envoie le dernier message, sans modifier l'entete.
 *
 ***************************************************************************************/
int tp_sendto_again ()
{
    int n = sendto (tp_sLoc, tp_bufSnd, tp_sendto_len, 0, 
	(struct sockaddr *) &tp_adrFar, tp_lgFar);

    if (n < 0) perror ("ERROR sendto");

    return n;
}


/***************************************************************************************
 * Callback des signaux
 *
 ***************************************************************************************/
void tp_hand_signal (sig)
int sig;
{
    if (sig == SIGTERM) {
#	ifdef DEBUG1
	printf ("tel%d SIGTERM : QUIT\n", tp_ctx);
#	endif
	tp_quit ();
	exit (0);
    }

    else if (sig == SIGALRM) switch (tp_alrm_flag) {

	case F_ACQUIT :
	case F_AGAIN :
	    if (tp_alrm_sec >= TP_ALRM_ACMAX) tp_hand_signal (SIGTERM);
#	    ifdef DEBUG1
	    printf ("tel%d SIGALRM : SEND AGAIN ==============\n", tp_ctx);
#	    endif
	    tp_sendto_again ();
	    tp_timeout (F_AGAIN);
	    break;
	case F_TMAX :
	case F_WAIT :
#	    ifdef DEBUG1
	    printf ("tel%d SIGALRM : CIAO\n", tp_ctx);
#	    endif
	    sprintf (tp_mSnd, "31"); /* CIAO */
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	    break;
	case F_SPY :
	    tp_spy_send ();
	    tp_timeout (F_SPY);
	    break;
	default :
#	    ifdef DEBUG1
	    printf ("tel%d SIGALRM : ERROR, FLAG = %d\n", tp_ctx, tp_alrm_flag);
#	    endif
	    break;
    }
}


/***************************************************************************************
 * Fixe nature timeout et duree associe'e
 *
 * tp_timeout (F_TMAX)		limite vie infinie
 * tp_timeout (F_WAIT)		en etat d'attente avant suicide
 * tp_timeout (F_ACQUIT)	demande 1er acquittement
 * tp_timeout (F_AGAIN)		acquittements suivants
 * tp_timeout (F_SPY)		calcule pourcentage execution
 ***************************************************************************************/
void tp_timeout (flag)
int flag;
{
    switch (tp_alrm_flag = flag) {

	case F_TMAX :	tp_alrm_sec = TP_ALRM_TMAX;	alarm (tp_alrm_sec);	break;
	case F_WAIT :	tp_alrm_sec = TP_ALRM_WAIT;	alarm (tp_alrm_sec);	break;
	case F_ACQUIT :	tp_alrm_sec = TP_ALRM_ACQUIT;	alarm (tp_alrm_sec);	break;
	case F_AGAIN :	tp_alrm_sec += TP_ALRM_ACQUIT;	alarm (TP_ALRM_ACQUIT);	break;
	case F_SPY :	tp_alrm_sec = TP_ALRM_SPY;	alarm (tp_alrm_sec);	break;
	default : printf ("tel%d timeout : ERROR, FLAG = %d\n", tp_ctx, tp_alrm_flag);
    }

#   ifdef DEBUGA
    printf ("timeout = %d   flag = %d\n", tp_alrm_sec, tp_alrm_flag);
#   endif
}


/***************************************************************************************
 * calcule nb_sec.nb_micro_sec de l'instant
 *
 ***************************************************************************************/
double tp_gettime ()
{
  struct timeval tp;

  if (gettimeofday (&tp, (struct timezone *) NULL) == 0)
       return (double) tp.tv_sec + (double) tp.tv_usec / (double) 1E6;
  else return (double) 0;
}


/****************************************************************************************
 * Ecriture dans un buffer ; utilise'par tp_encode_* ()
 * Les strings sont range's a la queue leu leu, separes par des '\0'
 *
 * APPEL :
 *	tp_wr_str (&p, "salut");
 *	tp_wr_int (&p, 1234);
 *	tp_wr_dbl (&p, 3.14);
 *	tp_wr_dblv (&p, "%.2f", 3.14);
 *
 * RQ : sur Linux, sprintf return strlen ; sur Sun, sprintf return s !!
 ****************************************************************************************/
void tp_wr_str (p, s)
char **p, *s;
{
    sprintf (*p, "%s", s);
    *p += 1 + strlen (*p);
}

void tp_wr_int (p, d)
char **p;
int d;
{
    sprintf (*p, "%d", d);
    *p += 1 + strlen (*p);
}

void tp_wr_dbl (p, f)
char **p;
double f;
{
    sprintf (*p, "%f", f);
    *p += 1 + strlen (*p);
}

void tp_wr_dblv (p, u, f)
char **p, *u;
double f;
{
    sprintf (*p, u, f);
    *p += 1 + strlen (*p);
}

/****************************************************************************************
 * Lecture dans un buffer ; utilise' dans tp_decode_* ().
 * Les strings sont range's a la queue leu leu, separes par des '\0'
 *
 * APPEL :
 *    char   *ligne = tp_rd (&p);
 *    int     x = atoi (tp_rd (&p));
 *    double  y = atof (tp_rd (&p));
 *
 ****************************************************************************************/
char *tp_rd (p)
char **p;
{
    char *q = *p;
    *p += 1 + strlen (*p);
    return q;
}


/****************************************************************************************
 * Affichage d'un buffer
 * Les strings sont range's a la queue leu leu, separes par des '\0'
 * Les '\0' sont repre'sente's par des '~'
 *
 ****************************************************************************************/
void tp_buf_affi (buf, bufsize)
char *buf;
int bufsize;
{
    int i;

    printf ("Taille = %d\n", bufsize);
    printf ("DEBUT------------\n");
    { int i; for (i = 0; i < bufsize; i++) 
	if (buf[i] == '\0') putchar ('~');
	else putchar (buf[i]); }
    printf ("\nFIN--------------\n");
}

/****************************************************************************************
 * Encodage des sections DEF dans tel_oper ()
 *
 ****************************************************************************************/
void   tp_encode_def (int ordre, ...)
{
    va_list  ap;
    char     *p;

    /* init */
    
    p = tp_Buf1; tp_Buf1Size = 0;

    /* lecture arguments */
    for ( va_start (ap, ordre);
	  ordre != 0 && p < tp_Buf1 + TP_GROSBUF;
	  ordre = va_arg (ap, int) )
    switch (ordre) { 

	case IPS_NAME :
	case IPS_IDF :
	    tp_wr_int (&p, ordre);
	    tp_wr_str (&p, va_arg (ap, char *));
	break;

	case IPS_NAME2 :
	    tp_wr_int (&p, IPS_NAME);
	    tp_wr_str (&p, va_arg (ap, char *)); p--; /* suppr le '\0' pour concatener */
	    tp_wr_str (&p, va_arg (ap, char *));
	break;

	case IPS_IMAGE_IN  : 
	case IPS_IMAGE_OUT :
	    tp_wr_int (&p, ordre);
	    tp_wr_int (&p, va_arg (ap, int));    /* num */
	    tp_wr_str (&p, va_arg (ap, char *)); /* legende */
	break;

	case IPS_FIG_IMAGE :
	    tp_wr_int (&p, ordre);
	    tp_wr_int (&p, va_arg (ap, int));    /* num_fig */
	    tp_wr_int (&p, va_arg (ap, int));    /* num_image */
	    tp_wr_str (&p, va_arg (ap, char *)); /* legende */
	break;

	case IPS_BORDER_ANY :
	case IPS_BORDER_POSIT :
	case IPS_BORDER_NOONE :
	    tp_wr_int (&p, ordre);
	break;

	case IPS_BORDER :
	case IPS_SAME_SIZE :
	    tp_wr_int (&p, ordre);
	    tp_wr_int (&p, va_arg (ap, int));
	break;

	case IPS_PAR_REAL    : 
	case IPS_PAR_CHK_BOX :
	case IPS_PAR_SLIDER  :
	case IPS_PAR_INT     : 
	case IPS_PAR_TEXT    : 
	case IPS_PAR_CH_ABR  :
	case IPS_PAR_CHOICE  :

	    tp_wr_int (&p, ordre);
	    tp_wr_int (&p, va_arg (ap, int));    /* num */
	    tp_wr_str (&p, va_arg (ap, char *)); /* legende */

	    switch (ordre) { /* #2 */

		case IPS_PAR_REAL :
		    tp_wr_str (&p, va_arg (ap, char *)); /* borne_inf */
		    tp_wr_str (&p, va_arg (ap, char *)); /* borne_sup */
		    tp_wr_str (&p, va_arg (ap, char *)); /* val_defaut */ 
		break;

		case IPS_PAR_SLIDER :
		case IPS_PAR_INT :
		    tp_wr_int (&p, va_arg (ap, int)); /* borne_inf */
		    tp_wr_int (&p, va_arg (ap, int)); /* borne_sup */
		    tp_wr_int (&p, va_arg (ap, int)); /* val_defaut */
		break;

		case IPS_PAR_CHK_BOX :
		    tp_wr_int (&p, va_arg (ap, int)); /* val_defaut */
		break;

		case IPS_PAR_TEXT :
		    tp_wr_int (&p, va_arg (ap, int));    /* taille_max */
		    tp_wr_str (&p, va_arg (ap, char *)); /* val_defaut */ 
		break;
 
		case IPS_PAR_CH_ABR :
		case IPS_PAR_CHOICE : {
		    char *nom;

		    tp_wr_int (&p, va_arg (ap, int)); /* val_defaut */

		    while ((nom = va_arg (ap, char *)) != (char *) NULL) {
			tp_wr_str (&p, nom); 
		    }
		    tp_wr_str (&p, ""); /* fin */
		} break;
 
	    } /* switch (ordre) #2 */
        break;

	default : printf ("ERROR 107 in section DEF: syntax\n");

    } /* switch (ordre) #1 */

    /* fin lecture args */
    va_end (ap);

    /* NULL final */
    tp_wr_str (&p, "0");

    /* taille buffer */
    tp_Buf1Size = p - tp_Buf1;
    if (tp_Buf1Size > TP_GROSBUF) {
	printf ("ERROR 109 in section DEF: definition too large\n");
	tp_Buf1Size = 0;
    }

    /* CONTROLE */
#   ifdef DEBUG
    tp_buf_affi (tp_Buf1, tp_Buf1Size);
#   endif
}


/****************************************************************************************
 * Decodage des params pour la section CHOICE, encode'e par telop_encode_cho
 *
 ****************************************************************************************/
void tp_decode_cho ()
{
    char *p;
    int ordre, num;

    /* init */
    p = tp_Buf1;
    memset (&tp_r, 0, sizeof(tp_r));

    /* lecture arguments */
    while ((ordre = atoi (tp_rd (&p))) != 0) switch (ordre) {

	case IPS_PAR_REAL :
	    tp_r.param_type[num = atoi (tp_rd (&p))+1] = ordre;
	    tp_r.param_adr[num] = (double *) malloc (sizeof(double));
	    *((double *) tp_r.param_adr[num]) = atof (tp_rd (&p));
	    break;
	case IPS_PAR_TEXT :
	    tp_r.param_type[num = atoi (tp_rd (&p))+1] = ordre;
	    tp_r.param_adr[num] = (char *) malloc (atoi (tp_rd (&p))+1);
	    strcpy ((char *) tp_r.param_adr[num], tp_rd (&p));
	    break;
	case IPS_PAR_CHK_BOX :
	case IPS_PAR_SLIDER :
	case IPS_PAR_INT :
	case IPS_PAR_CH_ABR :
	case IPS_PAR_CHOICE :
	    tp_r.param_type[num = atoi (tp_rd (&p))+1] = ordre;
	    (int) tp_r.param_adr[num] = atoi (tp_rd (&p));
	    break;
	default : 
	    printf ("tel_oper : encoding ERROR in section CHOICE\n");
	    return;
    }  
}

/****************************************************************************************
 * Encodage des params et elem. grise's dans la section CHOICE
 *
 ****************************************************************************************/
void tp_encode_cho ()
{
    char *p;
    int i;

    /* prologue */
    p = tp_Buf1; tp_Buf1Size = 0;

    /* encodage des parametres modifie's (par_set) */
    for (i = 1; i < OPER_VECM; i++) 
    if (tp_r.param_type[i] != 0 && tp_r.par_set[i]) {

	tp_wr_int (&p, tp_r.param_type[i]);  /* ordre toujours != 0 final */
	tp_wr_int (&p, i-1);		     /* numero */

	switch (tp_r.param_type[i]) {

	    case IPS_PAR_REAL :
		tp_wr_dbl (&p, *((double *) tp_r.param_adr[i]));
		break;
	    case IPS_PAR_TEXT : 
	        tp_wr_str (&p, (char *) tp_r.param_adr[i]);
		break;
	    case IPS_PAR_CHK_BOX :
	    case IPS_PAR_SLIDER :
	    case IPS_PAR_INT : 
	    case IPS_PAR_CH_ABR :
	    case IPS_PAR_CHOICE : 
		tp_wr_int (&p, (int) tp_r.param_adr[i]);
		break;
	}
    }

    /* encodage des grise's modifie's */
    for (i = 1; i < OPER_VECM; i++) 
    if (tp_r.param_type[i] != 0 && tp_r.inactif[i] > 0) {
	tp_wr_int (&p, IPS_PAR_ACTIVE);   /* ordre toujours != 0 final */
	tp_wr_int (&p, i-1);		  /* numero */
	tp_wr_int (&p, tp_r.inactif[i]);
    }

    /* NULL final */
    tp_wr_str (&p, "0");

    /* taille buffer */
    tp_Buf1Size = p - tp_Buf1;
    if (tp_Buf1Size > TP_GROSBUF) {
	printf ("ERROR 109 in section DEF: definition too large\n");
	tp_Buf1Size = 0;
    }

    /* CONTROLE */
#   ifdef DEBUG
    tp_buf_affi (tp_Buf1, tp_Buf1Size);
#   endif
}

/****************************************************************************************
 * Libere les params dans la section CHOICE
 *
 ****************************************************************************************/
void tp_free_cho ()
{
    int i;

    for (i = 1; i < OPER_VECM; i++) 
    switch (tp_r.param_type[i]) {
	    case IPS_PAR_REAL :
	    case IPS_PAR_TEXT :
		free (tp_r.param_adr[i]); tp_r.param_adr[i] = NULL;
		break; 
    }
}

/****************************************************************************************
 * Decodage des infos pour la section DIM, encode'e par telop_encode_dim
 *
 ****************************************************************************************/
void tp_decode_dim ()
{
    char *p;
    int ordre, num;

    /* init : on conserve les params qui sont deja dans tp_r */
    p = tp_Buf1;

    /* lecture arguments */
    while ((ordre = atoi (tp_rd (&p))) != 0) switch (ordre) {

	case IPS_IMAGE_INFOS :
	    tp_r.ima_n = atoi (tp_rd (&p));
	    tp_r.ima_cmin = atoi (tp_rd (&p));
	    break;

	case IPS_IMAGE :
	    num = atoi (tp_rd (&p));
	    tp_r.ima_x[num] = atoi (tp_rd (&p));
	    tp_r.ima_y[num] = atoi (tp_rd (&p));
	    break;
	default : 
	    printf ("tel_oper : encoding ERROR in section DIM\n");
	    return;
    }  
}

/****************************************************************************************
 * Encodage des infos de la section DIM
 *
 ****************************************************************************************/
void tp_encode_dim ()
{
    char *p;
    int i;

    /* prologue */
    p = tp_Buf1; tp_Buf1Size = 0;

    /* encodage des tailles des images */
    for (i = 1; i < OPER_VECM; i++) 
    if (tp_r.ima_x[i] || tp_r.ima_y[i]) {

	tp_wr_int (&p, IPS_IMAGE);  		/* ordre toujours != 0 final */
	tp_wr_int (&p, i);			/* numero */

	tp_wr_int (&p, tp_r.ima_x[i]);		/* x, y */
	tp_wr_int (&p, tp_r.ima_y[i]);

	tp_wr_int (&p, tp_r.ima_lut[i]);	/* lut */

	tp_r.ima_name[i][IMAGE_CH1LEN] = 0;	/* par securite */
	tp_wr_str (&p, tp_r.ima_name[i]);	/* nom */
    }

    /* NULL final */
    tp_wr_str (&p, "0");

    /* taille buffer */
    tp_Buf1Size = p - tp_Buf1;
    if (tp_Buf1Size > TP_GROSBUF) {
	printf ("ERROR 109 in section DEF: definition too large\n");
	tp_Buf1Size = 0;
    }

    /* CONTROLE */
#   ifdef DEBUG
    tp_buf_affi (tp_Buf1, tp_Buf1Size);
#   endif
}


/****************************************************************************************
 * preparation de tp_r : cmin, xytot, .. (a virer dans l'avenir)
 *
 ****************************************************************************************/
void tp_prepare_tp_r ()
{
    if (tp_r.ima_n > 0) {
	tp_r.cmin = tp_r.ima_cmin;
	tp_r.xsiz = tp_r.ima_x[1];
	tp_r.ysiz = tp_r.ima_y[1];

	tp_r.xmax = tp_r.xsiz + tp_r.cmin;
	tp_r.xtot = tp_r.xsiz + tp_r.cmin*2;
	tp_r.ymax = tp_r.ysiz + tp_r.cmin;
	tp_r.ytot = tp_r.ysiz + tp_r.cmin*2;
	tp_r.xytot = tp_r.xtot * tp_r.ytot;
    }      
}

/****************************************************************************************
 * Decodage des infos pour la section FIG, preparation de tp_r
 * et appels multiples de tel_oper
 *
 ****************************************************************************************/
void tp_appels_fig (idf, buf)
char *idf, *buf;
{
    int dp, n;

    while (*buf) { /* Condition d'arret insuffisante car on ne lit pas char a char */

	tp_r.fig_event = 0; /* init pour condition d'arret */
	n = sscanf (buf, " %d %d%n", &tp_r.fig_event, &tp_r.fig_num, &dp);
	buf += dp;
	if (n < 2) return; /* securite */

	switch (tp_r.fig_event) {
	    case FIG_REFRESH :
	    case FIG_DONE :
		break;

	    case FIG_PUSH :
	    case FIG_RELEASE :
	    case FIG_DRAG :
	    case FIG_CANCEL : 
		sscanf (buf, " %d %d %lf %lf%n", 
		    &tp_r.fig_butt, &tp_r.fig_zoom, &tp_r.fig_x, &tp_r.fig_y, &dp);
		buf += dp;
		break;

	    default : return; /* Condition d'arret normale avec ev = 0 */
	}

	tel_oper (IPS_ACT_FIG, idf, &tp_r);
    }
}

/****************************************************************************************
 * Dessins dans la section FIG
 *
 ****************************************************************************************/
void tp_fig_draw (int ordre, ...)
{
    va_list  ap;

    /* lecture arguments */
    va_start (ap, ordre);

    /* Ecriture ordre et numero figure */
    tp_wr_int (&tp_BufDrawP, ordre);					/* ordre != 0 */
    tp_wr_int (&tp_BufDrawP, va_arg (ap, int));				/* num figure */

    switch (ordre) { 
	case FIG_CLEAN :
	case FIG_DO_RUN :
	    break;
	case FIG_COLOR :
	case FIG_MODE :
	    tp_wr_int (&tp_BufDrawP, va_arg (ap, int));			/* color ou mode */
	    break;
	case FIG_DRAW_LINE :
	case FIG_DRAW_RECT :
	case FIG_FILL_RECT :
	case FIG_DRAW_ELLIP :
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* coordonnees */
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    break;
	case FIG_DRAW_TRIANG :
	case FIG_FILL_TRIANG :
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* coordonnees */
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    break;
	case FIG_DRAW_ARC :
	case FIG_FILL_ARC :
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* coordonnees */
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_int  (&tp_BufDrawP, va_arg (ap, int));		/* angles */
	    tp_wr_int  (&tp_BufDrawP, va_arg (ap, int));		/* 0 .. 360*64 */
	    break;
	case FIG_DRAW_CIRC :
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* coordonnees */
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* rayon */
	    break;
	case FIG_DRAW_POINT :
	case FIG_DRAW_CROSS :
	case FIG_DRAW_CURSOR :
	case FIG_DRAW_MARK :
	case FIG_CENTER :
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* coordonnees */
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    break;
	case FIG_DRAW_LABEL :
	case FIG_DRAW_VALUE :
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));	/* coordonnees */
	    tp_wr_dblv (&tp_BufDrawP, "%.2lf", va_arg (ap, double));
	    tp_wr_int  (&tp_BufDrawP, va_arg (ap, int));		/* label */
	    break;
    }

    /* fin lecture args, et test buffer */
    va_end (ap);
    tp_fig_realloc_draw ();
}


/****************************************************************************************
 * Gestion du buffer de dessins dans la section FIG
 *
 * tp_fig_realloc_draw() assure qu'on a toujours 1024 octets disponibles devant soi.
 ****************************************************************************************/
void tp_fig_init_draw ()
{
    tp_BufDrawP = tp_BufDraw = (char *) malloc (2048);
    tp_BufDrawSize = (tp_BufDraw == NULL) ? 0 : 2048;
}

void tp_fig_free_draw ()
{
    if (tp_BufDraw != NULL) free (tp_BufDraw);
    tp_BufDrawP = tp_BufDraw = NULL;
    tp_BufDrawSize = 0;
}

void tp_fig_realloc_draw ()
{
    if (tp_BufDraw + tp_BufDrawSize < tp_BufDrawP + 1024) {

	/* On ajoute 2048 pour etre tranquile un bon moment */
	char *nouv = realloc (tp_BufDraw, tp_BufDrawSize + 2048);

#	ifdef DEBUG
	printf ("REALLOC DRAW size = %d  %s\n", 
	    tp_BufDrawSize+2048, (nouv == NULL) ? "NULL" : "OK");
#	endif

	if (nouv == NULL) return;
	tp_BufDrawP += nouv - tp_BufDraw;
	tp_BufDraw = nouv;
	tp_BufDrawSize += 2048;
    }
}

/****************************************************************************************
 * Ordonne le calcul chaque seconde du pourcentage de l'execution;
 *
 * Provoque chaque seconde l'appel de tp_spy_send ()
 *
 ****************************************************************************************/
void tp_spy_start (adr_valeur, val_debut, val_fin)
int *adr_valeur, val_debut, val_fin;
{
    tp_spy_adr = adr_valeur;
    tp_spy_deb = val_debut;
    tp_spy_fin = val_fin;
    sprintf (tp_mSnd, "399 0");
    tp_sendto_str ();
    tp_timeout (F_SPY);
}

/****************************************************************************************
 * Lit chaque seconde le contenu de adr_valeur, et envoie a IPS le pourcentage calcule'
 * a partir de val_debut et val_fin. IPS met a jour la LED de l'operateur.
 *
 ****************************************************************************************/
void tp_spy_send ()
{
    int r = 0, d = tp_spy_fin - tp_spy_deb;

    if (d != 0) r = ((*tp_spy_adr - tp_spy_deb) *100) / d;
    if (r < 0) r = 0;
    else if (r > 100) r = 100;

    sprintf (tp_mSnd, "399 %d", r);
    tp_sendto_str ();
}

/****************************************************************************************
 * Adapte le format des short pour toutes les datas
 *
 * Rq on pourrait faire l'economie de re'adapter les images IN alors qu'on
 *    ne renvoie que les OUT vers IPS ...
 ****************************************************************************************/
void tp_short_adapt (pareil)
int pareil;
{
    int n;

    if (pareil) return;

    for (n = 1; n <= tp_r.ima_n; n++) {

	/* cas d'une image */
	int t,
	    xytot = (tp_r.ima_x[n]+2*tp_r.ima_cmin) *
		    (tp_r.ima_y[n]+2*tp_r.ima_cmin) ;

	for (t = 0; t < xytot; t++)
	    tp_r.pix[n][t] = tp_short_echange (tp_r.pix[n][t]);
    }
}


/***************************************************************************************
 * Dialogue
 *
 ***************************************************************************************/
void tp_dialogue ()
{
    int len, motd, ordre;

    /* Premier contact */
    sprintf (tp_mSnd, "11 %d %s", getpid (), TP_NUM_PROTOCOL); /* HELO */
    if (tp_sendto_str () < 0) return;
    tp_timeout (F_ACQUIT);

    do {

	/*
	 * attente reponse bloquante
	*/
	if (tp_recvfrom () < 0) return; /* erreur */
	tp_timeout (F_TMAX); /* limite vie infinie */
	if (sscanf (tp_mRcv, " %d%n", &ordre, &motd) != 1) return; /* traitement */

	switch (ordre) { /* switch #1 */


	/* ------------------- C O M M A N D E S - G E N E R A L E S -------------- SHORT
	 * "SHORT a b"
	 *
	 * IPS envoie un test pour etablir si la representation des short 
	 * est la meme entre la machine locale et la machine distante.
	 * En retour : demande la requete.
	*/
	case 14 : {

	    int a, b;

	    sscanf (tp_mRcv+motd, " %d %d", &a, &b);
	    tp_short_pareil = (ntohs(b) == a);
#	    ifdef DEBUG1
	    printf ("tel: format short pareil : %d\n", tp_short_pareil);
#	    endif
	    sprintf (tp_mSnd, "%d", ordre+3);
	    tp_sendto_str ();
	    tp_timeout (F_WAIT);
	} break;


	/* ------------------------------------------------------------------------ WAIT
	 * "WAIT"
	 *
	 * IPS donne l'ordre de mettre le timeout pour quitter
	 * Pas de retour.
	*/
	case 22 : {

	    tp_timeout (F_WAIT);
	} break;

	/* ------------------------------------------------------------------------ DODO
	 * "DODO"
	 *
	 * Variante de WAIT avec un timeout long
	 * IPS envoie DODO quand il fait qqchose de long et qu'il rappelera plus tard.
	 * Pas de retour.
	*/
	case 24 : {

	    tp_timeout (F_TMAX);
	} break;

	/* ------------------------------------------------------------------------ QUIT
	 * "QUIT"
	 *
	 * IPS donne l'ordre de quitter immediatement.
	 * Pas de retour (sauf BYE par tp_quit).
	*/
	case 34 : {

	    return;
	} break;

	/* -------------------------------------------------------------------------- REP
	 * "REP nb_bloc taille_bloc taille_totale"
	 *
	 * IPS donne infos sur buffer
	 * En retour : demande du premier bloc
	*/
	case 60 : {

	    /* preparation du buffer tp_Buf1 (il est statique) */
	    sscanf (tp_mRcv+motd, " %d %d %d",
		&tp_Buf1_nb, &tp_Buf1_tb, &tp_Buf1Size);
	    tp_Adr1 = tp_Buf1; /* Le BLOC:66 travaille avec tp_Adr1 */

	    sprintf (tp_mSnd, "%d 0", ordre+3);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ------------------------------------------------------------------------ SEND
	 * "SEND num_bloc"
	 *
	 * IPS demande l'envoi du bloc num_bloc
	 * Attention il faut que tp_Adr1 = qqchose, par ex. = tp_Buf1
	 * De meme la taille doit avoir ete stockee dans tp_Buf1Size.
	 * En retour : BLOC num taille bloc
	*/
	case 52 : {

	    int nb, t;

	    sscanf (tp_mRcv+motd, " %d", &nb);

	    t = (TP_GRAIN*(nb+1) > tp_Buf1Size) ? tp_Buf1Size-TP_GRAIN*nb : TP_GRAIN;
	    sprintf (tp_mSnd, "%d %d %d ", ordre+3, nb, t);
	    /* Attention il faut que strlen (tp_mSnd) < TP_ORDBLOC */
	    memcpy (tp_mSnd+TP_ORDBLOC, tp_Adr1+TP_GRAIN*nb, t);
	    tp_sendto (TP_ORDBLOC+t);
	    tp_timeout (F_ACQUIT);
	} break;

	/* ------------------------------------------------------------------------ BLOC
	 * "BLOC num taille bloc"
	 *
	 * IPS envoie un bloc
	 * Attention il faut que tp_Adr1 = qqchose, par ex. = tp_Buf1
	 * En retour : demande du bloc suivant (ordre-3) ou stop (ordre+3)
	*/
	case 66 : {

	    int n, t;

	    sscanf (tp_mRcv+motd, " %d %d", &n, &t);
	    memcpy (tp_Adr1+tp_Buf1_tb*n, tp_mRcv+TP_ORDBLOC, t);

	    /* Demande bloc suivant */
	    if (n+1 < tp_Buf1_nb)
		 sprintf (tp_mSnd, "%d %d", ordre-3, n+1);
	    else sprintf (tp_mSnd, "%d", ordre+3);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ------------------------------------------------------------------------ DONE
	 * "DONE"
	 *
	 * IPS dit qu'il a tout recu
	 * En retour : ord_suiv
	*/
	case 58 : {

	    /* Ici liberation de la memoire : pas besoin car static */

	    /* On appelle ord_suiv */
	    sprintf (tp_mSnd, "%d", tp_ord_suiv);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;


	/* --------------- C O M M A N D E S - D E - S E C T I O N ------------- DEF_ACT
	 * "DEF_ACT idf"
	 *
	 * IPS donne l'action IPS_ACT_DEF a faire sur idf
	 * En retour : REP nb_bloc taille_bloc taille_totale ord_suiv
	*/
	case 100 : {

	    char idf[64];

	    sscanf (tp_mRcv+motd, " %s", idf);
	    tp_Buf1Size = 0;
	    if (tel_oper (IPS_ACT_DEF, idf, &tp_r ) == IPS_ACT_Naction)
		printf ("ERROR in tel_oper : no DEF section for idf = \"%s\"\n", idf);

	    tp_Adr1 = tp_Buf1; /* Le SEND:52 travaille avec tp_Adr1 */
	    tp_ord_suiv = ordre+5;
	    sprintf (tp_mSnd, "%d %d %d %d", 49, 
		(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ------------------------------------------------- CHO_ACT / DIM_ACT / FIG_ACT
	 * "CHO_ACT idf is_init  nb_cho  num val  num val  ..."
	 * IPS donne l'action IPS_ACT_CHOICE a faire sur idf
	 * il dit (is_init) si c'est un appel d'init. a l'ouverture de la parbox
	 * ou un <default> ou un <cancel> ;
	 * il dit combien (nb_cho) d'appels a CO_BEGIN_CHOICE dans tel_oper
	 * il faudra faire pour chaque couple (num, val).
	 *
	 * "DIM_ACT idf"
	 * IPS donne l'action DIM_EXE_ACT a faire sur idf
	 *
	 * "FIG_ACT idf"
	 * IPS donne l'action FIG_ACT a faire sur idf
	 *
	 * En retour demande d'encodage : ENCO
	*/
	case 200 : /* CHO_ACT */
	case 300 : /* DIM_ACT */
	case 400 : /* FIG_ACT */
	{
	    /* on conserve "idf .." pour CHO_GO / DIM_GO / FIG_GO */
	    strcpy (tp_Buf2, tp_mRcv+motd);

	    /* On dit d'encoder les params */
	    sprintf (tp_mSnd, "%d", ordre+3);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ---------------------------------------------------------------------- CHO_GO
	 * "CHO_GO"
	 *
	 * IPS dit que tout est envoye, on peut reconstruire les params,
	 * puis appeler la section CHOICE, puis encoder le resultat.
	 * En retour : REP2 nb_bloc taille_bloc taille_totale ord_suiv
	*/
	case 208 : {

	    int i, d, n;
	    char *c, idf[64];

	    /* decodage des params */
	    tp_decode_cho ();

	    /* appels (multiples) de section CHOICE
	     * Dans CHO_ACT, on avait conserve' "idf is_init nb_cho num val .." 
	     * dans tp_Buf2
	    */
	    sscanf (tp_Buf2, " %s %d %d%n", idf, &tp_r.chce_ini, &n, &d); 
	    c = tp_Buf2 + d;
	    for (i = 0; i < n; i++) {
		sscanf (c, " %d %d%n", &tp_r.chce_num, &tp_r.chce_val, &d); c += d;
		if (tel_oper (IPS_ACT_CHOICE, idf, &tp_r) == IPS_ACT_Naction) break;
	    }

	    /* re-encodage des params + grise', puis free params */
	    tp_encode_cho ();
	    tp_free_cho ();

	    /* on dit combien de blocs on va envoyer */
	    tp_Adr1 = tp_Buf1; /* Le SEND:52 travaille avec tp_Adr1 */
	    tp_ord_suiv = ordre+5;
	    sprintf (tp_mSnd, "%d %d %d %d", 49,
		(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* -------------------------------------------------------- DIM_DECO1 / FIG_DECO
	 * "DIM_DECO1" : IPS dit que tout est envoye, on peut reconstruire les params.
	 * En retour demande d'encodage : ENCO2
	 *
	 * "FIG_DECO" : IPS dit que tout est envoye, on peut reconstruire les params.
	 * En retour demande l'envoi des evenements : EVT
	*/
	case 308 :
	case 408 : {

	    /* decodage des params */
	    tp_decode_cho ();

	    /* On passe a la suite */
	    sprintf (tp_mSnd, "%d", ordre+3);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ---------------------------------------------------------------------- DIM_GO
	 * "DIM_GO"
	 *
	 * IPS dit que tout est envoye, on peut reconstruire les infos images pour DIM
	 * puis appeler la section CO_DIM, puis encoder le resultat.
	 * En retour boucle REP
	*/
	case 316 : {

	    char idf[64];

	    /* decodage des infos images */
	    tp_decode_dim ();

	    /* appel de section DIM */
	    sscanf (tp_Buf2, " %s", idf);
	    tel_oper (IPS_ACT_DIM, idf, &tp_r);

	    /* re-encodage des infos */
	    tp_encode_dim ();

	    /* on dit combien de blocs on va envoyer */
	    tp_Adr1 = tp_Buf1; /* Le SEND:52 travaille avec tp_Adr1 */
	    tp_ord_suiv = ordre+5;
	    sprintf (tp_mSnd, "%d %d %d %d", 49,
		(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* --------------------------------------------------------------------- EXE_REP
	 * "EXE_REP nb_bloc taille_bloc taille_totale"
	 *
	 * IPS donne infos buffer et demande creation buffer data (image)
	 * En retour demande du premier bloc
	*/
	case 330 : {

	    /* lecture parametres buffer */
	    sscanf (tp_mRcv+motd, " %d %d %d",
		&tp_Buf1_nb, &tp_Buf1_tb, &tp_Buf1Size);

	    /* Creation buffer ; BLOC:66 travaille avec tp_Adr1 */
	    (char *) tp_r.pix[tp_num_data] = tp_Adr1 = (char *) malloc (tp_Buf1Size);

	    /* Taux de transfert : ajoute Bufsize et depart chrono */
	    tp_time_size += tp_Buf1Size;
	    tp_time_rate = tp_gettime() - tp_time_rate;

	    if (tp_Adr1 != NULL)
		 sprintf (tp_mSnd, "63 0"); /* SEND */
	    else sprintf (tp_mSnd, "371 - Malloc error in op");  /* INFOS */
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ---------------------------------------------------------- EXE_FOR / EXE_NEXT
	 * "EXE_FOR"
	 * IPS dit de mettre d'abord le compteur a 0, puis de faire "EXE_NEXT".
	 *
	 * "EXE_NEXT"
	 * IPS dit de demander l'image suivante, ou de passer a READY.
	 * En retour "ASK num IPS_IMAGE" ou "READY"
	*/
	case 324 : /* EXE_FOR */
	{
	    tp_num_data = 0;

	    /* Taux de transfert : init */
	    tp_time_size = 0;
	    tp_time_rate = tp_gettime();
	}
	case 336 : {

	    /* Taux de transfert : stop chrono et cumul temps */
	    tp_time_rate = tp_gettime() - tp_time_rate;

	    if (++tp_num_data <= tp_r.ima_n)
		 sprintf (tp_mSnd, "327 %d %d", tp_num_data, IPS_IMAGE); /* ASK */
	    else sprintf (tp_mSnd, "339"); /* READY */
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;


	/* ---------------------------------------------------------------------- EXE_GO
	 * "EXE_GO"
	 *
	 * IPS dit d'executer la section EXE.
	 * En retour "FOR"
	*/
	case 342 : {
	    char idf[64];

	    /* Adaptation du format des short */
	    tp_short_adapt (tp_short_pareil);

	    /* preparation de tp_r : cmin, xytot, .. (a virer dans l'avenir) */
	    tp_prepare_tp_r ();

	    /* - Appel de section EXE, temps td3-td2
	     * - Durant l'execution de l'operateur, appel possible de tp_spy_start
	     *   qui lance : tp_timeout (F_SPY);
	     *   Il est desactive' par le prochain tp_timeout.
	    */
	    sscanf (tp_Buf2, " %s", idf);
	    tp_time_exec = tp_gettime ();
	    tel_oper (IPS_ACT_EXE, idf, &tp_r);
	    tp_time_exec = tp_gettime() - tp_time_exec;

	    /* Adaptation du format des short */
	    tp_short_adapt (tp_short_pareil);

	    /* On dit a IPS de demander les resultats */
	    sprintf (tp_mSnd, "%d", ordre+3); /* FOR */
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* --------------------------------------------------------------------- EXE_ASK
	 * "EXE_ASK num IPS_IMAGE taille_tot"
	 *
	 * IPS demande la data de numero num, de nature IPS_IMAGE, de taille taille_tot
	 * En retour : REP nb_bloc taille_bloc taille_tot
	 */
	case 348 : {

	    int num, nat;

	    sscanf (tp_mRcv+motd, " %d %d %d", &num, &nat, &tp_Buf1Size);

	    switch (nat) {

		case IPS_IMAGE : {
		    tp_Adr1 = (char *) tp_r.pix[num];
		    sprintf (tp_mSnd, "%d %d %d %d", ordre+3,
			(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
		} break;
		default : 
#		    ifdef DEBUG
		    printf ("ERREUR : type de data non prevu\n");
#		    endif
		    sprintf (tp_mSnd, "22"); /* WAIT */
	    }

	    tp_ord_suiv = ordre+9;
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;


	/* ------------------------------------------------------------------- EXE_READY
	 * "EXE_READY"
	 *
	 * IPS demande de liberer datas, et reencoder params
	 * En retour : REP nb_bloc taille_bloc taille_totale ord_suiv
	 */
	case 360 : {

	    int i;

	    /* Liberer datas */
	    for (i = 1; i <= tp_r.ima_n; i++) free (tp_r.pix[i]);

	    /* re-encodage des params + grise', puis free params */
	    tp_encode_cho ();
	    tp_free_cho ();

	    /* on dit combien de blocs on va envoyer */
	    tp_Adr1 = tp_Buf1; /* Le SEND:52 travaille avec tp_Adr1 */
	    tp_ord_suiv = ordre+5;
	    sprintf (tp_mSnd, "%d %d %d %d", 49,
		(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;


	/* ------------------------------------------------------------------- EXE_STAT
	 * "EXE_STAT"
	 *
	 * IPS demande infos a mettre dans footer : le temps d'execution, 
	 * le taux de transfert, etc
	 * En retour : INFOS infos
	 */
	case 368 : {

	    sprintf (tp_mSnd, "%d    Time = %.3f s    Rate = %.1f k/s", 
		ordre+3, tp_time_exec,
		tp_time_size / (1024 * tp_time_rate) );
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;


	/* -------------------------------------------------------- FIG_GO / FIG_NEXT
	 * "FIG_GO" ou "FIG_NEXT"
	 *
	 * IPS envoie une ligne d'evenements.
	 * On les traite, on appelle la section FIG, on recoit des dessins a faire.
	 * En retour : REP nb_bloc taille_bloc taille_totale ord_suiv
	 */
	case 414 :

	    /* Init buffer dessins : appele' la premiere fois */
	    tp_fig_init_draw ();

	case 422 : {
	    char idf[64];

	    /* retrouve idf */
	    sscanf (tp_Buf2, " %s", idf);

	    /* Re-init buffer dessins */
	    tp_BufDrawP = tp_BufDraw;

	    /* Decodage params de la ligne et appels multiples de la section FIG :
	     *     tel_oper (IPS_ACT_FIG, idf, &tp_r);
	     * Dans cette section FIG, des tp_fig_draw_() sont appeles, qui memorisent
	     * une liste de dessins a faire.
	    */
	    tp_appels_fig (idf, tp_mRcv+motd);

	    /* on va envoyer a IPS les dessins a faire ; on rajoute un "0" final */
	    tp_wr_int (&tp_BufDrawP, 0);

	    /* Le SEND:52 travaille avec tp_Adr1, tp_Buf1Size, tp_ord_suiv */
	    tp_Adr1     = tp_BufDraw; 
	    tp_Buf1Size = tp_BufDrawP - tp_BufDraw + 16; /* +16 par securite */
	    tp_ord_suiv = 419;

	    sprintf (tp_mSnd, "%d %d %d %d", 49,
		(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);

	} break;


	/* ------------------------------------------------------------------- FIG_READY
	 * "FIG_READY"
	 *
	 * Libere datas dessins, reencode params, libere params.
	 * En retour : REP nb_bloc taille_bloc taille_totale ord_suiv
	*/
	case 424 : {

	    /* Libere buffer dessins */
	    tp_fig_free_draw ();

	    /* re-encodage des params + grise', puis free params */
	    tp_encode_cho ();
	    tp_free_cho ();

	    /* on dit combien de blocs on va envoyer */
	    tp_Adr1 = tp_Buf1; /* Le SEND:52 travaille avec tp_Adr1 */
	    tp_ord_suiv = ordre+5;
	    sprintf (tp_mSnd, "%d %d %d %d", 49,
		(tp_Buf1Size-1)/TP_GRAIN+1, TP_GRAIN, tp_Buf1Size);
	    tp_sendto_str ();
	    tp_timeout (F_ACQUIT);
	} break;

	/* ------------------------------------------------------------------------- 
	 * Erreur
	*/
	default : { 
#	    ifdef DEBUG
	    printf ("tel : Ordre %d inconnu !\n", ordre); 
#	    endif
	    return;
	}

	} /* switch #1 */

    } while (1);
}


/***************************************************************************************
 * Fonction principale
 *
 ***************************************************************************************/
main (argc,argv)
int argc;
char *argv[];
{

    if (tp_decode_params (argc, argv, tp_params, TP_TOT_PARAMS) < 1) {

	printf ("\nThis is an IPS tel_oper, using IPS_SOCKET protocol V %s\n",
	    TP_NUM_PROTOCOL);
	printf ("Copyright 1995 by Edouard Thiel - All Rights Reserved\n");
	printf ("(Edouard.Thiel@lim.univ-mrs.fr)\n\n"); 
	printf ("To get help        -help\n");
	printf ("To run from ips    -ips num_port num_context\n");
	printf ("On remote host     -host name\n");
	exit (0);
    }

    if (TP_IS_HELP) {
	printf ("\nNo help available.\n\n");
    }

    if (TP_IS_IPS) {

	if (TP_IS_HOST)
	     tp_init (
		atoi(argv[tp_params[1].pos+1]),
		atoi(argv[tp_params[1].pos+2]),
		argv[tp_params[2].pos+1] );
	else tp_init (
		atoi(argv[tp_params[1].pos+1]),
		atoi(argv[tp_params[1].pos+2]),
		"" );

	tp_dialogue ();
	tp_quit();

    }
}

